/*****************************************************************************/
/*                                                                           */
/*                        AutocorrelationCalc()                              */
/*                                                                           */
/*      Function to calc autocorrelation function of phase or freq data      */
/*      Ref: Percival & Walden Eqn (5)                                       */
/*		Mean value is removed from input data                                */
/*      Input data can have gaps but they will be removed                    */
/*		Analysis limits are used                                             */
/*		Output array for plotting does not have analysis limit header        */
/*		It must be preallocated to # analysis points                         */
/*		By setting bMethod and bZero to 0, one can calc just lag 1           */
/*                                                                           */
/*      Parameters:                                                          */
/*          F_TYPE fZ[]  = phase or freq time domain data input              */
/*          float a[]	 = autocorrelation results (preallocated array)      */
/*			int nLag     = # lags to perform calc for <= # analysis points   */
/*          BOOL bType   = input data type (0=phase, 1=freq)                 */
/*			BOOL bMethod = calc type (0=direct, 1=FFT)                       */
/*			BOOL bZero   = lag zero flag for direct method (0=skip, 1=do)    */
/*                                                                           */
/*      Return int      = # autocorrelation points, or -1 if error           */
/*                                                                           */
/*      Revision record:                                                     */
/*          08/04/03    Created                                              */
/*			08/10/03	Added separate parameter for # lags                  */
/*			08/11/03	Added faster FFT method                              */
/*			08/12/03	Added bMethod                                        */
/*			08/19/03	Added nAF                                            */
/*			08/19/03	Added bZero                                          */
/*			01/07/04	Summation upper limit corrected                      */
/*			11/12/06	Adapted for DLL from Stable_r.c						 */
/*						Eliminate calls to AllocErrorMessage()               */
/*                                                                           */
/* (c) Copyright 2003-6 Hamilton Technical Services  All Rights Reserved     */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

// NR function declarations
void arealft(float data[], int n, int isign);
void atwofft(float data1[], float data2[], float fft1[], float fft2[],
	int n);
void afour1(float data[], int nn, int isign);

int AutocorrelationCalc(F_TYPE fZ[],
	float a[], int nLags, int nAF, BOOL bType, BOOL bMethod, BOOL bZero)
{
    // Local variables
	int i;                          // Array index
    int n;                          // # analysis points
	int s;							// Analysis start point
	int l;							// Start lag for direct method
	float v;						// Variance of input data
	F_TYPE fMean;					// Mean value of input data
	F_TYPE fStdDev;					// Std dev of input data
	int k;							// Lag index

	// NR variables
	int n2;
	unsigned no2;
	float dum;
	float *fft;
	float *d;

	// Average data
	if(bType==FREQ_DATA)
	{
		AverageFreqData(fZ, nAF);
	}
	else // Phase data
	{
		AveragePhaseData(fZ, nAF);
	}

	// Find # analysis points
	n=(int)(fZ[2]-fZ[1]+1);

	// Find analysis start
	s=(int)fZ[1];

	// Find mean
    CalcMean(fZ, &fMean, bType);

	// Remove mean
	{
		// Normalize data
		NormalizeData(fZ, fMean, bType);
	}

	// Find variance
	// Note:  This mathematical variance applies to both phase & freq data
	CalcFreqStdDev(fZ, &fStdDev);
	v=(float)(fStdDev*fStdDev);

    // Make sure there are no gaps in the analysis data
    FillGaps(fZ, bType);

	// Calculate autocorrelation
	if(!bMethod) // Direct calc
	{
		// Set start lag
		if(bZero)
		{
			l=0;
		}
		else
		{
			l=1;
		}

		// Direct aalculate autocorrelation
		// Lag k goes from lag 0 to n-1
		for(k=l; k<nLags; k++)
		{
			// Initialize sum
			a[k]=0.0;

			// Perform summation
			// Array index i goes from 1 to n-k with allowance made for
			// zero array base, analysis start and array offset
			for(i=ARRAY_OFFSET+s-1; i<ARRAY_OFFSET+s+n-k-1; i++)
			{
				a[k]+=(float)(fZ[i]*fZ[i+k]);
			}

			// Scale sum
			a[k]/=(n*v);
		}
	}
	else // Alternative (faster) FFT calc
	{
		// Uses NR twofft(), realft(), four1()
		// twofft() can be replaced with realft()
		// data1[] and data2[] same for autocorrelation, of length n
		// They must be zero-padded to an integer power of 2
		// ans[] must be of length 2n
		// NR arrays are 1 based

		// Determine size of zero-padded integer power of 2 data array n2
		n2=(int)pow(2.0, (double)ceil(log(n)/log(2)));

		// Allocate working data array d[] inc +1 for 1 base
		if((d=(float*)malloc((n2+1)*sizeof(float)))==NULL)
		{
			// Error return
			return -1; // Error code
		}
		
		// Copy fZ into d[] zero-padded & 1-based w/o header
		// Copy data
		for(i=0; i<n; i++)
		{
			d[i+1]=(float)fZ[i+ARRAY_OFFSET];
		}
		// Do zero-padding
		for(i=n; i<n2; i++)
		{
			d[i+1]=0.0;
		}

		// Note: From here on use n2 not n as array size
		
		// Allocate FFT working data array fft[]
		if((fft=(float*)malloc(((n2<<1)+1)*sizeof(float)))==NULL)
		{
			// Clean up
			free(d);

			// Error return
			return -1; // Error code
		}

		// Revised NR code
		atwofft(d, d, fft, a, n2);

		no2 = n2>>1;

		for(i=2; i<=n2+2; i+=2)
		{
			a[i-1] = (fft[i-1]*(dum=a[i-1])+fft[i]*a[i])/no2;
			a[i] = (fft[i]*dum-fft[i-1]*a[i])/no2;
		}

		a[2]=a[n2+1];
		arealft(a, n2, -1);

		// At this point we have to reconcile the 1-based, double size a[]
		// with the usage of the calling function
		// The results also need to be divided by the variance v * n

		// Scale results and convert them to 0 based array
		for(i=0; i<n2; i++)
		{
			a[i]=a[i+1]/(v*n);
		}
	
		// Autocorrelation results are in 1st n points of a[]
		// in "wraparound order" - just use 1st n/2 points

		// Free working arrays
		free(d);
		free(fft);
	}

	// Done
	return n;
}

// Local NR subroutines revised with 'a' predended to their names

void arealft(float data[], int n, int isign)
{
	int i,i1,i2,i3,i4,np3;
	float c1=0.5,c2,h1r,h1i,h2r,h2i;
	double wr,wi,wpr,wpi,wtemp,theta;

	theta=3.141592653589793/(double) (n>>1);
	if (isign == 1) {
		c2 = -0.5;
		afour1(data,n>>1,1);
	} else {
		c2=0.5;
		theta = -theta;
	}
	wtemp=sin(0.5*theta);
	wpr = -2.0*wtemp*wtemp;
	wpi=sin(theta);
	wr=1.0+wpr;
	wi=wpi;
	np3=n+3;
	for (i=2;i<=(n>>2);i++) {
		i4=1+(i3=np3-(i2=1+(i1=i+i-1)));
		h1r=c1*(data[i1]+data[i3]);
		h1i=c1*(data[i2]-data[i4]);
		h2r = -c2*(data[i2]+data[i4]);
		h2i=c2*(data[i1]-data[i3]);
		data[i1]=(float)(h1r+wr*h2r-wi*h2i);
		data[i2]=(float)(h1i+wr*h2i+wi*h2r);
		data[i3]=(float)(h1r-wr*h2r+wi*h2i);
		data[i4]=(float)(-h1i+wr*h2i+wi*h2r);
		wr=(wtemp=wr)*wpr-wi*wpi+wr;
		wi=wi*wpr+wtemp*wpi+wi;
	}
	if (isign == 1) {
		data[1] = (h1r=data[1])+data[2];
		data[2] = h1r-data[2];
	} else {
		data[1]=c1*((h1r=data[1])+data[2]);
		data[2]=c1*(h1r-data[2]);
		afour1(data,n>>1,-1);
	}
}

void atwofft(float data1[], float data2[], float fft1[], float fft2[],
	int n)
{
	int nn3,nn2,jj,j;
	float rep,rem,aip,aim;

	nn3=1+(nn2=2+n+n);
	for (j=1,jj=2;j<=n;j++,jj+=2) {
		fft1[jj-1]=data1[j];
		fft1[jj]=data2[j];
	}
	afour1(fft1,n,1);
	fft2[1]=fft1[2];
	fft1[2]=fft2[2]=0.0;
	for (j=3;j<=n+1;j+=2) {
		rep=(float)(0.5*(fft1[j]+fft1[nn2-j]));
		rem=(float)(0.5*(fft1[j]-fft1[nn2-j]));
		aip=(float)(0.5*(fft1[j+1]+fft1[nn3-j]));
		aim=(float)(0.5*(fft1[j+1]-fft1[nn3-j]));
		fft1[j]=rep;
		fft1[j+1]=aim;
		fft1[nn2-j]=rep;
		fft1[nn3-j] = -aim;
		fft2[j]=aip;
		fft2[j+1] = -rem;
		fft2[nn2-j]=aip;
		fft2[nn3-j]=rem;
	}
}

#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr

void afour1(float data[], int nn, int isign)
{
	unsigned long n,mmax,m,j,istep,i;
	double wtemp,wr,wpr,wpi,wi,theta;
	float tempr,tempi;

	n=nn << 1;
	j=1;
	for (i=1;i<n;i+=2) {
		if (j > i) {
			SWAP(data[j],data[i]);
			SWAP(data[j+1],data[i+1]);
		}
		m=nn;
		while (m >= 2 && j > m) {
			j -= m;
			m >>= 1;
		}
		j += m;
	}
	mmax=2;
	while (n > mmax) {
		istep=mmax << 1;
		theta=isign*(6.28318530717959/mmax);
		wtemp=sin(0.5*theta);
		wpr = -2.0*wtemp*wtemp;
		wpi=sin(theta);
		wr=1.0;
		wi=0.0;
		for (m=1;m<mmax;m+=2) {
			for (i=m;i<=n;i+=istep) {
				j=i+mmax;
				tempr=(float)(wr*data[j]-wi*data[j+1]);
				tempi=(float)(wr*data[j+1]+wi*data[j]);
				data[j]=data[i]-tempr;
				data[j+1]=data[i+1]-tempi;
				data[i] += tempr;
				data[i+1] += tempi;
			}
			wr=(wtemp=wr)*wpr-wi*wpi+wr;
			wi=wi*wpr+wtemp*wpi+wi;
		}
		mmax=istep;
	}
}
#undef SWAP
