/*****************************************************************************/
/*                                                                           */
/*                               ACFNoiseID()                                */
/*                                                                           */
/*      Function to estimate alpha of power law noise process using          */
/*		Greenhall lag 1 autocorrelation function                             */
/*                                                                           */
/*		Can be used with phase or frequency data                             */
/*      Data is restored to original state                                   */
/*		Round fAlpha result to get estimated integer noise type              */
/*		No parameter validity checking                                       */
/*		Requires a minimum of about 30 data points for good results          */
/*		(10 for useful results)                                              */
/*                                                                           */
/*      Parameters:                                                          */
/*          F_TYPE fData[]  = Phase or freq data array                       */
/*			int nAF         = Averaging factor                               */
/*          float fR1       = Lag 1 autocorrelation of input data            */
/*			float fAlpha    = Estimated alpha as decimal value               */
/*			int nAlpha      = Estimated alpha as rounded integer value       */
/*			int nDmax		= Maximum differencing allowed                   */
/*							  (2 for Allan, 3 for Hadamard deviation)        */
/*			int nDmin		= Initial differencing order                     */
/*                                                                           */
/*      Return int          = d(0 thru 3)=success, -1=error                  */
/*                                                                           */
/*      Note:  The nDmin parameter is normally set to 0.  It can be used     */
/*		to difference the data up-front by a fixed amount sufficient to      */
/*		blueshift the noise so that further differencing isn't necessary.    */
/*		This will avoid jumps in the estimated alpha caused by increments    */
/*      in d, but has the disadvantage of making the alpha estimates noisier */
/*		if d is larger that necessary.                                       */
/*                                                                           */
/*      Reference: W.J. Riley and C.A. Greenhall, "Power Law Noise Identi-   */
/*      fication Using the Lag 1 Autocorrelation", 18th EFTF, 3-7 Apr 2004.  */
/*                                                                           */
/*      Revision record:                                                     */
/*          09/09/03    Created                                              */
/*			09/10/03	Added nAlpha argument                                */
/*			01/28/04	Added nDmax argument and return of d                 */
/*			01/30/04	Added nDmin argument                                 */
/*			02/03/04	Improved dmin logic                                  */
/*			11/12/06	Adapted for DLL from Stable_r.c						 */
/*						Changed 1st arg from fZ to fData                     */
/*						Added BUFFER_SIZE and EXTRA_MEM macros               */
/*						Added new 2nd arg BOOL bDataType                     */
/*                                                                           */
/* (c) Copyright 2003-6 Hamilton Technical Services  All Rights Reserved     */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int ACFNoiseID(F_TYPE fData[],
	BOOL bDataType, int nAF, float *fR1, float *fAlpha, int *nAlpha,
	int nDmax, int nDmin)
{
    // Local macros
	#define DELTAMAX 0.25			// Differencing threshold
	#define BUFFER_SIZE 128			// Text buffer array size
	#define EXTRA_MEM 0				// Extra memory allocation (set to zero)

	// Local variables
	char szBuffer[BUFFER_SIZE+1]; 	// Text buffer
	F_TYPE *fSave;					// Pointer to saved data array
	float fOne[2];					// Lag 1 only autocorrelation array
	float fDelta;					// Greenhall delta
	int i;							// General index
	int nNum;						// # points in original data array
	int nN;							// # points in working data array
	int nD;							// Greenhall d = difference order
									// (d = 0 thru nDmax)

	// Get size of data array
	nNum=(int)fData[0];
	
	// Allocate memory to save original data
	if((fSave=(F_TYPE*)malloc((nNum+ARRAY_OFFSET+EXTRA_MEM)*
		sizeof(F_TYPE)))==NULL)
	{
		// Return error code
		return -1;
	}
	else // Save original data
	{
		// Save data
		for(i=0; i<nNum+ARRAY_OFFSET; i++)
		{
			fSave[i]=fData[i];
		}
	}

	// Find noise type using Greenhall lag 1 ACF method
	// Calc lag 1 autocorrelation r1
	// Find delta = r1/(1+r1)
	// If delta < 0.25, find noise type p
	// Else find 1st differences of data and repeat

	// Initialize difference order (Greenhall d)
	nD=0;

	// Set working size
	nN=nNum;

	// Do initial autocorrelation calc to average data, remove mean & fill gaps
	// Calc autocorrelation for undifferenced data
	AutocorrelationCalc(fData, fOne, 2, nAF, bDataType, 0, 0);

	// Assign r1 result
	*fR1=fOne[1];

	// For starters, just calc delta and p
	// for basic undifferenced autocorrelation (d=0)
	fDelta=(float)(fOne[1]/(1.0+fOne[1]));

	// Check if noise stationary (fDelta<0.25)
	// Stationary, noise ID can be found below
	// Nonstationary noise must be differenced until it is
	while(nD<nDmin || (fDelta>=DELTAMAX && nD<nDmax))
	{
		// Nonstationary, must apply increment operator
		// Note:  fData has been averaged at this point
		// and any gaps have been filled
		// Take 1st differences of the working data
		for(i=0; i<nN-1; i++)
		{
			fData[i+ARRAY_OFFSET]=
				fData[i+ARRAY_OFFSET+1]-fData[i+ARRAY_OFFSET];
		}

		// Adjust # and order
		fData[0]-=1.0;
		fData[2]-=1.0;
		nN-=1;
		nD+=1;

		// Find the lag 1 autocorrelation by direct method
		// The AutocorrelationCalc() function takes care
		// of averaging, filling gaps & removing mean
		// Note:  The autocorrelation results array only
		// has one (lag 1) point = fOne[1]
		// Calc autocorrelation
		AutocorrelationCalc(fData, fOne, 2, 1, bDataType, 0, 0);

		// Calc delta
		fDelta=(float)(fOne[1]/(1.0+fOne[1]));
	}

	// Assign alpha result as decimal value
	// Note: Add 2 to estimated alpha for phase data
	*fAlpha=-2*fDelta-2*nD+(1-bDataType)*2;

	// Calc the estimated noise type as rounded integer value
	sprintf(szBuffer, "%0.0f", *fAlpha);
	*nAlpha=atoi(szBuffer);

	#if(0) // For testing
	TRACE_EXP(*fR1);
	TRACE_INT(nD);
	TRACE_EXP(fOne[1]);
	TRACE_EXP(fDelta);
	TRACE_EXP(*fAlpha);
	#endif

	// Restore original data
	for(i=0; i<nNum+ARRAY_OFFSET; i++)
	{
		fData[i]=fSave[i];
	}
						
	// Free saved data memory
	free(fSave);

	// Done - return d (success)
	return nD;
}
