/***************************************************************************/
/*                                                                         */
/*                              CalcBias1()                                */
/*                                                                         */
/*      Function to calculate bias function B1(N, r, ) = ratio of         */
/*      standard to Allan variance as a function of N = # samples,  r =    */
/*      T/ = dead time ratio and  = power law noise exponent of y().   */
/*	    r = 1 for no dead time.  B1 = 1 for N = 2.                         */
/*                                                                         */
/*  	References:    (1) NBS Technical Note 375, J.A. Barnes, "Tables    */
/*						   of Bias Functions B1 and B2, for Variances      */
/*                         Based on Finite Samples of Processes with       */
/*                         Power Law Spectral Densities", January, 1969.   */
/*					   (2) NIST Technical Note 1337, TN-296 to TN-335,     */
/*						   J.A. Barnes and D.W. Allan, "Variances Based    */
/*						   on Data with Dead Time Between the              */
/*                         Measurements".                                  */
/*    	        	   (3) J.A. Barnes, "The Analysis of Time and          */
/*						   Frequency Data", Austron, Inc, December, 1991.  */
/*                                                                         */
/*		Note: 		   Faster direct solution CalcBias() function          */
/*					   available for r=1 zero dead time, int  case.       */
/*                                                                         */
/*  	Parameters:    int num = # samples (tested for 4 to 1024)          */
/*				  	             OK for 2 to INT_MAX, large # takes longer */
/*					   float r = dead time ratio (tested for.001 to 8192)  */
/*                	             Bounded to 1e-4 < r < 1e6 to avoid O/F    */
/*					   float mu= noise type (tested for -2 to +2)          */
/*                			     Get -1 error code for mu < -2 or mu > 2   */
/*                                                                         */
/*		Return:        float   = B1 bias function value,                   */
/*                               or -1 if error                            */
/*                                                                         */
/*		Revision record:                                                   */
/*			12/22/91   Drafted & debugged                                  */
/*			12/23/91   Added more bounds traps                             */
/*			12/24/91   Added traps for special cases                       */
/*			12/31/91   Renamed                                             */
/*			01/05/92   More error trapping                                 */
/*          01/16/92   Edited per FrequenC documentation                   */
/*			01/18/92   Edited title block                                  */
/*			02/03/96   Modified for use as Win 3.1 DLL                     */
/*			01/01/98   Changes for MS VC++ compatibility & warnings        */
/*                                                                         */
/* (c) Copyright 1991-9  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

float CalcBias1(int num, float r, float mu)
{
	double f;                                                    /* term f */
	double p;                                              /* product term */
	double s;                                            /* summation term */
	int i;                                              /* summation index */

	if(num<=1)                                      /* NG if # samples < 2 */
	{
		return(-1.0);                                        /* error code */
	}
	r=(float)fabs(r);                              /* avoid negative ratio */

	if((mu<-2)||(mu>2))                               /* avoid <-2 or >2 */
	{
		return(-1.0);                                        /* error code */
	}                                                            /* end if */

	if(num==2)                                   /* check for special case */
	{
		return(1.0);
	}                                                            /* end if */

	if(mu==2)                                    /* check for special case */
	{
		p=num;                  /* done in two steps to avoid int overflow */
		return(float)((p*(num+1))/6);
	}                                                            /* end if */

	if((r==1)&&(mu==1))                          /* check for special case */
	{
		return(float)(num/2);
	}                                                            /* end if */

	if((mu==-1)&&(r>=1))                         /* check for special case */
	{
		return(1.0);
	}                                                            /* end if */

	if((mu==-2)&&(r!=0)&&(r!=1))                 /* check for special case */
	{
		return(1.0);
	}                                                            /* end if */

	if((mu==0)||(mu==-2))                           /* check for =0 or -2 */
	{
		mu+=(float)1e-6;                           /* to avoid singularity */
	}                                                            /* end if */

	if(r<.0001)                                      /* check for r near 0 */
	{
		r=(float)1e-4;                             /* to avoid singularity */
	}                                                            /* end if */

	if(r>1e6)                                    /* check for very large r */
	{
		r=1e6;                                        /* to avoid overflow */
	}                                                            /* end if */

	p=num;                                                 /* initialize p */
	p*=(num-1);                 /* done in two steps to avoid int overflow */
	s=0;                                         /* initialize sum to zero */

	for(i=1; i<num; i++)                                   /* do summation */
	{
		f=2*pow(i*r, mu+2) - pow((i*r)+1, mu+2) - pow(fabs((i*r)-1), mu+2);
		s+=(f*(num-i))/p;
	}                                                           /* end for */

	f=2*pow(r, mu+2) - pow(r+1, mu+2) - pow(fabs(r-1), mu+2);

	return(float)((1+s)/(1+(f/2)));
}                                                       /* end CalcBias1() */

/***************************************************************************/
