/*****************************************************************************/
/*                                                                           */
/*                                CalcBias2()                                */
/*                                                                           */
/*      Function to calculate the bias function B2(r, u) = ratio of          */
/*      Allan variance with dead time to Allan variance without dead time    */
/*      as a function of r and u, where r = T/tau = dead time ratio and      */
/*      u = power law noise exponent of sigma-tau.                           */
/*      r = 1 for no dead time and B2(1, u) = 1.                             */
/*                                                                           */
/*      References:    (1) NBS Technical Note 375, J.A. Barnes, "Tables      */
/*                         of Bias Functions B1 and B2 ...", Jan. 1969.      */
/*                     (2) NIST Technical Note 1337, TN-296 to TN-335,       */
/*                         J.A Barnes and D.W. Allan, "Variances Based on    */
/*                         Data with Dead Time Between the Measurements".    */
/*                     (3) J.A. Barnes, "The Analysis of Time and            */
/*                         Frequency Data", Austron, Inc, December, 1991.    */
/*                                                                           */
/*      Parameters:    float r =  dead time ratio                            */
/*                                Tested for.0001 to 4096)                   */
/*                                Bounded to 1e-4 < r < 1e6 to avoid O/F     */
/*                     float mu=  noise type (mu = -2 to +2)                 */
/*                                Get -1 error code for mu < -2 or mu > 2    */
/*                                                                           */
/*      Return:        float   =  B2 bias function value,                    */
/*                                or -1 if error                             */
/*                                                                           */
/*      Revision record:                                                     */
/*          09/21/02   Copied from CalcBias2() of FrequenC library           */
/*          09/22/02   Fixed error in F PM case                              */
/*			03/08/03   Brought into Ver 2.0 of FrequenC lib                  */
/*                                                                           */
/* (c) Copyright 1991-2003  Hamilton Technical Services  All Rights Reserved */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float CalcBias2(float r, float mu)
{
    double f;                                                      /* term f */
    double d;                                            /* denominator term */

    r=(float)fabs(r);                                /* avoid negative ratio */

    if((mu<-2)||(mu>2))                                 /* avoid <-2 or >2 */
    {
        return(-1.0);                                          /* error code */
    }                                                              /* end if */

    if(r==0)                                      /* no need to calc for r=0 */
    {
        return(0.0);
    }                                                              /* end if */

    if(r==1)                                      /* no need to calc for r=1 */
    {
        return(1.0);
    }                                                              /* end if */

    if(mu==2)                                        /* special case for =2 */
    {
        return(r*r);
    }                                                              /* end if */

    if((mu==1)&&(r>=1))                      /* special case for =2 and r1 */
    {
        return(((3*r)-1)/2);
    }                                                              /* end if */

    if((mu==-1)&&(r>=0))                    /* special case for =-1 and r0 */
    {
        if(r<=1)
        {
            return(r);
        }                                                          /* end if */
        else
        {
            return(1.0);
        }                                                        /* end else */
    }                                                              /* end if */

    if(mu==-2)                        /* special case for =-2 and r!=0 or 1 */
    {
        return((float)(2.0/3.0));
    }                                                              /* end if */

    if(mu==0)                                               /* check for =0 */
    {
        mu+=(float)1e-6;                             /* to avoid singularity */
    }                                                              /* end if */

    if(r<.0001)                                        /* check for r near 0 */
    {
        r=(float)1e-4;                               /* to avoid singularity */
    }                                                              /* end if */

    if(r>1e6)                                      /* check for very large r */
    {
        r=1e6;                                          /* to avoid overflow */
    }                                                              /* end if */


    f=2*pow(r, mu+2) - pow(r+1, mu+2) - pow(fabs(r-1), mu+2);
    d=2*(1-pow(2, mu));

    return(float)((1+(f/2))/d);
}                                                         /* end CalcBias2() */

/*****************************************************************************/

