/***************************************************************************/
/*                                                                         */
/*					     CalcBisectionDrift()                              */
/*                                                                         */
/*		Function to calculate frequency drift using the average frequency  */
/*		over the first and last halves of the data points.                 */
/*	                                                                       */
/*		Parameters: 	y[]		frequency data array                       */
/*                                                                         */
/*		Return: 		float	frequency drift                            */
/*                                                                         */
/*		Notes:  This is the best estimator of drift for the combination    */
/*				of white and random walk FM noise (see reference).         */
/*				Tau=1 is assumed.   Drift is per tau interval.             */
/*				This function calls FrequenC function CalcMean().          */
/*                                                                         */
/* 		Reference:                                                         */
/* 		M. Weiss, D. Allan, and D. Howe, "Confidence on the Second Diff-   */
/*		erence Estimation of Frequency Drift", 1992 IEEE Frequency Control */
/*		Symposium, pp.300-305, June, 1992.                                 */
/*                                                                         */
/*		Revision record:                                                   */
/*			11/01/92	Created                                            */
/*			02/03/96	Modified for use as Win 3.1 DLL                    */
/*			01/01/98	Changes for MS VC++ compatibility & warnings       */
/*                                                                         */
/*	(c) Copyright 1992-8  Hamilton Technical Services  All Rights Reserved */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"                              /* FrequenC header file */

float CalcBisectionDrift(F_TYPE y[])
{
	F_TYPE start, end, mid;         /* original analysis limits & midpoint */
	F_TYPE a1, a2;                                   /* frequency averages */
	float d;                                            /* frequency drift */

	/* save original analysis limits */
	start=y[1];
	end=y[2];

	/* find midpoint of analysis data */
	mid = ((y[2]-y[1])+1)/2;                 /* not necessarily an integer */

	/* calc avg freq over 1st half */
	y[1]=1;
	y[2]=floor(mid);           /* rounded down to the next smaller integer */
	CalcMean(y, &a1, FREQ);                 /* calc avg freq over 1st half */

	/* calc avg freq over 2nd half */
	y[1]=y[2]+1;
	y[2]=end;
	CalcMean(y, &a2, FREQ);

	/* calc drift */
	d=(float)((a2-a1)/(mid));

	/* restore analysis limit */
	y[1]=start;

	return(d);                                         /* frequency drift */
}                                             /* end CalcBisectionDrift() */

/***************************************************************************/

