/***************************************************************************/
/*                                                                         */
/*							 CalcChiSqrProb()                              */
/*                                                                         */
/*		Function to calculate the chi-square probability, the area under   */
/*      the chi-square distribution.                                       */
/*                                                                         */
/*		Reference:	   Collected Algorithms from CACM, Vol. I, #299,       */
/*					   I.D. Hill and M.C. Pike, 1965.                      */
/*	                                                                       */
/*		Parameters:	   float x	= chi-square value                         */
/*					   float df	= # of degrees of freedom                  */
/*                                                                         */
/*		Return:	       float  	= chi-square probability value,            */
/*                                or -1 if error                           */
/*                                                                         */
/*		Revision record:                                                   */
/*			12/30/91   Created                                             */
/*			12/31/91   Renamed                                             */
/*			01/18/92   Edited title block                                  */
/*			02/03/96   Modified for use as Win 3.1 DLL                     */
/*			01/01/98	Changes for MS VC++ compatibility & warnings       */
/*                                                                         */
/* (c) Copyright 1991-8  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

float CalcChiSqrProb(float x, float df)

{
	int f;
    BOOL even;
	BOOL bigx=FALSE;
	float chiprob;
	double a, c, e, y, z, s;

	if(x > 1416)        /* avoid O/F of exp(-0.5*x) < DBL_MIN  2.225e-308 */
	{
		bigx=TRUE;
	}                                                            /* end if */

	f = (int) df;
	if (x < 0.0 || f < 1.0)
	{
		return (-1.0);                                       /* error code */
	}                                                            /* end if */
	a = .5 * x;
	even = (f % 2);
	even = !even;
	if (even || (f > 2 && ! bigx))
	{
		y = exp(-a);
	}                                                            /* end if */
	if (even)
	{
		s = y;
	}                                                            /* end if */
	else
	{
		s = 2.0 * CalcNormalProb((float)-sqrt(x));
	}                                                          /* end else */
	if (f > 2)
	{
		x = (float)(0.5 * (df - 1.0));
		if (even)
		{
			z = 1.0;
		}                                                        /* end if */
		else
		{
			z = .5;
		}                                                      /* end else */
		if (bigx)
		{
			e = (even) ? 0.0 : .572364942925;
			c = log(a);
			for ( ; z <= x; z += 1.0)
			{
				e = log(z) + e;
				s = exp(c * z - a - e) + s;
			}                                                   /* end for */
			chiprob = (float)s;
		}                                                        /* end if */
		else
		{
			e = (even) ? 1.0 : .564189583548 / sqrt(a);
			c = 0.0;
			for ( ; z <= x; z += 1.0)
			{
				e = e * a / z;
				c += e;
			}                                                   /* end for */
		}                                                      /* end else */
		chiprob = (float)(c * y + s);
	}                                                            /* end if */
	else
	{
		chiprob = (float)s;
	}                                                          /* end else */

	return (chiprob);
}                                                  /* end CalcChiSqrProb() */

/***************************************************************************/
