/*****************************************************************************/
/*                                                                           */
/*							  ConvDomain()                                   */
/*                                                                           */
/*		Function to do time-frequency domain conversions                     */
/*                                                                           */
/*		Parameters:		float h	   = Noise parameter h()                    */
/*                      float *p_z = Pointer to output parameter z:          */
/*								   = Sy(f), 1/Hz    if type=1                */
/*								   = Sx(f), sec/Hz if type=2                */
/*                                 = S(f), rad/Hz if type=3                */
/*								   = (f), dBc/Hz   if type=4                */
/*								   = y()          if type=5                */
/*                                 = Mod y()      if type=6                */
/*								   = x(), sec     if type=7                */
/*						float tau  = Averaging time , sec                   */
/*						float f    = Fourier frequency f, Hz                 */
/*						float v    = Carrier frequency, Hz                   */
/*						float fh   = System high frequency cut off, Hz       */
/*					    int type   = Input parameter type:                   */
/*                                   1=Sy(f), 2=Sx(f), 3=S(f), 4=(f)       */
/*							         5=y(), 6=Mod y(), 7=x()           */
/*					    int alpha  = Power law noise type,                  */
/*								   = Log=log slope of Sy(f)=h()f^          */
/*								     White PM =+2, Flicker PM =+1          */
/*								     White FM = 0, Flicker FM =-1          */
/*                                   Random Walk FM =-2                     */
/*						int n      = # samples averaged                      */
/*								     sampling time = tau_0 = tau/n           */
/*                                                                           */
/*		Return:		    int		   = Status: +1=OK, -1=Error                 */
/*                                                                           */
/*		Notes:			This function converts the noise parameter h()      */
/*						to an output parameter determined by type.           */
/*						Depending on the conversion desired, the             */
/*						parameters t, f, v, fh, alpha and n are used.        */
/*						Use the function CalcDomain() for the inverse        */
/*						conversion from one of the time or frequency         */
/*						noise parameters to h().  A general domain          */
/*						conversion is performed by using both functions.     */
/*						Calls FrequenC Library function CalcRatio().         */
/*						Original z unchanged if bad parameter (error).       */
/*                                                                           */
/*		Revision record:                                                     */
/*			05/03/97	Adapted from ConvDomain() of STABLE_A.C              */
/*			01/01/98	Changes for MS VC++ compatibility & warnings         */
/*			02/28/07	Fixed error for mod and time sigma calc              */
/*                                                                           */
/* (c) Copyright 1992-2007 Hamilton Technical Services All Rights Reserved   */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int ConvDomain(float h, float *p_z,
	float tau, float f, float v, float fh, int type, int alpha, int n)
{
	double pi;                                                          /*  */
	double a;
	double b;
	double c;
	double d;
	double e;
	double Sx;                                                      /* Sx(f) */
	double Sp;                                                      /* S(f) */
	double L;                                                        /* (f) */
	double Sy;                                                      /* Sy(f) */
	double sy;                                                      /* y() */
	double sx;                                                      /* x() */
	double mod_sy;                                              /* Mod y() */

	/* check parameters */

	if( (h==0)   || (tau<=0) || (f<=0)     || (v<=0)    || (fh<=0) ||
		(type<1) || (type>7) || (alpha<-2) || (alpha>2) || (n<1)      )
	{
		return(-1);
	}

	/* initialize constants */

	pi=4*atan(1);
	a=4*pi*pi/6;
	b=2*log(2);
	c=0.5;
	d=(1.038+(3*log(2*pi*fh*tau)))/(4*pi*pi);
	e=(3*fh)/(4*pi*pi);

	/* perform basic frequency domain conversion */

	Sy=h*pow(f, alpha);

	/* perform basic time domain conversion */

	switch(alpha)
	{
		case  2:	/* W  */
		{
			sy=sqrt(e*Sy/(f*f*tau*tau));
			break;
		}
		case  1:    /* F  */
		{
			sy=sqrt(d*Sy/(f*tau*tau));
			break;
		}
		case  0:    /* W f */
		{
				sy=sqrt(c*Sy/tau);
				break;
		}
		case -1:    /* F f */
		{
				sy=sqrt(b*f*Sy);
				break;
		}
		case -2:    /* RW f */
		{
				sy=sqrt(a*f*f*Sy*tau);
				break;
		}
		default:
		{
				return(-1);                                         /* error */
		}
	}

	/* complete specific conversion */

	switch(type)
	{
		case 1:   /* Sy(f) */
		{
			*p_z=(float)Sy;
			break;
		}
		case 2:   /* Sx(f) */
		{
			Sx=Sy/(4*pi*pi*f*f);
			*p_z=(float)Sx;
			break;
		}
		case 3:   /* S(f) */
		{
			Sp=Sy*v*v/(f*f);
			*p_z=(float)Sp;
			break;
		}
		case 4:   /* (f) */
		{
			Sp=Sy*v*v/(f*f);
			L=10*log10(Sp/2);
			*p_z=(float)L;
			break;
		}
		case 5:   /* sy() */
		{
			*p_z=(float)sy;
			break;
		}
		case 6:   /* Mod sy() */
		{
			#if(0) // This test was wrong until 02/28/07
			// It is perfectly OK for tau/n = tau0 to be less than 1 (e.g. 1e-3)
			// The 2nd arg of CalcRatio cannot be zero
			// We have already tested that both tau and n are positive and non-zero
			// So tau/n is also positive and non-zero - no further test required
			if( (tau/n)<1 )
			{
				return(-1);
			}
			#endif

			mod_sy=sqrt(sy*sy*CalcRatio(alpha, n, (float)(2*pi*fh*tau/n)));
			*p_z=(float)mod_sy;
			break;
		}
		case 7:   /* sx() */
		{
			#if(0) // This test was wrong until 02/28/07
			// It is perfectly OK for tau/n = tau0 to be less than 1 (e.g. 1e-3)
			// The 2nd arg of CalcRatio cannot be zero
			// We have already tested that both tau and n are positive and non-zero
			// So tau/n is also positive and non-zero - no further test required
			if( (tau/n)<1 )
			{
				return(-1);
			}
			#endif

			mod_sy=sqrt(sy*sy*CalcRatio(alpha, n, (float)(2*pi*fh*tau/n)));
			sx=tau*mod_sy/sqrt(3);
			*p_z=(float)sx;
			break;
		}
		default:
		{
			return(-1);                                             /* error */
		}
	}

		return(1);

}                                                        /* end ConvDomain() */

/*****************************************************************************/
