/*****************************************************************************/
/*                                                                           */
/*							  CalcDomain()                                   */
/*                                                                           */
/*		Function to calc time-frequency domain noise parameters              */
/*                                                                           */
/*		Parameters:		float *p_h = Pointer to noise parameter h()         */
/*                      float  z   = input parameter:                        */
/*								   = Sy(f), 1/Hz    if type=1                */
/*								   = Sx(f), sec/Hz if type=2                */
/*                                 = S(f), rad/Hz if type=3                */
/*								   = L(f), dBc/Hz   if type=4                */
/*								   = y()          if type=5                */
/*                                 = Mod y()      if type=6                */
/*								   = x(), sec     if type=7                */
/*						float tau  = Averaging time , sec                   */
/*						float f    = Fourier frequency f, Hz                 */
/*						float v    = Carrier frequency, Hz                   */
/*						float fh   = System high frequency cut off, Hz       */
/*					    int type   = Input parameter type:                   */
/*                                   1=Sy(f), 2=Sx(f), 3=S(f), 4=(f)       */
/*							         5=y(), 6=Mod y(), 7=x()           */
/*					    int alpha  = Power law noise type,                  */
/*								   = Log=log slope of Sy(f)=h()f^          */
/*								     White PM =+2, Flicker PM =+1          */
/*								     White FM = 0, Flicker FM =-1          */
/*                                   Random Walk FM =-2                     */
/*						int n      = # samples averaged                      */
/*								     sampling time = tau_0 = tau/n           */
/*                                                                           */
/*		Return:		    int		   = Status: +1=OK, -1=Error                 */
/*                                                                           */
/*		Notes:			This function calculates the power law noise         */
/*						parameter h() for an input parameter type.          */
/*						Depending on the conversion desired, the             */
/*						parameters t, f, v, fh, alpha and n are used.        */
/*						Use the function ConvDomain() for the inverse        */
/*						conversion from h() to one of the time or           */
/*						frequency noise parameters .  A general domain       */
/*						conversion is performed by using both functions.     */
/*						Calls FrequenC Library function CalcRatio().         */
/*						Original h unchanged if bad parameter (error).       */
/*                                                                           */
/*		Revision record:                                                     */
/*			05/03/97	Adapted from CalcDomain() of STABLE_A.C              */
/*			01/01/98	Changes for MS VC++ compatibility & warnings         */
/*                                                                           */
/* (c) Copyright 1992-8  Hamilton Technical Services  All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int CalcDomain(float *p_h, float z,
	float tau, float f,	float v, float fh, int type, int alpha, int n)
{
	double pi;                                                          /*  */
	double a;
	double b;
	double c;
	double d;
	int index;                                                 /* case index */

	/* check parameters */

	if( (z==0)   || (tau<=0) || (f<=0)     || (v<=0)    || (fh<=0) ||
		(type<1) || (type>7) || (alpha<-2) || (alpha>2) || (n<1)      )
	{
		return(-1);
	}

	/* initialize constants */

	pi=4*atan(1);
	a=4*pi*pi;
	b=2*log(2);
	c=2*pi*fh*tau;
	d=1.038+(3*log(c));

	index=((alpha+2)*7)+type;

	/* process noise data entry */

	switch(index)
	{
		case 1:     /* Sy(f) for RW f =-2 */
					{
						*p_h=z*f*f;
						break;
					}
		case 2:     /* Sx(f) for RW f =-2 */
					{
						*p_h=(float)(a*z*f*f*f*f);
						break;
					}
		case 3:     /* S(f) for RW f =-2 */
					{
						*p_h=(float)(z*f*f*f*f/(v*v));
						break;
					}
		case 4:     /* (f) for RW f =-2 */
					{
						*p_h=(float)(2*pow(10,(z/10))/(v*v));
						*p_h*=f*f*f*f;
						break;
					}
		case 5:     /* y() for RW f =-2 */
					{
						*p_h=(float)(6*z*z/(a*tau));
						break;
					}
		case 6:     /* Mod y() for RW f =-2 */
					{
						*p_h=6*z*z;
						*p_h/=CalcRatio(2, n, (float)(c/n));
						*p_h/=(float)(a*tau);
						break;
					}
		case 7:     /* x() for RW f =-2 */
					{
						*p_h=3*6*z*z;
						*p_h/=CalcRatio(2, n, (float)(c/n));
						*p_h/=(float)(a*tau*tau*tau);
						break;
					}
		case 8:     /* Sy(f) for F f =-1 */
					{
						*p_h=z*f;
						break;
					}
		case 9:     /* Sx(f) for F f =-1 */
					{
						*p_h=(float)(a*f*f*f*z);
						break;
					}
		case 10:    /* S(f) for F f =-1 */
					{
						*p_h=f*f*f*z/(v*v);
						break;
					}
		case 11:    /* (f) for F f =-1 */
					{
						*p_h=(float)(2*pow(10,(z/10))/(v*v));
						*p_h*=f*f*f;
						break;
					}
		case 12:    /* y() for F f =-1 */
					{
						*p_h=(float)(z*z/(b));
						break;
					}
		case 13:    /* Mod y() for F f =-1 */
					{
						*p_h=z*z;
						*p_h/=(float)b;
						*p_h/=CalcRatio(1, n, (float)(c/n));
						break;
					}
		case 14:    /* x() for F f =-1 */
					{
						*p_h=(float)(3*z*z/(b));
						*p_h/=CalcRatio(1, n, (float)(c/n));
						*p_h/=tau*tau;
						break;
					}
		case 15:    /* Sy(f) for W f =0 */
					{
						*p_h=z;
						break;
					}
		case 16:    /* Sx(f) for W f =0 */
					{
						*p_h=(float)(a*f*f*z);
						break;
					}
		case 17:    /* S(f) for W f =0 */
					{
						*p_h=f*f*z/(v*v);
						break;
					}
		case 18:    /* (f) for W f =0 */
					{
						*p_h=(float)(f*f*2*pow(10,(z/10))/(v*v));
						break;
					}
		case 19:    /* y() for W f =0 */
					{
						*p_h=2*z*z*tau;
						break;
					}
		case 20:    /* Mod y() for W f =0 */
					{
						*p_h=2*z*z*tau;
						*p_h/=CalcRatio(0, n, (float)(c/n));
						break;
					}
		case 21:    /* x() for W f =0 */
					{
						*p_h=3*2*z*z*tau;
						*p_h/=CalcRatio(0, n, (float)(c/n));
						*p_h/=tau*tau;
						break;
					}
		case 22:    /* Sy(f) for F  =1 */
					{
						*p_h=z/f;
						break;
					}
		case 23:    /* Sx(f) for F  =1 */
					{
						*p_h=(float)(a*f*z);
						break;
					}
		case 24:    /* S(f) for F  =1 */
					{
						*p_h=f*z/(v*v);
						break;
					}
		case 25:    /* (f) for F  =1 */
					{
						*p_h=(float)(f*2*pow(10,(z/10)));
						*p_h/=v*v;
						break;
					}
		case 26	:   /* y() for F  =1 */
					{
						*p_h=(float)(a*z*z*tau*tau);
						*p_h/=(float)d;
						break;
					}
		case 27:    /* Mod y() for F  =1 */
					{
						*p_h=(float)(a*z*z);
						*p_h/=CalcRatio(-1, n, (float)(c/n));
						*p_h*=tau*tau;
						*p_h/=(float)d;
						break;
					}
		case 28:    /* x() for F  =1 */
					{
						*p_h=(float)(3*a*z*z*tau*tau);
						*p_h/=CalcRatio(-1, n, (float)(c/n));
						*p_h/=tau*tau;
						*p_h/=(float)d;
						break;
					}
		case 29:    /* Sy(f) for W  =2 */
					{
						*p_h=z/(f*f);
						break;
					}
		case 30:    /* Sx(f) for W  =2 */
					{
						*p_h=(float)(a*z);
						break;
					}
		case 31:    /* S(f) for W  =2 */
					{
						*p_h=z/(v*v);
						break;
					}
		case 32:    /* (f) for W  =2 */
					{
						*p_h=(float)(2*pow(10,(z/10)));
						*p_h/=v*v;
						break;
					}
		case 33:    /* y() for W  =2 */
					{
						*p_h=(float)(a*z*z);
						*p_h*=tau*tau/(3*fh);
						break;
					}
		case 34:    /* Mod y() for W  =2 */
					{
						*p_h=(float)(a*z*z);
						*p_h*=tau*tau;
						*p_h/=CalcRatio(-2, n, (float)(c/n));
						*p_h/=3*fh;
						break;
					}
		case 35:    /* x() for W  =2 */
					{
						*p_h=(float)(3*a*z*z);
						*p_h*=tau*tau;
						*p_h/=CalcRatio(-2, n, (float)(c/n));
						*p_h/=3*tau*tau*fh;
						break;
					}
		default:	break;

	}

		return(1);

}                                                        /* end CalcDomain() */

/*****************************************************************************/
