/***************************************************************************/
/*                                                                         */
/*							CalcDegFree()                                  */
/*                                                                         */
/*      Function to calculate the # of chi-square degrees of freedom       */
/*      for phase or frequency data.                                       */
/*                                                                         */
/*		Reference:     NIST Technical Note 1337, p.TN-85, Table 12-4       */
/*                     (as corrected).                                     */
/*                                                                         */
/*		Parameters:    int alpha = power-law exponent of Sy(f) noise type  */
/*					   int n     = # phase data points                     */
/*					             = # frequency data points + 1             */
/*					   int m     = fully-overlapping averaging factor      */
/*                                                                         */
/*		Return:        float     = # chi-square degrees of freedom,        */
/*                                 or -1 if error                          */
/*                                                                         */
/*		Revision record:                                                   */
/* 			12/30/91   Created                                             */
/*			12/31/91   Renamed                                             */
/*			01/04/92   Changed switch from (2-a) to a                      */
/*					   Added default case for NG alpha                     */
/*					   Changed n and m parameters to ints                  */
/*					   Do calcs with double versions dn and dm             */
/*					   Added error traps                                   */
/*			01/18/92   Edited title block                                  */
/*			02/03/96	Modified for use as Win 3.1 DLL                    */
/*			01/01/98	Changes for MS VC++ compatibility & warnings       */
/*                                                                         */
/* (c) Copyright 1991-8  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

float CalcDegFree(int alpha, int n, int m)

{
	double dn;                                      /* double version of n */
	double dm;                                      /* double version of m */
	double df;                                     /* # degrees of freedom */
	double df1,df2,df3;                         /* terms of df calculation */

	if( (n==0) || (m==0) )
	{
		return(-1.0);                                        /* error code */
	}

	dn=(double) abs(n);                         /* make sure n is positive */
	dm=(double) abs(m);                         /* make sure m is positive */

	switch(alpha)          /* alpha is exponent of power-law noise process */
	{                                    /* slope of Sy(f) on log-log plot */
			case 2:
			{
				if(dn==dm)
				{
					return(-1.0);                            /* error code */
				}
				df=((dn+1)*(dn-(2*dm)))/(2*(dn-dm));  /* white phase noise */
				break;                                          /* alpha=2 */
			}
			case 1:
			{
				if(dn==1)
				{
					return(-1.0);                           /* error code */
				}
				df1=log((((2*dm)+1)*(dn-1))/(4));   /* flicker phase noise */
				df2=log((dn-1)/(2*dm));                        /* alpha=1) */
				if(df1*df2<0)
				{
					return(-1.0);                            /* error code */
				}
				df=exp(sqrt(df1*df2));
				break;
			}
			case 0:
			{
				df1=(4*dm*dm)/((4*dm*dm)+5);      /* white frequency noise */
				df2=(3*(dn-1))/(2*dm);                          /* alpha=0 */
				df3=(2*(dn-2))/(dn);
				df=(df2-df3)*df1;
				break;
			}
			case -1:
			{
				if(dm==1.0)                     /* flicker frequency noise */
				{                                              /* alpha=-1 */
					df=(2*(dn-2)*(dn-2))/((2.3*dn)-4.9);
					break;
				}                                                /* end if */
				else
				{
					df=(5*dn*dn)/((4*dm)*(dn+(3*dm)));
					break;
				}                                              /* end else */
			}
			case -2:
			{
				if(dn==3)
				{
					return(-1.0);                            /* error code */
				}
				df1=(dn-1)*(dn-1);          /* random-walk frequency noise */
				df2=(dn-2)/((dm)*(dn-3)*(dn-3));               /* alpha=-2 */
				df3=(3*dm)*(1-dn)+(4*dm*dm);
				df=df2*(df1+df3);
				break;
			}
			default:                                           /* NG alpha */
			{
				df=-1.0;                                     /* error code */
			}
	}                                                        /* end switch */

	return(float)df;                     /* chi-squared degrees of freedom */
}                                                     /* end CalcDegFree() */

/***************************************************************************/
