/*****************************************************************************/
/*                                                                           */
/*                       CalcDiffusionFreqDrift()                            */
/*                                                                           */
/*      Function to calculate diffusion frequency drift                      */
/*      Fit fractional frequency data to time-shifted diffusion model:       */
/*                      y(t) = a + b*sqrt(t+c)                               */
/*                      where: y(t) = fractional frequency                   */
/*                             t    = time                                   */
/*                             a    = offset term                            */
/*                             b    = sqrt time multiplier term              */
/*                             c    = time shift term                        */
/*                                                                           */
/*      Parameters:     F_TYPE y[]  = frequency data                         */
/*                                    y[0] = # data points                   */
/*                                    y[1] = analysis start                  */
/*                                    y[2] = analysis end                    */
/*                      F_TYPE *p_a  = pointer to offset term                */
/*                      F_TYPE *p_b  = pointer to sqrt time multiplier term  */
/*                      F_TYPE *p_c  = pointer to time shift term            */
/*                      F_TYPE *p_v  = pointer to fit variance               */
/*                                                                           */
/*      Return:         int          = # non-zero anlaysis points,           */
/*                                     or -1 if error                        */
/*                                                                           */
/*      Notes:          Zeros treated as gaps in frequency data.             */
/*                      A value of c is chosen, the diffusion fit parameters */
/*                      are found, and c is varied iteratively until the     */
/*                      rms error is below a limit criterion.                */
/*                      Need at least 2 non-zero data points.                */
/*                                                                           */
/*      Revision record:                                                     */
/*          06/09/97    Created from CalcLogFreqDrift() of FrequenC Lib and  */
/*                      CalcDiffusionFreqDrift() of Stable/Win               */
/*			06/10/97	Running - converges OK                               */
/*			12/09/97	Added to FrequenC Library                            */
/*			01/01/98	Changes for MS VC++ compatibility & warnings         */
/*                                                                           */
/* (c) Copyright 1997-8 Hamilton Technical Services   All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int CalcDiffusionFreqDrift(F_TYPE y[],
	F_TYPE *p_a, F_TYPE *p_b, F_TYPE *p_c, F_TYPE *p_v)
{
	int i;                                                /* freq data index */
	int num = (int)(*(y+2) - *(y+1) +1);          /* # frequency data points */

	#if(0) // Original code of Release 1.01 - Doesn't always converge
	double error_limit=0.00000001;    /* fractional error limit for variance */
	#endif

	#if(1) // Trial code 06/20/97
	// This seems better - converges for all known test cases & results OK
	double error_limit=0.000001;      /* fractional error limit for variance */
	#endif

	double err=1.0;                                  /* sum of squared error */
	double prev_err=0;    /* initialized so (err-prev_err)/err > error_limit */                                           /* ditto */
	double sum_x;                                      /* sum of x=sqrt(t+c) */
	double sum_y;                                              /* sum of y=t */
	double sum_xx;                                             /* sum of x^2 */
	double sum_xy;                                              /* sum of xy */
	double d=100.0;                                  /* c iteration factor */

	*p_c=0.0;                                         /* initial value for c */

	while(fabs(((err-prev_err)/err))>error_limit)                 /* iterate */
	{
		prev_err=err;                                 /* save previous error */
		sum_x = 0;                                     /* sum of x=sqrt(t+c) */
		sum_y = 0;                                               /* sum of y */
		sum_xx = 0;                                            /* sum of x^2 */
		sum_xy = 0;                                             /* sum of xy */
		num = (int)(*(y+2) - *(y+1) +1);          /* # frequency data points */

		for( i=(int)(*(y+1)+2); i<=(int)(*(y+2)+2); i++)  /* accumulate sums */
		{
			if(*(y+i))                                       /* gap in data? */
			{
				sum_x  += sqrt(((double)(i-2))+ *p_c);
				sum_y  += *(y+i);
				sum_xx += ((double)(i-2))+ *p_c;
				sum_xy += sqrt(((double)(i-2))+ *p_c) * *(y+i);
			}                                                      /* end if */
			else
			{
				num--;
			}                                                    /* end else */
		}                                                         /* end for */

		if(num<2)                        /* must have at least 2 data points */
		{
			return(-1);
		}                                                          /* end if */

		*p_b = sum_xy - (sum_x * sum_y / num);            /* find parameters */
		*p_b /= sum_xx- (sum_x * sum_x / num);
		*p_a = (sum_y - (*p_b * sum_x ))/ num;

		err=0;                                  /* calc sum of squared error */

		for( i=(int)(*(y+1)+2); i<=(int)(*(y+2)+2); i++)
		{
			if(*(y+i))
			{
				// error = SQR [ fit - actual = a + b*sqrt(t+c) - y(t) ]
				err += SQR(*p_a + *p_b * sqrt((double)(i-2)+ *p_c) - *(y+i));
			}
		}

		if(err>prev_err)                                       /* wrong way? */
		{
			d *= -0.1;
		}

		*p_c+= d;                             /* modify c for next iteration */

		if(*p_c<0.0)                              /* avoid sqrt domain error */
		{
			d *= -0.1;
			*p_c=0.0;
		}

	}

	*p_v = err;

	return(num);
}                                         /* end NewCalcDiffusionFreqDrift() */

/*****************************************************************************/
