/*****************************************************************************/
/*                                                                           */
/*  			       		CalcFreqHadamardDev()                            */
/*                                                                           */
/*		Function to calculate Hadamard variance from frequency data          */
/*                                                                           */
/*                   1    M-2                                                */
/*		H y() =    [y(i+2) -2y(i+1) -y(i)]                        */
/*				   6(M-2) i=1                                                */
/*                                                                           */
/*                                                                           */
/*		Parameters:		F_TYPE y[]    = frequency data                       */
/*                                      y[0] = # data points                 */
/*			                            y[1] = analysis start                */
/*                                      y[2] = analysis end                  */
/*                      float *p_dev = pointer to Hadamard deviation         */
/*                                                                           */
/*		Return:         int           = # analysis points,                   */
/*										or -1 if error                       */
/*                                                                           */
/*		Note:	        Embedded zeros treated as gaps in freq data.         */
/*						Must have at least 1 analysis point; this            */
/*                      requires at least 3 adjacent non-gap freq data       */
/*                      points.                                              */
/*	                                                                         */
/*		Reference:		S. Hutsell,	Proc. 27th PTTI Meeting, Dec. 1995.      */
/*                                                                           */
/*		Revision record:                                                     */
/*			05/03/96	Adapted from hadamardev_f() of STABLE_A.C            */
/*			01/01/98	Changes for MS VC++ compatibility & warnings         */
/*                                                                           */
/* (c) Copyright 1996-8  Hamilton Technical Services  All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int CalcFreqHadamardDev(F_TYPE y[],
	F_TYPE *p_dev)
{
	double sum;                            /* summing variable to calc sigma */
	int i;                                              /* index for summing */
	int num = 0;                                        /* # analysis points */

	for( i = (int)(*(y+1)+2), sum = 0; i < (int)(*(y+2)+1); i++)
												 /* index goes from 1 to n-2 */
					   /* for n data points from 1 to n; data starts at y[3] */
	{
		if( *(y+i) && *(y+i+1) && *(y+i+2) )
									  /* skip if any freq data value is zero */
		{
			sum += SQR( *(y+i+2) - 2 * *(y+i+1) + *(y+i) );
			num++;
		}
	}

	if(num>0)
	{
		*p_dev = ( sqrt( sum / (6.0 * num )));
		return(num);                                    /* # analysis points */
	}
	else                                               /* no analysis points */
	{
		return(-1);                                                 /* error */
	}
}

/*****************************************************************************/
