/*****************************************************************************/
/*                                                                           */
/*                              CalcFastMTIE()                               */
/*                                                                           */
/*      Function to calc MTIE (maximum time interval error) for phase data   */
/*      Parameters:                                                          */
/*          F_TYPE fX[]        = data array, where:                          */
/*                                           F_TYPE fX[0] = # data points    */
/*                                           F_TYPE fX[1] = analysis start   */
/*                                           F_TYPE fX[2] = analysis end     */
/*          plotdata *pResults = MTIE results sigmadata structure            */
/*                                                                           */
/*      Return:                                                              */
/*          k   = Outer index (# MTIE points), or -1 if error                */
/*                                                                           */
/*      sigmadata structure elements for sigma data:                         */
/*          int     points        number of sigma data points                */
/*          float   *tau          pointer to averaging time data array       */
/*          float   *num          pointer to number of data points array     */
/*          float   *sigma        pointer to sigma data array                */
/*          float   *ci           pointer to confidence interval             */
/*          float   *sig_min      pointer to lower sigma error bar array     */
/*          float   *sig_max      pointer to upper sigma error bar array     */
/*          float   *std_dev      pointer to standard deviation array        */
/*          int     *alpha        pointer to noise type exponent array       */
/*          int     *af           pointer to averaging factor                */
/*                                                                           */
/*      Note: Signature of this function is necessarily different from       */
/*      CalcMTIE() because it includes the outer averaging factor loop.      */
/*      It is used differently in main program.                              */
/*                                                                           */
/*      Basic Algorithm:                                                     */
/*          An n-point wide window is moved thru the phase data and the      */
/*          difference between the maximum and minimum phase (time) values   */
/*          is found at each window position.  MTIE is the overall maximum   */
/*          of this time interval error over the whole data set.             */
/*                                                                           */
/*      Fast Algorithm:                                                      */
/*          Calc MTIE in octave steps only.  Store max and min for each tau  */
/*          = AF*tau0 in two vectors.  For next AF, just search these        */
/*          vectors, not all data, for next maxs and mins.                   */
/*                                                                           */
/*      Notes:                                                               */
/*          (1) This function does not handle gaps.                          */
/*          (2) No abort option or progress message for this function        */
/*          (4) # time intervals is N-1, where N = # phase data points, so   */
/*              AF can go from 1 to N-1.                                     */
/*          (3) B&M algorithm uses AF sequence 1,3,7,15,...2^k -1            */
/*              where k is outer index that goes from 1 to <= log2(N)        */
/*                                                                           */
/*      Reference 1:                                                         */
/*          S. Bregni, "Clock Stability Characterization and Measurement     */
/*          in Telecommunications", IEEE Transactions on Instrumentation     */
/*          and Measurement, Vol. 46, No. 6, Dec. 1997, pp. 1284-1294,       */
/*          Eq. 14.                                                          */
/*      Reference 2:                                                         */
/*          S. Bregni and S. Maccabruni, "Fast Computation of Maximum Time   */
/*          Interval Error by Binary Decomposition", IEEE Transactions on    */
/*          Instrumentation and Measurement, Vol. 49, No. 6, Dec. 2000, pp.  */
/*          1240-1244.                                                       */
/*                                                                           */
/*      Revision record:                                                     */
/*          01/26/01    Drafted from CalcMTIE()                              */
/*          01/27/01    Running                                              */
/*          01/28/01    Added progress message                               */
/*                      Changed 2nd param to plotdata struct                 */
/*                      Changed min & max arrays from F_TYPE to float        */
/*                      Tried various speed enhancements                     */
/*                      Moved here from stable_r.c to use compiler speed     */
/*                      optimizations - they work the best                   */
/*                      Copy in stable_r.c still has debug code if needed.   */
/*          01/30/01    Added results on-the-fly & progress display          */
/*			03/22/03	Adapted for FrequenC DLL                             */
/*						Added abort flag & progress function arguments       */
/*						Not yet fully-integrated into FrequenC DLL           */
/*						Function prototype noy in frequenc.h                 */
/*          12/06/17	Added MIN and MAX macros                             */
/*                      Removed calls to ProgressFunction()                  */
/*                                                                           */
/* (c) Copyright 2001-3  Hamilton Technical Services  All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

// Struct for sigma run data
struct sigmadata                        /* structure template for sigma data */
{
    int points;                               /* number of sigma data points */
	int rows;					         /* # rows of results = # tau values */
    float *tau;                      /* pointer to averaging time data array */
    float *num;                    /* pointer to number of data points array */
    float *sigma;                             /* pointer to sigma data array */
    float *ci;         /* pointer to confidence interval from SIGMA.TAU file */
    float *sig_min;                /* pointer to lower sigma error bar array */
    float *sig_max;                /* pointer to upper sigma error bar array */
    float *std_dev;                   /* pointer to standard deviation array */
	float *df;						  /* pointer to degrees of freedom array */
    int *alpha;            /* pointer to power-law noise type exponent array */
    int *af;                            /* pointer to averaging factor array */
};

int CalcFastMTIE(F_TYPE fX[],
	struct sigmadata *pResults, BOOL *bAbort,
	void *ProgressFunction(int nPercentDone, char *szMessage))
{
    // Local variables
    // Ints
    int i;                          // Inner and general inner index
    int j;                          // Middle index
    // int last_j;                  // Previous middle index
    int k;                          // Outer index
    int p;                          // Span
    int nStart;                     // Analysis start
    int nEnd;                       // Analysis end
    int nNum;                       // # analysis points
    int nAF;                        // Averaging factor
    int nStop;                      // Max value of outer index
    // int tic;                     // Progress interval
    // Floats 
    float fMTIE;                    // Current MTIE value
    float *fSmall;                  // Min phase data value array
    float *fBig;                    // Max phase data value array

    // Initialize progress display
    // ProgressFunction(0, "Calculating MTIE");

    // Set analysis limits
    nStart=(int)fX[1];
    nEnd=(int)fX[2];
    nNum=nEnd-nStart+1;

    // Allocate working memory for max and min vectors
    // First pass they must be same size as analysis data array (w/o header)
    // Together they hold all points in overlapping big/small pairs
    // No point in reallocing as they get smaller

    // Allocate maximum array
    if((fBig=(float*)malloc((nNum)*sizeof(float)))==NULL)
    {
        return -1; // Error
    }

    // Allocate minimum array
    if((fSmall=(float*)malloc((nNum)*sizeof(float)))==NULL)
    {
        free(fBig);
        return -1; // Error
    }

    // Initialize the max and min arrays by copying the phase data into them
    for(i=0; i<nNum; i++)
    {
        // Copy data
        fBig[i]=(float)fX[nStart+ARRAY_OFFSET+i-1];
        fSmall[i]=(float)fX[nStart+ARRAY_OFFSET+i-1];
    }
    
    // Calc MTIE
    // Outer k loop over octave AFs up to max nAF = nNum-1
    // nAF is 2^k -1 and max k is log2(nNum)

    // Find maximum outer index = log2(nNum) as truncated integer
    nStop=(int)(log(nNum)/log(2));

    // Calc MITE points at octave averaging factors up to largest possible
    for(k=1; k<=nStop; k++)
    {
        // Find the current averaging factor nAF = 2^k -1
        nAF=(int)(pow(2, k))-1;

        // Find the current span p = 2^(k-1)
        p=(int)(pow(2, k-1));

        // Initialize previous middle index
        // last_j=0;

        // Update the min & max vectors for this AF
        // Loop through all nNum-AF points
        for(j=0; j<nNum-nAF; j++)
        {
            /* Set 1% interval for progress report */
            // tic=(int)ceil((nNum-nAF)/100);

            // The max and min values are already initialized               
            // Find max & min values for each step in this window of span p
            // Note: This is apparently the section of code that takes the
            // most time as nAF and k get larger, and p gets much larger
            // But its hard to see how one can do any better than the
            // max() and min() macros.
            // This code was tried with index arithmitic outside loop
            // and just incrementing inside - little difference in speed
            // Pointer arithmitic was also tried - ditto
            // What does help is to ask the complier to optimize for speed
            // That makes 5:1 improvement for any of the code versions
            for(i=1; i<=p; i++)
            {
                fBig[j]=MAX(fBig[j+i], fBig[j]);
                fSmall[j]=MIN(fSmall[j+i], fSmall[j]);
            }

            #if(0)
            // Display progress
            // Note:  This code provides updates only if index has changed
            // by at least tic amount (1%)
            if((j-last_j)>=tic)
            {
                ProgressFunction((int)(100.0*j/(nNum-nAF)),
                    "Calculating MTIE");
                last_j=j;
            }
            #endif
        }

        // Initialize the MTIE value
        // Note that the results array is zero-based
        fMTIE=(float)(fBig[0]-fSmall[0]);
        
        // Loop thru the rest of the nNum-nAF points in each vector
        for(i=1; i<nNum-nAF; i++)
        {
            // MTIE is the overall maximum p-p excursion
            fMTIE=(float)(MAX((fBig[i]-fSmall[i]), fMTIE));
        }

        // Store results
        pResults->sigma[k-1]=fMTIE;
        pResults->points=k;
        pResults->af[k-1]=nAF;
        pResults->num[k-1]=(float)(nNum-nAF);
        // Note: tau calc external to this function

		#if(0) // Is this needed? If so, can't do this way in DLL
		// Display results
        SendMessage(hDialog, WM_COMMAND,
            GET_WM_COMMAND_MPS(IDC_RUN_DISPLAY, NULL, 0));
		#endif

        // Check for abort
        if(bAbort)
        {
            // Bail out - program flow goes to code below, then to IDCANCEL
            // handler, and finally back to where this function was called
            break;
        }
    }

    // Free working memory
    free(fBig);
    free(fSmall);

    // Done
    // Clear progress display
    // ProgressFunction(0, ""); 
    return k;
}

/*****************************************************************************/


