/***************************************************************************/
/*                                                                         */
/*  					   CalcFreqModSigma()                              */
/*                                                                         */
/*		Function to calculate modified Allan deviation for freq dat        */
/*                                                                         */
/*		Parameters:		F_TYPE y[]    = frequency data                     */
/*                                      y[0] = # data points               */
/*                                      y[1]  = analysis start             */
/*                                      y[2] = analysis end                */
/*                      F_TYPE *p_sig = pointer to mod Allan deviation     */
/*			            int m         = averaging factor = tau/tau_0       */
/*                                                                         */
/*		Return:         int           = # analysis points,                 */
/*                                      or -1 if error                     */
/*                                                                         */
/*		Notes:          Zeros treated as gaps in frequency data.		   */
/*			            Frequency data is not averaged before calling      */
/*                      this function.  The data consists of num-1 freq    */
/*                      samples at interval tau_0.  m must be >0.          */
/*			            Denominator term (num-3m+1) must be >=1            */
/*			            This sets max m=(int)(num/3), where num=           */
/*			            (# frequency points +1) before averaging.          */
/*                                                                         */
/*		Revision record:                                                   */
/*			12/16/91	Created                                            */
/*			12/31/91	Renamed                                            */
/*          01/16/92    Edited per FrequenC documentation                  */
/*          01/17/92    Edited title                                       */
/*			01/18/92	Edited title block                                 */
/*			01/19/92	Added error traps                                  */
/*			02/03/96	Modified for use as Win 3.1 DLL                    */
/*			01/01/98	Changes for MS VC++ compatibility & warnings       */
/*                                                                         */
/* (c) Copyright 1991-8  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

int CalcFreqModSigma(F_TYPE y[],
	F_TYPE *p_sig, int m)
{
	double sum_i, sum_j, sum_k;     /* summing variables to calc mod sigma */
	float fm;                                        /* float version of m */
	int i;                                 /* index for inner summing loop */
	int j;                                 /* index for outer summing loop */
	int k;                               /* index for inner averaging loop */
	int n_avg;                               /* # points in averaging loop */
	int n_sum=0;                         /* # points in outer summing loop */
	int num = (int)(*(y+2)-*(y+1)+2);  /* # phase data points for analysis */
													 /* = # freq points +1 */
	if( (m<1) || (num<3*m) )
	{
		return(-1);                                               /* error */
	}

	fm=(float)m;
	sum_j=0;
	for(j=1; j<=num-(3*m)+1; j++)                            /* outer loop */
		  /* j goes from 1 to N-3m+1 where N=# phase points & m=avg factor */
										  /* regardless of analysis limits */
	{
		sum_i=0;
		for(i=j+(int)(*(y+1))-1; i<=m+j-1+(int)(*(y+1))-1; i++)   /* inner */
				   /* i is the actual freq data index;  it includes a term */
					/* *(y+1)-1 to account for the analysis starting point */
				 /* the data array offset +2 is included in the code below */
		{
			sum_k=0;
			n_avg=0;                        /* # freq differences averaged */
			for(k=i; k<=i+m-1; k++)             /* find frequency averages */
			{
				if(*(y+k+2+m) && *(y+k+2))       /* skip if either is zero */
				{
					sum_k += *(y+k+2+m) - *(y+k+2);
					n_avg++;
				}                                                /* end if */
			}                                                   /* end for */
			if(n_avg)                /* test to avoid div by 0 if all gaps */
			{
				sum_i+=sum_k/n_avg;
			}
		}                                                       /* end for */
		if(sum_i)
		{
			sum_j += SQR(sum_i);
			n_sum++;
		}                                                            /* if */
	}                                                           /* end for */
	if(n_sum>0)
	{
		*p_sig = sqrt((sum_j)/(2.0*fm*fm*n_sum));
		return(n_sum);                                /* # analysis points */
	}
	else
	{
		return(-1);                                               /* error */
	}
}                                                /* end CalcFreqModSigma() */

/***************************************************************************/
