/***************************************************************************/
/*                                                                         */
/*  					   CalcFastModSigma()                              */
/*                                                                         */
/*		Function to calculate modified Allan variance for phase data       */
/*		Uses faster algorithm that avoids recalculation of inner sums      */
/*                                                                         */
/*		Reference:      D.W. Allan, "Time and Frequency Metrology:         */
/*                      Current Status and Future Considerations",         */
/*                      Proc. 5th EFTF, pp. 1-9, March, 1991               */
/*                                                                         */
/*		Parameters:		F_TYPE x[]    = phase data                         */
/*                                      x[0] = # data phase points         */
/*			                            x[1]  = analysis start             */
/*                                      x[2] = analysis end                */
/*                      F_TYPE *p_sig = pointer to mod Allan deviation     */
/*			            F_TYPE tau    = analysis averaging time = mtau_0  */
/*			            int m         = averaging factor = tau/tau_0       */
/*                                                                         */
/*		Return:         int           = # analysis points,                 */
/*										or -1 if error                     */
/*                                                                         */
/*		Notes:  		No gaps allowed in data; check for embedded zeros  */
/*                      before using this function. If gaps, use (slower)  */
/*                      CalcPhaseModSigma(), or use FillGaps() first.      */
/*			            Phase data is not averaged before calling this     */
/*                      function.  The data consists of num phase samples  */
/*                      at interval tau_0.  Denominator term (num-3m+1)    */
/*                      must be >0.  This sets max m=(int)(num/3),         */
/*			            where num=# phase points before averaging.         */
/*                                                                         */
/*		Revision record:                                                   */
/*			12/16/91	Created                                            */
/*			12/31/91	Renamed                                            */
/*          01/16/92    Edited per FrequenC documentation                  */
/*          01/17/92    Edited title                                       */
/*			01/18/92	Edited title block                                 */
/*			02/03/96	Modified for use as Win 3.1 DLL                    */
/*			01/01/98	Changes for MS VC++ compatibility & warnings       */
/*                                                                         */
/* (c) Copyright 1991-8  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

int CalcFastModSigma(F_TYPE x[],
	F_TYPE *p_sig, F_TYPE tau, int m)
{
	double sum_i, sum_j;       /* summing variables to calc modified sigma */
	float fm;                                        /* float version of m */
	int i;                                 /* index for inner summing loop */
	int j;                                 /* index for outer summing loop */
	int s = (int)*(x+1);                                 /* analysis start */
	int e = (int)*(x+2);                                   /* analysis end */
	int n_sum=0;                         /* # points in outer summing loop */
	int num = e-s+1;                   /* # phase data points for analysis */

	fm=(float)m;
	sum_j=0;
	sum_i=0;

	if( (num<3*m) || (m<1)  || (tau<=0) )
	{
		return(-1);                                               /* error */
	}



/* 1st time find the average of all m phase points by summing them */
	for(i=s; i<m+s; i++)               /* i is the actual phase data index */
	  /* it includes the term s to account for the analysis starting point */
				 /* the data array offset +2 is included in the code below */
	{
		sum_i += *(x+i+2+(2*m)) - (2*(*(x+i+2+m))) + *(x+i+2);
	}

	sum_j += SQR(sum_i);
	n_sum++;

/* for rest of points need only drop 1st sum and add next sum */
	for(j=2; j<=num-(3*m)+1; j++)  		      /* j goes from 2 to num-3m+1 */
								/* where num=# phase points & m=avg factor */
										  /* regardless of analysis limits */
	{
		i=j+s-2;                       /* index of old sum_i being dropped */
		sum_i -= *(x+i+2+(2*m)) - (2*(*(x+i+2+m))) + *(x+i+2);

		i=m+j-1+s-1;                    /* index of next sum_i being added */
		sum_i += *(x+i+2+(2*m)) - (2*(*(x+i+2+m))) + *(x+i+2);

		sum_j += SQR(sum_i);
		n_sum++;
	}                                                            /* end if */

	if(n_sum>0)                                          /* avoid div by 0 */
	{
		*p_sig = sqrt((sum_j)/(2.0*tau*tau*fm*fm*n_sum));
		return(n_sum);                                /* # analysis points */
	}
	else
	{
		return(-1);                                               /* error */
	}
}                                                /* end CalcFastModSigma() */

/***************************************************************************/
