/***************************************************************************/
/*                                                                         */
/*                         CalcFreqOverlapSigma()                          */
/*                                                                         */
/*      Function to calculate Allan deviation using overlapping samples    */
/*      of frequency data                                                  */
/*                                                                         */
/*      Parameters:     F_TYPE y[]    = frequency data array               */
/*                                      y[0] = # data points               */
/*                                      y[1]   = analysis start            */
/*                                      y[2] = analysis end                */
/*                      F_TYPE *p_sig = pointer to Allan deviation         */
/*                      int m         = averaging factor                   */
/*                                                                         */
/*      Return:         int           = # analysis points,                 */
/*                                      or -1 if error                     */
/*                                                                         */
/*      Note:           Zeros treated as gaps in frequency data.           */
/*                      Must have at least 1 analysis point; this          */
/*                      requires at least two adjacent non-gap frequency   */
/*                      data points.  m must be >0 and the # non-gap       */
/*                      frequency data points M must be > 2m-1.            */
/*                                                                         */
/*      Revision record:                                                   */
/*          12/16/91    Created                                            */
/*          12/31/91    Renamed                                            */
/*          01/18/92    Edited title block                                 */
/*                      Added error traps                                  */
/*          02/03/96    Modified for use as Win 3.1 DLL                    */
/*          01/01/98    Changes for MS VC++ compatibility & warnings       */
/*          01/29/99    Test for zero 1st difference removed to avoid      */
/*                      problem with low-precision data where two          */
/*                      adjacent frequency data point are identical.       */
/*			03/08/03    Brought into Ver 2.0 of FrequenC lib               */
/*                                                                         */
/* (c)Copyright 1991-2003 Hamilton Technical Services All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

int CalcFreqOverlapSigma(F_TYPE y[],
	F_TYPE *p_sig, int m)
{
    double sum_j, sum_i;                /* summing variables to calc sigma */
    float fm=(float) m;                              /* float version of m */
    int i;                                    /* inner index for averaging */
    int j;                      /* outer index for summing 1st differences */
    int num=0;                                        /* # analysis points */
    int n;                                      /* # frequency data points */

    n=(int)(*(y+2)-*(y+1)+2);                 /* total # phase data points */
                                         /* N=M+1 where M is # freq points */
    if( (m<1) || (n<2*m) )
    {
        return(-1);                                               /* error */
    }

    sum_j=0;
    for( j=1; j<=n-(2*m); j++)                               /* outer loop */
                                 /* j goes over basic range from 1 to N-2m */
    {
        sum_i=0;
        for(i=(int)(j+(*(y+1)-1)); i<=(int)(j+(*(y+1))-1+m-1); i++)  
                                                             /* inner loop */
                        /* i index includes analysis start limit of *(y+1) */
                       /* data array offset of 2 is included in code below */
        {
            if(*(y+i+m+2) && *(y+i+2))
            {
                sum_i += *(y+i+m+2) - *(y+i+2);
            }                                                    /* end if */
        }                                                       /* end for */
  
        sum_j += SQR(sum_i);
        num++;
    }                                                           /* end for */
    if(num>0)
    {
        *p_sig=sqrt(sum_j/(2.0*fm*fm*num));
        return(num);                                  /* # analysis points */
    }
    else
    {
        return(-1);                                               /* error */
    }
}                                            /* end CalcFreqOverlapSigma() */

/***************************************************************************/

