/*****************************************************************************/
/*                                                                           */
/*                        ConvFreqToPhase()                                  */
/*                                                                           */
/*      Function to convert frequency data to phase (time) data              */
/*                                                                           */
/*      Parameters:     F_TYPE y[]   = frequency data                        */
/*                                     y[0] = # data points                  */
/*                                     y[1] = analysis start                 */
/*                                     y[2] = analysis end                   */
/*                      F_TYPE x[]   = phase data                            */
/*                      F_TYPE tau   = time interval between freq samples    */
/*                      BOOL do_norm = normalization flag:                   */
/*                                     0=don't normalize, 1=normalize        */
/*                                                                           */
/*      Return:        int           = # frequency data gaps,                */
/*                                     or -1 if error                        */
/*                                                                           */
/*      Notes:          Calls FrequenC functions mean() and normalize().     */
/*                      Freq data normalized before conversion if do_norm    */
/*                      is TRUE.  This option allows original freq data      */
/*                      to remain unnormalized.  tau must be >0.             */
/*                      Zeros treated as gaps in frequency data.             */
/*                      Phase data in gap extrapolated from last previous    */
/*                      non-zero frequency value.                            */
/*                      Function processes n = y[2] - y[1] +1 freq points    */
/*                      and produces n+1 phase points.                       */
/*                                                                           */
/*      Revision record:                                                     */
/*          08/28/02    Created from ConvFreqToPhase() of FrequenC library   */
/*                      Changed code to preserve gap instesd of linear       */
/*                      integrated phase during gap                          */
/*          08/30/02    Use average freq for gap interpolation               */
/*			03/08/03    Brought into Ver 2.0 of FrequenC lib                 */
/*                                                                           */
/* (c) Copyright 1991-2003 Hamilton Technical Services  All Rights Reserved  */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int ConvFreqToPhase(F_TYPE y[],
	F_TYPE x[],	F_TYPE tau, BOOL do_norm)
{
    int g;                                                    /* # freq gaps */
    int i;                                                        /* y index */
    int j;                                                        /* x index */
    int num;                                                /* # freq points */
    // F_TYPE y_last;                            /* last non-zero freq value */
    F_TYPE y_avg;                                      /* average freq value */
    F_TYPE xbar, *p_xbar;                    /* avg of x[] and pointer to it */

    if(tau<=0)
    {
        return(-1);                                                 /* error */
    }

    num = (int)(y[2]-y[1]+1);                     /* set num = # freq points */
    x[0] = num;                                            /* # phase points */
    x[1] = 1;                                        /* # of 1st phase point */
    x[2] = num;                                     /* # of last phase point */
    p_xbar = &xbar;                             /* assign address to pointer */

    /* put freq data to be converted into x[] array */
    j=3;                                         /* initialize index for x[] */
    for(i = (int)(y[1]+2); i <= (int)(y[2]+2); i++ )
    {
        x[j] = y[i];                                       /* copy freq data */
        j++;                                              /* increment index */
    }

    // Find average frequency value
    CalcMean(x, &y_avg, 1);                                 /* calc avg freq */

    if(do_norm)                  /* normalize frequency data to be converted */
    {
        if((CalcMean( x, p_xbar, 1))==-1)          /* calc average frequency */
        {
            return(-1);                                  /* pass along error */
        }
        if((NormalizeData( x, xbar, 1))==0)      /* normalize frequency data */
        {
            return(-1);                                  /* pass along error */
        }
    }                                                              /* end if */

    /* shift normalized frequency data up 1 position in x[] array */
    for(i = num+2; i >= 3; i-- )
    {
        x[i+1] = x[i];
    }
                                                /* add 1 to # of data points */
    x[0]++;
    x[2]++;
    num++;

    /* convert frequency data to phase data */
    x[3] = 0;                                 /* set 1st phase value to zero */
    // y_last = 0;                                       /* ditto for y_last */
    g=0;                                                /* initialize # gaps */
    for(i = 4; i < num+3; i++ )           
                                    /* phase data goes from y[3] to y[num+2] */
    {
        if( x[i] )                                   /* freq point not a gap */
        {
            // y_last = x[i];                           /* save current freq */
            x[i] = x[i-1] + ( x[i] * tau );
                   /* x[i] is freq on RH side, converted to phase on LH side */
        }                                                          /* end if */
        else                                          /* freq point is a gap */
        {
            #if(0) // Original code - use last freq for interpolation
            x[i] = x[i-1] + ( y_last * tau );
            #endif

            #if(1) // New code 08/30/02 - use avg freq for interpolation
            // Avg freq gives better result for large gaps
            x[i] = x[i-1] + ( y_avg * tau );
            #endif

            // New code added 08/28/02
            // Make phase value zero to preserve gap
            x[i-1]=0.0;

            g++;
        }                                                        /* end else */
    }                                                             /* end for */

    return( g );
}                                                   /* end ConvFreqToPhase() */

/*****************************************************************************/

