/***************************************************************************/
/*                                                                         */
/*                        CalcGreenhallModSigma()                          */
/*                                                                         */
/*		Function to calculate modified Allan variance for phase data.      */
/*		Uses Greenhall algorithm, which is much faster for large n and m.  */
/*                                                                         */
/*  	Reference:		C.A. Greenhall, "A Shortcut for Computing the      */
/*                  	Modified Allan Variance", AFCS '92.                */
/*                                                                         */
/*		Parameters:		F_TYPE x[]	  =	phase data array                   */
/*                                  	x[0] = total # phase data points   */
/*                                  	x[1] = analysis start              */
/*                                  	x[2] = analysis end                */
/*                  	F_TYPE *p_sig = pointer to mod Allan deviation     */
/*						F_TYPE tau	  =	analysis averaging time = m*0     */
/*										where 0 is measurement interval   */
/*										of phase data                      */
/*						int m         =	averaging factor = /0            */
/*                                                                         */
/*		Return:			int 		  =	# analysis points, or -1 if error  */
/*                                                                         */
/*		Notes:			No gaps are allowed in phase data.                 */
/*						Averaging factor, m, is n in Greenhall paper.      */
/*						# data points, n, is N in Greenhall paper.         */
/*						Must have at least 1 analysis point; this requires */
/*						at least 3 phase data points.  m must be >0,       */
/*						and cannot exceed (int)(n/3).  tau must be >0.     */
/*                                                                         */
/*		Revision record:                                                   */
/*			04/24/92	Drafted & debugged                                 */
/*			04/25/92	Added use of start and end analysis limits         */
/*          04/26/92	Changed to dynamic memory allocation for w[]       */
/*			02/03/96	Modified for use as Win 3.1 DLL                    */
/*			01/01/98	Changes for MS VC++ compatibility & warnings       */
/*                                                                         */
/*	(c) Copyright 1992-8  Hamilton Technical Services 	All Right Reserved */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"                              /* FrequenC header file */

int CalcGreenhallModSigma(F_TYPE x[],
	F_TYPE *p_sig, F_TYPE tau, int m)
{
	F_TYPE *w;                                     /* pointer to w[] array */
	double sum=0;                                    /* summation variable */
	float fm=(float)m;                               /* float version of m */
	int i;                                                  /* array index */
	int j;                                             /* difference index */
	int n=(int)(x[2]-x[1]+1);                             /* # data points */

	if( (n<3*m) || (m<1)  || (tau<=0) )         /* check for NG parameters */
	{
		return(-1);                                          /* error code */
	}                                                            /* end if */

										   /* allocate space for w[] array */
	if((w=(F_TYPE *)malloc((n+1)*sizeof(F_TYPE)))==NULL)
	{
		printf("\nCan't Allocate Memory To Load Phase Data");
		return(-1);                                          /* error code */
	}                                                            /* end if */

/* x[] index starts at i+3 because actual phase data starts at x[3] */
								/* code does not handle gaps in phase data */
	w[0]=0;                                                      /* w[0]=0 */
	for(i=0; i<n; i++)          /* calc w[i]'s as cumulative sum of x[i]'s */
	{
		w[i+1]=w[i]+x[i+3];                                 /* calc next w */
	}                                                           /* end for */

	for(j=1; j<=3; j++)                 /* calc 1st, 2nd & 3rd differences */
	{
		for(i=0; i<=n-j*m; i++)                        /* calc differences */
		{
			w[i]=w[i+m]-w[i];             /* calc difffrence with stride=m */
		}                                                       /* end for */
	}                                                           /* end for */

	for(i=0; i<=n-3*m; i++)                       /* do final calculations */
	{
		sum+=w[i]*w[i];                           /* sum squares of w[i]'s */
	}                                                           /* end for */

	n=n-3*m+1;                                        /* # analysis points */
	*p_sig=sqrt(sum/(2*fm*fm*tau*tau*n));                     /* Mod sigma */
	free(w);                                          /* deallocate memory */
	return(n);                                            /* normal return */
}                                           /* end CalcGreenhallModSigma() */

/***************************************************************************/
