/***************************************************************************/
/*                                                                         */
/*                           CalcHadamardB1()                              */
/*                                                                         */
/*      Function to calc the Hadamard B1 Bias Function BH1(N, 1, mu),      */
/*      the ratio of the standard to Hadamard variances as a function of   */
/*      N = # samples and integer mu = slope of log sigma vs log tau plot  */
/*      for r = 1 (no dead time).  This function applies for integer mu =  */
/*      -1 thru 3 and N > 1.  BH1 = 1 for N = 2 for any mu.  This function */
/*      can be used to ID power law noises from alpha = 0 to -4.           */
/*                                                                         */
/*      Noise types handled:    Alpha   Mu  FM                             */
/*                              -4      3   RR                             */
/*                              -3      2   FW                             */
/*                              -2      1   RW                             */
/*                              -1      0    F                             */
/*                               0     -1    W                             */
/*                                                                         */
/*      References:    (1) NBS Technical Note 375, J.A. Barnes, "Tables    */
/*                         of Bias Functions B1 and B2, for Variances      */
/*                         Based on Finite Samples of Processes with       */
/*                         Power Law Spectral Densities", January, 1969.   */
/*                     (2) NIST Technical Note 1318, J.A. Barnes and D.W.  */
/*                         Allan, "Variances Based on Data with Dead Time  */
/*                         Between the Measurements", 1990.                */
/*                     (3) NIST Technical Note 1337, TN-296 to TN-335,     */
/*                         J.A. Barnes and D.W. Allan, "Variances Based    */
/*                         on Data with Dead Time Between the              */
/*                         Measurements", 1990.                            */
/*                     (4) J.A. Barnes, "The Analysis of Time and          */
/*                         Frequency Data", Austron, Inc, December, 1991.  */
/*                     (5) C.A Greenhall, "A Hadamard B1 Bias Function"    */
/*                         (private communication), 04/23/99.              */
/*                                                                         */
/*      Parameters:    int num = # samples                                 */
/*                               Get -1 error code for n <2                */
/*                     int mu  = noise type                                */
/*                               Get -1 error code for mu < -1 or mu > 3   */
/*                                                                         */
/*      Return:        float   = B1 bias function value,                   */
/*                               or -1 if error                            */
/*                                                                         */
/*      Revision record:                                                   */
/*          04/24/99   Drafted & debugged                                  */
/*                                                                         */
/* (c) Copyright 1999    Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

float CalcHadamardB1(int num, int mu)
{
    double n=(double)num;       // Double version of num
    double m=(double)mu;        // Double version of mu
    double b;                   // BH1 bias function
    double b1, b2, b3;          // Working variables

    if(n<2)                     // Num must be at least 2
    {
        return(-1.0);           // Error code
    }

    switch(mu)                  // Switch according to mu
                                // Different formulae are required for
                                // mu = 0 & 2 to avoid singularities
    {
        case -1:                // White FM (alpha=0)
        case  1:                // Random Walk FM (alpha=-2)
        case  3:                // Random Run FM (alpha=-4)
        {
            b1=2*pow(1-(1/n), m+2);
            b2=2*pow(1+(1/n), m+2);
            b3=pow(n, m);
            b=b3*(4-b2-b1);
            b1=4/(n*n);
            b2=pow(2, m+3);
            b+=-8+b2+b1;
            b1=pow(3, m);
            b2=pow(2, m);
            b3=-15+24*b2-9*b1;
            b*=n/((n-1)*b3);
            break;
        }

        case 0:                 // Flicker FM (alpha=-1)
        {
            b1=1-(1/n);
            b2=1+(1/n);
            b=log(b1);
            b*=-2*b1*b1;
            b3=log(b2);
            b3*=-2*b2*b2;
            b+=b3;
            b-=4*log(n)/(n*n);
            b+=8*log(2);
            b1=24*log(2)-9*log(3);
            b*=n/((n-1)*b1);
            break;
        }

        case 2:                 // Flicker Walk FM (alpha=-3)
        {
            b1=1-(1/n);
            b2=1+(1/n);
            b=log(b1);
            b*=b1*b1*b1*b1;
            b3=log(b2);
            b3*=b2*b2*b2*b2;
            b+=b3;
            b*=-2*n*n;
            b-=4*log(n)/(n*n);
            b-=24*log(n);
            b+=32*log(2);
            b1=96*log(2)-81*log(3);
            b*=n/((n-1)*b1);
            break;
        }

        default:                // Unallowed mu
        {
            b=-1.0;             // Error code
            break;
        }
    }                                                        

    return((float)b);           // Return valid value of BH1 bias function
}

/***************************************************************************/

