/*****************************************************************************/
/*                                                                           */
/*                                 CalcInvChiSqr()                           */
/*                                                                           */
/*      Code to replace CalcInvChiSqr() of FrequenC32 library                */
/*                                                                           */
/*      1st function calculates closed-form approximation to inverse chi     */
/*      square for large # degrees of freedom (OK for df>10)                 */
/*                                                                           */
/*      References: (1) C. Greenhall/JPL via e-mail 12/06/96                 */
/*                  (2) Abramowitz & Stegun, Handbook of Mathematical        */
/*                      Functions, Sections 26.2.22 & 26.4.17                */
/*                                                                           */
/*      2nd function uses a WJR iterative process to calculate the           */
/*      value of chi squared associated with a given probability for         */
/*      a certain # of degrees of freedom.  Calls CalcChiSqrProb().          */
/*                                                                           */
/*      Parameters:    float p  = chi squared probability value              */
/*                     float df = # of degrees of freedom (not necessarily   */
/*                                an integer)                                */
/*                                                                           */
/*      Return:        float    = chi square value,                          */
/*                                or -1 if error                             */
/*                                                                           */
/*      Revision record:                                                     */
/*          02/03/97   Adapted from CalcInvChiSqr() of FrequenC Library      */
/*          02/04/97   Added closed-form function for large df.  Faster,     */
/*                     and avoids possible convergence problems.             */
/*          02/05/97   Fixed problem with wrong tail for new large df calc   */
/*          05/16/00   Use NR routines to calc chi sqr probability           */
/*                     Fixes discontunity in chi sqr vs. df                  */
/*          04/13/02   Watch out when df<1 - can hang up                     */
/*			03/08/03   Brought into Ver 2.0 of FrequenC lib                  */
/*          12/06/17   Added MIN macro usage                                 */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

// Prototypes for Numerical Recipes functions used here - not exported
float gammln(float xx);
float gammp(float a, float x);
float gammq(float a, float x);
void gcf(float* gammcf, float a, float x, float* gln);
void gser(float* gamser, float a, float x, float* gln);

float CalcInvChiSqr(float p, float df)
{
    // Local variables
    double x;                       // Chi-squared value
    double prob;                    // Degrees of freedom
    double sign;                    // Iteration control sign
    double div;                     // Iteration control divisor

    // Closed-form variables
    double a0=2.30753;              // Coefficients from A&S 26.2.22
    double a1=0.27601;              // "
    double b1=0.99229;              // "
    double b2=0.04481;              // "
    double b;
    double n;
    double p1;
    double s;
    double t;
    double y;

    if(p<=0.00001)                    /* check for small, zero or negative p */
    {                                     /* no/slow convergence for small p */
        return(-1.0);                                          /* error code */
    }

    // Use closed-form approximation for df > 100
    if(df>100)
    {
        // Use opposite tail of distribution
        p=1-p;

        // Calc approximation to inverse chi sqr for large df
        p1=MIN(p, 1-p);
        t=sqrt(-2.0*log(p1));                       // A&S 26.2.22
        n=t-((a0+(a1*t))/(1+(t*(b1+(t*b2)))));      // "
        b=2.0/(9.0*df);                             // A&S 26.4.17

        if((p-0.5)<0) s=-1.0;                       // Greenhall revision
        else if((p-0.5)>0) s=1.0;                   // 1 function for
        else s=0.0;                                 // both tails

        y=1-b+(s*n*sqrt(b));                        // A&S 26.4.17
        x=df*pow(y, 3.0);                           // "

        return (float)x;
    }

    // Iterative solution for df<=100
    x = df+(0.5-p)*df*0.5;                    /* start value for chi squared */

    prob=gammp(df/2, (float)(x/2));
    prob=1-prob;

    div = 0.1;

    while(fabs((prob-p)/p)>0.0001)                     /* accuracy criterion */
    {
        if(prob-p>0)                                   /* save sign of error */
        {
            sign=1.0;
        }
        else
        {
            sign=-1.0;
        }
        x += df*(prob-p)/div;                         /* iteration increment */
        if(x>0)                                   /* make sure argument is + */
        {
            prob=gammp(df/2, (float)(x/2));
            prob=1-prob;
        
        }
        else                      /* otherwise restore it & reduce increment */
        {
            x -= df*(prob-p)/div;
            div*=2;
        }
        if(((prob-p)/sign)<0)                  /* did sign of error reverse? */
        {
            div*=2;                            /* reduce increment if it did */
        }
    }
    return (float)x;
}

/***************************************************************************/
/*                                                                         */
/*    Numerical Recipes Routines to Calculate Chi Square Probability       */
/*                                                                         */
/*    Call with p=gammp(df/2, x/2);                                        */
/*    where p=chi sqr prob, df=deg of freedom, x=chi sqr val               */
/*                                                                         */
/***************************************************************************/

// Defines
#define ITMAX 100
#define EPS 3.0e-7

/***************************************************************************/
/*                                                                         */
/*                           gamlm()                                       */
/*                                                                         */
/***************************************************************************/
float gammln(float xx)
{
    double x,tmp,ser;
    static double cof[6]={76.18009173,-86.50532033,24.01409822,
        -1.231739516,0.120858003e-2,-0.536382e-5};
    int j;

    x=xx-1.0;
    tmp=x+5.5;
    tmp -= (x+0.5)*log(tmp);
    ser=1.0;
    for(j=0;j<=5;j++)
    {
        x += 1.0;
        ser += cof[j]/x;
    }
    return (float)(-tmp+log(2.50662827465*ser));
}
/***************************************************************************/
/*                                                                         */
/*                           gammp()                                       */
/*                                                                         */
/***************************************************************************/
float gammp(float a, float x)
{
    float gamser,gammcf,gln;

    if(x < 0.0 || a <= 0.0)
    {
        // TRACE_STR("Invalid arguments in routine GAMMP");
    }
    if(x < (a+1.0))
    {
        gser(&gamser,a,x,&gln);
        return gamser;
    }
    else
    {
        gcf(&gammcf,a,x,&gln);
        return (float)(1.0-gammcf);
    }
}
/***************************************************************************/
/*                                                                         */
/*                           gammq()                                       */
/*                                                                         */
/***************************************************************************/
float gammq(float a, float x)
{
    float gamser,gammcf,gln;

    if(x < 0.0 || a <= 0.0)
    {
        // TRACE_STR("Invalid arguments in routine GAMMQ");
    }
    if(x < (a+1.0))
    {
        gser(&gamser,a,x,&gln);
        return (float)(1.0-gamser);
    }
    else
    {
        gcf(&gammcf,a,x,&gln);
        return gammcf;
    }
}
/***************************************************************************/
/*                                                                         */
/*                           gammcf()                                      */
/*                                                                         */
/***************************************************************************/
void gcf(float* gammcf, float a, float x, float* gln)
{
    int n;
    float gold=0.0,g,fac=1.0,b1=1.0;
    float b0=0.0,anf,ana,an,a1,a0=1.0;

    *gln=gammln(a);
    a1=x;
    for(n=1;n<=ITMAX;n++)
    {
        an=(float) n;
        ana=an-a;
        a0=(a1+a0*ana)*fac;
        b0=(b1+b0*ana)*fac;
        anf=an*fac;
        a1=x*a0+anf*a1;
        b1=x*b0+anf*b1;
        if(a1)
        {
            fac=(float)1.0/a1;
            g=b1*fac;
            if(fabs((g-gold)/g) < EPS)
            {
                *gammcf=(float)exp(-x+a*log(x)-(*gln))*g;
                return;
            }
            gold=g;
        }
    }
}

/***************************************************************************/
/*                                                                         */
/*                           gser()                                        */
/*                                                                         */
/***************************************************************************/
void gser(float* gamser, float a, float x, float* gln)
{
    int n;
    float sum,del,ap;

    *gln=gammln(a);
    if(x <= 0.0)
    {
        if(x < 0.0)
        {
            // TRACE_STR("x less than 0 in routine GSER");
        }
        *gamser=0.0;
        return;
    }
    else
    {
        ap=a;
        del=sum=(float)1.0/a;
        for(n=1;n<=ITMAX;n++)
        {
            ap += 1.0;
            del *= x/ap;
            sum += del;
            if(fabs(del) < fabs(sum)*EPS)
            {
                *gamser=sum*(float)exp(-x+a*log(x)-(*gln));
                return;
            }
        }
        return;
    }
}

/*****************************************************************************/

