/***************************************************************************/
/*                                                                         */
/*  						CalcLogFreqDrift()                             */
/*                                                                         */
/*		Function to calculate log frequency drift                          */
/*		Fit fractional frequency data to log drift model:                  */
/*			            y(t) = aùln(bt+1)+c                                */
/*			            where: y(t) = fractional frequency                 */
/*					           t	= time                                 */
/* 					           a    = multiplicative term                  */
/*					           b    = log time term                        */
/*					           c	= constant term                        */
/*                                                                         */
/*      Reference:      MIL-O-55310B, General Specification for Crystal    */
/*                      Oscillators                                        */
/*                                                                         */
/*		Parameters:		F_TYPE y[]  = frequency data                       */
/*                                    y[0] = # data points                 */
/*			                          y[1] = analysis start                */
/*                                    y[2] = analysis end                  */
/*      	            F_TYPE *p_a  = pointer to multiplicative term      */
/*      	            F_TYPE *p_b  = pointer to time constant term       */
/*			            F_TYPE *p_c  = pointer to constant term            */
/*			            F_TYPE *p_v  = pointer to fit variance             */
/*                                                                         */
/*      Return:         int          = # non-zero anlaysis points,         */
/*                                     or -1 if error                      */
/*                                                                         */
/*	    Notes:          Zeros treated as gaps in frequency data.           */
/*		                A value of b is chosen, the linear fit parameters  */
/*                      are found, and b is varied iteratively until the   */
/*                      rms error is below a limit criterion.              */
/*						Need at least 2 non-zero data points.              */
/*                                                                         */
/*		Revision record:                                                   */
/*			12/16/91	Created                                            */
/*			12/31/91	Renamed                                            */
/*          01/18/92    Edited title block                                 */
/*			01/19/92	Changed error trap                                 */
/*			02/03/96	Modified for use as Win 3.1 DLL                    */
/*			01/01/98	Changes for MS VC++ compatibility & warnings       */
/*			08/29/07	Tightened error limit                              */
/*  												                       */
/* (c) Copyright 1991-2007 Hamilton Technical Services All Rights Reserved */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

int CalcLogFreqDrift(F_TYPE y[], F_TYPE *p_a, F_TYPE *p_b, F_TYPE *p_c,
	F_TYPE *p_v)
{
	int i;                                              /* freq data index */
	int num = (int)(*(y+2)-*(y+1)+1);           /* # frequency data points */
	//double error_limit=0.00001;   /* fractional error limit for variance */
	double error_limit=1e-6;        /* fractional error limit for variance */
	double x;                                               /* x=ln(b*t+1) */
	double err=1.0;                                /* sum of squared error */
	double prev_err=0;  /* initialized so (err-prev_err)/err > error_limit */
	double sum_x;                                     /* sum of x=ln(bt+1) */
	double sum_y;                                              /* sum of y */
	double sum_xx;                                           /* sum of x^2 */
	double sum_xy;                                            /* sum of xy */
	double d=0.1;                                    /* b iteration factor */

	#if(0) // For testing
	int iter=1;
	char szBuffer[BUFFER_SIZE+1];       // Text buffer
	FILE *fOut;					        // File pointer

	// Open file
	fOut=fopen("LogFit.dat", "w");

	// Compose error limit heading
	_snprintf(szBuffer, sizeof(szBuffer), "error_limit = %e\n", error_limit);
	// Write line to file
	fputs(szBuffer, fOut);
	// Compose column headings
	_snprintf(szBuffer, sizeof(szBuffer),
		"#			a			b				c				"
		"d				err				lim\n");
	// Write line to file
	fputs(szBuffer, fOut);
	#endif

	*p_b=1.0;                                       /* initial value for b */

	while(fabs(((err-prev_err)/err))>error_limit)               /* iterate */
	{
		prev_err=err;                               /* save previous error */
		sum_x = 0;                                    /* sum of x=ln(bt+1) */
		sum_y = 0;                                             /* sum of y */
		sum_xx = 0;                                          /* sum of x^2 */
		sum_xy = 0;                                           /* sum of xy */
		num = (int)(*(y+2)-*(y+1)+1);           /* # frequency data points */

		for( i=(int)(*(y+1)+2); i<=(int)(*(y+2)+2); i++)
			                                    /* accumulate sums for fit */
		{
			if(*(y+i))                                     /* gap in data? */
			{
				x=log((*p_b * (i-2))+1);
				sum_x  += x;
				sum_y  += *(y+i);
				sum_xx += x*x;
				sum_xy += x * *(y+i);
			}                                                    /* end if */
			else
			{
				num--;
			}                                                  /* end else */
		}                                                       /* end for */

		if(num<2)                      /* must have at least 2 data points */
		{
			return(-1);
		}                                                        /* end if */

		*p_a = sum_xy - (sum_x * sum_y / num);          /* find parameters */
		*p_a /= sum_xx- (sum_x * sum_x / num);
		*p_c = (sum_y - (*p_a * sum_x ))/ num;

		err=0;                                /* calc sum of squared error */
		for( i=(int)(*(y+1)+2); i<=(int)(*(y+2)+2); i++)
		{
			if(*(y+i))
			{
				err += SQR(*p_a * log((*p_b * (i-2))+1) + *p_c - *(y+i));
			}
		}

		if(err>prev_err)                                     /* wrong way? */
		{
			d *=-0.5;
		}
		*p_b += *p_b * d;                   /* modify b for next iteration */
		
		#if(0) // For testing
		// Write results of iteration to file so it can be examined
		// Compose line
		_snprintf(szBuffer, sizeof(szBuffer), "%d\t%e\t%e\t%e\t%e\t%e\t%e\n",
			iter, *p_a, *p_b, *p_c, d, err, (err-prev_err)/err);

		// Write line to file
		fputs(szBuffer, fOut);

		// Increment iteration #
		iter++;
		#endif

	}
		*p_v = err;

		#if(0) // For testing
		// Close file
		fclose(fOut);
		#endif

		return(num);
}                                                /* end CalcLogFreqDrift() */

/***************************************************************************/


