/*****************************************************************************/
/*                                                                           */
/*  			       		 CalcPhaseHadamardDev()                          */
/*                                                                           */
/*		Function to calculate Hadamard variance from phase data              */
/*                                                                           */
/*                    1     N-3                                              */
/*		H y() =    [x(i+3) -3x(i+2) + 3x(i+1) -x(i)]            */
/*				   6(N-3) i=1                                              */
/*                                                                           */
/*                                                                           */
/*		Parameters:		F_TYPE x[]    = phase data                           */
/*                                      x[0] = # data points                 */
/*			                            x[1] = analysis start                */
/*                                      x[2] = analysis end                  */
/*                      F_TYPE *p_dev = pointer to Hadamard deviation        */
/*			            F_TYPE tau    = time between phase measurements      */
/*                                                                           */
/*		Return:         int           = # analysis points,                   */
/*										or -1 if error                       */
/*                                                                           */
/*		Note:	        Embedded zeros treated as gaps in phase data.        */
/*						Must have at least 1 analysis point; this            */
/*                      requires at least 4 adjacent non-gap phase data      */
/*                      points.  Tau must be >0.                             */
/*	                                                                         */
/*		Reference:		T. Walter, "A Multi-Variance Analysis in the Time    */
/*						Domain", Proc. 24th PTTI Meeting, Dec. 1992.         */
/*                                                                           */
/*		Revision record:                                                     */
/*			04/26/96	Created from STAB_F45.C                              */
/*			01/01/98	Changes for MS VC++ compatibility & warnings         */
/*                                                                           */
/* (c) Copyright 1996-8 Hamilton Technical Services   All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int CalcPhaseHadamardDev(F_TYPE x[],
	F_TYPE *p_dev, F_TYPE tau)
{
	double sum;                          /* summing variable to calc sigma */
	int i;                                            /* index for summing */
	int num = 0;                                      /* # analysis points */

	if(tau<=0)
	{
		return(-1);                                               /* error */
	}

	for( i = (int)(*(x+1)+2), sum = 0; i < (int)(*(x+2)); i++)
											   /* index goes from 1 to n-3 */
					 /* for n data points from 1 to n; data starts at x[3] */
	{
		if(    ( i==*(x+1)+2 && *(x+i+1) && *(x+i+2) && *(x+i+3) )
		   ||  ( *(x+i)      && *(x+i+1) && *(x+i+2) && *(x+i+3) )
		   ||  ( i==*(x+2)   && *(x+i  ) && *(x+i+1) && *(x+i+2) ) )
						  /* skip if any imbedded phase data value is zero */
		{
			sum += SQR( *(x+i+3) - 3 * *(x+i+2) + 3 * *(x+i+1) - *(x+i) );
			num++;
		}                                                        /* end if */
	}                                                           /* end for */

	if(num>0)
	{
		*p_dev = ( sqrt( sum / (6.0 * num * tau * tau )));
		return(num);                                  /* # analysis points */
	}                                                            /* end if */
	else                                             /* no analysis points */
	{
		return(-1);                                               /* error */
	}                                                          /* end else */
}

/***************************************************************************/
