/*****************************************************************************/
/*                                                                           */
/*  					   CalcPhaseModSigma()                               */
/*                                                                           */
/*		Function to calculate modified Allan variance for phase data         */
/*                                                                           */
/*		Parameters:	    F_TYPE x[]    = phase data                           */
/*                                      x[0] = # data phase points           */
/*			                            x[1] = analysis start                */
/*                                      x[2] = analysis end                  */
/*                      F_TYPE *p_sig = pointer to mod Allan deviation       */
/*			            F_TYPE tau    = analysis averaging time = mtau_0    */
/*			            int m	      = averaging factor = tau/tau_0         */
/*                                                                           */
/*		Return:         int           = # analysis points,                   */
/*										or -1 if error                       */
/*                                                                           */
/*		Notes:          Embedded zero phase value treated as gap in data     */
/*			            Phase data is not averaged before calling this       */
/*                      function.  The data consists of num phase samples    */
/*                      at interval tau_0.  Denominator term (num-3m+1)      */
/*                      must be >0.  This sets max m=(int)(num/3),           */
/*			            where num=# phase points before averaging.           */
/*						m must be >0 and tau must be >0.                     */
/*			            Use faster CalcFastModSigma() if no gaps in data.    */
/*                                                                           */
/*		Revision record:                                                     */
/*			12/16/90	Created                                              */
/*			12/31/91	Renamed                                              */
/*          01/18/92    Edited title block                                   */
/*			01/19/92	Added error traps                                    */
/*			02/03/96	Modified for use as Win 3.1 DLL                      */
/*			11/27/97	Modified for Win 32 compatibility                    */
/*						Fixed error with gaps                                */
/*						Replaces bad code in FREQUENC.DLL                    */
/*			12/09/97	Added to FrequenC Library                            */
/*			01/01/98	Changes for MS VC++ compatibility & warnings         */
/*                                                                           */
/* (c) Copyright 1990-8  Hamilton Technical Services  All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int CalcPhaseModSigma(F_TYPE x[],
	F_TYPE *p_sig, F_TYPE tau, int m)
{
	double sum_i, sum_j;              /* summing variables to calc mod sigma */
	float fm;                                          /* float version of m */
	int i;                                   /* index for inner summing loop */
	int j;                                   /* index for outer summing loop */
	int n_sum=0;                           /* # points in outer summing loop */
	int num = (int)(*(x+2)-*(x+1)+1);    /* # phase data points for analysis */

	if( (m<1) || (num<3*m) || (tau<=0) )
	{
		return(-1);                                                 /* error */
	}


	fm=(float)m;
	sum_j=0;
	for(j=1; j<=num-(3*m)+1; j++)                              /* outer loop */
			/* j goes from 1 to N-3m+1 where N=# phase points & m=avg factor */
											/* regardless of analysis limits */
	{
		sum_i=0;
		for(i=j+(int)(*(x+1))-1; i<=m+j-1+(int)(*(x+1))-1; i++)/* inner loop */
					/* i is the actual phase data index;  it includes a term */
					  /* *(x+1)-1 to account for the analysis starting point */
				   /* the data array offset +2 is included in the code below */
		{
			#if(0) // Original code
			if(    ( i==j+(*(x+1))-1     && *(x+i+2+m) && *(x+i+2+(2*m)) )
			   ||  ( *(x+i+2)            && *(x+i+2+m) && *(x+i+2+(2*m)) )
			   ||  ( i==m+j-1+(*(x+1))-1 && *(x+i+2+m) && *(x+i)       ) )
							/* skip if any imbedded phase data value is zero */
			#endif

			#if(1) // New code 11/27/97
			// Note: Skip calc if any of the 3 points used in the calc
			// are 0 unless they are the very first or the very last points
			// of the complete phase data array.  The middle OR handles the
			// ordinary case.  The top OR handles the first point case.  The
			// bottom OR handles the last point case.  For the first point
			// case, the first calc index x+i+2 must equal x+3, so i=1.  For
			// the last point case, the third calc index x+i+2+(2*m) must
			// equal x+ *(x) +2, so i=*(x) -2*m
			if(    ( i==1                && *(x+i+2+m) && *(x+i+2+(2*m)) )
			   ||  ( *(x+i+2)            && *(x+i+2+m) && *(x+i+2+(2*m)) )
			   ||  ( *(x+i+2)            && *(x+i+2+m) && i==*(x)-(2*m)) )
							/* skip if any imbedded phase data value is zero */
			#endif
			{
				sum_i += *(x+i+2+(2*m)) - (2*(*(x+i+2+m))) + *(x+i+2);
			}                                                      /* end if */

			#if(0) // For testing
			// Confirmed as first point of array
			if(i==1)
			{
				TRACE_EXP(*(x+i+2));
			}

			// Confirmed as last point of array
			if(i==*(x)-(2*m))
			{
				TRACE_EXP(*(x+i+2+(2*m)));
			}
			#endif

		}                                                         /* end for */
		if(sum_i)
		{
			sum_j += SQR(sum_i);
			n_sum++;
		}                                                          /* end if */
	}                                                             /* end for */
	if(n_sum>0)                                     /* make sure no div by 0 */
	{
		*p_sig = sqrt((sum_j)/(2.0*tau*tau*fm*fm*n_sum));
		return(n_sum);                                  /* # analysis points */
	}
	else
	{
		return(-1);                                                 /* error */
	}
}                                              /* end CalcNewPhaseModSigma() */

/*****************************************************************************/
