/***************************************************************************/
/*                                                                         */
/*                        CalcPhaseOverlapHadamardDev()                    */
/*                                                                         */
/*      Function to calc overlapping Hadamard variance from phase data     */
/*                                                                         */
/*                    1     N-3m                                           */
/*      H y() =     [x(i+3m) -3x(i+2m) + 3x(i+m) -x(i)]       */
/*                 6(N-3m) i=1                                           */
/*                                                                         */
/*                                                                         */
/*      Parameters:     F_TYPE x[]    = phase data                         */
/*                                      x[0] = # data points               */
/*                                      x[1] = analysis start              */
/*                                      x[2] = analysis end                */
/*                      F_TYPE *p_dev = pointer to Hadamard deviation      */
/*                      F_TYPE tau    = time between phase measurements    */
/*                      int    m      = averaging factor                   */
/*                                                                         */
/*      Return:         int           = # analysis points,                 */
/*                                      or -1 if error                     */
/*                                                                         */
/*      Note:           Embedded zeros treated as gaps in phase data.      */
/*                      Must have at least 1 analysis point; this          */
/*                      requires at least 4 adjacent non-gap phase data    */
/*                      points.  Tau must be >0.                           */
/*                                                                         */
/*      Reference:      T. Walter, "A Multi-Variance Analysis in the Time  */
/*                      Domain", Proc. 24th PTTI Meeting, Dec. 1992.       */
/*                      Modified by WJR to use fully-overlapping samples.  */
/*                                                                         */
/*      Revision record:                                                   */
/*          03/05/99    Created from CPHD.C of FrequenC32 Library          */
/*                                                                         */
/* (c) Copyright 1999   Hamilton Technical Services   All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

int CalcPhaseOverlapHadamardDev(F_TYPE x[],
	F_TYPE *p_dev, F_TYPE tau, int m)
{
    double sum;                          /* summing variable to calc sigma */
    int i;                                            /* index for summing */
    int num=0;                                        /* # analysis points */
    float fm=(float)m;                               /* float version of m */

    if(tau<=0)
    {
        return(-1);                                               /* error */
    }

    for(i=(int)(*(x+1)+2), sum=0; i<=(int)(*(x+2)+2-3*m); i++)
                                              /* index goes from 1 to n-3m */
                     /* for n data points from 1 to n; data starts at x[3] */
    {
        if(    ( i==*(x+1)+2 && *(x+i+m) && *(x+i+2*m) && *(x+i+3*m) )
           ||  ( *(x+i)      && *(x+i+m) && *(x+i+2*m) && *(x+i+3*m) )
           ||  ( i==*(x+2)   && *(x+i  ) && *(x+i+m) && *(x+i+2*m) ) )
                          /* skip if any imbedded phase data value is zero */
        {
            sum+=SQR((*(x+i+(3*m)))-3*(*(x+i+(2*m)))+3*(*(x+i+m))-(*(x+i)));
            num++;
        }                                                        /* end if */
    }                                                           /* end for */

    if(num>0)
    {
        *p_dev=sqrt(sum/(6.0*num*tau*tau*fm*fm));
        return(num);                                  /* # analysis points */
    }                                                            /* end if */
    else                                             /* no analysis points */
    {
        return(-1);                                               /* error */
    }                                                          /* end else */
}

/***************************************************************************/

