/*****************************************************************************/
/*                                                                           */
/*                             CalcPhaseStdDev()                             */
/*                                                                           */
/*      Function to calculate standard deviation from phase data             */
/*                                                                           */
/*      Parameters:     F_TYPE x[]    = phase data                           */
/*                                      x[0] = # data points                 */
/*                                      x[1] = analysis start                */
/*                                      x[2] = analysis end                  */
/*                      F_TYPE *p_dev = pointer to standard deviation        */
/*                      F_TYPE tau    = time interval between data points    */
/*                                                                           */
/*      Return:         int           = # analysis points,                   */
/*                                      or -1 if error                       */
/*                                                                           */
/*      Note:           Embedded zeros treated as gaps in phase data.        */
/*                      Must have at least 3 non-gap phase data points,      */
/*                      and tau must be >0.                                  */
/*                                                                           */
/*      Revision record:                                                     */
/*          12/16/90    Created                                              */
/*          12/31/91    Renamed                                              */
/*          01/18/92    Edited title block                                   */
/*          06/05/95    Corrected num per standard_dev() function of         */
/*                      STABLE.  S/B # frequency (not phase) data points.    */
/*          02/03/96    Modified for use as Win 3.1 DLL                      */
/*          01/01/98    Changes for MS VC++ compatibility & warnings         */
/*          05/15/00    Added New to function name & put into Stable_r.c     */
/*                      to fix long-standing error                           */
/*			03/08/03    Brought into Ver 2.0 of FrequenC lib                 */
/*                                                                           */
/* (c) Copyright 1990-2003 Hamilton Technical Services All Rights Reserved   */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int CalcPhaseStdDev(F_TYPE x[], F_TYPE *p_dev,
	F_TYPE tau)
{
    double sum=0;                                        /* summing variable */
    double avg;                                   /* mean value of freq data */
    int i;                                              /* index for summing */
    int num = (int)(*(x+2)-*(x+1));                /* # freq analysis points */

    if(tau<=0)
    {
        return(-1);                                                 /* error */
    }

    for(i=(int)(*(x+1)+2); i<=(int)(*(x+2)+1); i++)     /* calc average freq */
    {
        if(   ( *(x+i)        && *(x+i+1) )
           || ( i == *(x+1)+2 && *(x+i+1) )
           || ( i == *(x+2)+1 && *(x+i)   )  )
                                /* if don't have an embedded zero data point */
        {
            sum += *(x+i+1) - *(x+i);           /* freq is 1st diff of phase */
        }
        else
        {
            num--;
        }
    }

    if((num-2)>0)                        /* check for <3 non-gap data points */
    {
        avg = sum/num;
    }
    else
    {
        return(-1);                                                 /* error */
    }

    sum=0;
    for(i=(int)(*(x+1)+2); i<=(int)(*(x+2)+1); i++)         /* calc variance */
    {
        if(   ( *(x+i)        && *(x+i+1) )                       /* skip if */
           || ( i == *(x+1)+2 && *(x+i+1) )             /* non-imbedded zero */
           || ( i == *(x+2)+1 && *(x+i)   )  )
        {
            sum += SQR( ( *(x+i+1) - *(x+i) ) - avg );
        }                                                          /* end if */
    }                                                             /* end for */

    *p_dev = sqrt(sum/((num-1)*tau*tau));

    return(num);                                   /* # non-zero data points */
}                                                   /* end CalcPhaseStdDev() */

/*****************************************************************************/

