/*****************************************************************************/
/*                                                                           */
/*                        ConvPhaseToFreq()                                  */
/*                                                                           */
/*      Enhanced function to convert phase (time) data to frequency data     */
/*                                                                           */
/*      Parameters:     F_TYPE x[]  = phase data                             */
/*                                    x[0] = # phase data points             */
/*                                    x[1] = analysis start                  */
/*                                    x[2] = analysis end                    */
/*                      F_TYPE y[]  = frequency data                         */
/*                      F_TYPE tau  = time interval between phase samples    */
/*                      BOOL z      = flag to avoid zero freq                */
/*                                                                           */
/*      Return:         int         = # frequency data gaps,                 */
/*                                    or -1 if error                         */
/*                                                                           */
/*      Notes:          Embedded zeros treated as gaps in phase data.        */
/*                      Frequency data in gaps interpolated from nearest     */
/*                      non-gap phase values.  Function processes n =        */
/*                      x[2] - x[1] +1 phase points and produces n-1         */
/*                      frequency points.  tau must be >0.                   */
/*                                                                           */
/*      Revision record:                                                     */
/*          12/27/91    Created                                              */
/*          12/31/91    Renamed                                              */
/*          01/18/92    Edited title block                                   */
/*          01/19/92    Added error trap                                     */
/*          02/03/96    Modified for use as Win 3.1 DLL                      */
/*          01/01/98    Changes for MS VC++ compatibility & warnings         */
/*          05/22/01    Added flag for avoiding gaps if zero freq            */
/*			03/08/03    Brought into Ver 2.0 of FrequenC lib                 */
/*                                                                           */
/* (c) Copyright 1991-2003 Hamilton Technical Services All Rights Reserved   */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int ConvPhaseToFreq(F_TYPE x[], F_TYPE y[],
	F_TYPE tau,	BOOL z)
{
    int g;                                                   /* # phase gaps */
    int i;                                                        /* x index */
    int j;                                                        /* y index */
    int num;                                               /* # phase points */

    if(tau<=0)
    {
        return(-1);                                                 /* error */
    }

    num=(int)(x[2]-x[1]+1);                      /* set num = # phase points */
    y[0]=num-1;                                             /* # freq points */
    y[1]=1;                                           /* # of 1st freq point */
    y[2]=num-1;                                      /* # of last freq point */

    /* put phase data to be converted into y[] array */
    j=3;                                         /* initialize index for y[] */
    for(i=(int)(x[1]+2); i<=(int)(x[2]+2); i++ )
    {
        y[j]=x[i];                                        /* copy phase data */
        j++;                                              /* increment index */
    }

    /* convert phase data to freq data */
    g=0;                                                /* initialize # gaps */
    for(i=3; i<=num+2; i++)
    {
        if( (y[i] && y[i+1]) || (i==3 && y[i+1]) || (y[i] && i==num+2) )
         /* neither phase point a gap, or 1st point with non-zero 2nd point, */
                           /* or last point with non-zero next-to-last point */
        {
            y[i]=(y[i+1]-y[i])/tau;

            /* avoid zero freq = gap */
            if(!y[i] && z)
            {
                y[i]=1e-99;
            }
        }                                                          /* end if */
        else                                                 /* imbedded gap */
        {
            y[i]=0;
            g++;
        }                                                        /* end else */
    }                                                             /* end for */

    return(g);
}                                                   /* end ConvPhaseToFreq() */

/*****************************************************************************/

