/*****************************************************************************/
/*                                                                           */
/*                     ConvPhaseToFreqUsingTimetags()                        */
/*                                                                           */
/*      Function to convert phase (time) data to frequency data              */
/*      using spacing between phase timetags instead of fixed tau            */
/*      to handle irregularly-spaced data                                    */
/*                                                                           */
/*      Parameters:     F_TYPE x[]    = phase data                           */
/*                                      x[0] = # phase data points           */
/*                                      x[1] = analysis start                */
/*                                      x[2] = analysis end                  */
/*                      F_TYPE y[]    = frequency data                       */
/*                      F_TYPE t[]    = phase timetag array                  */
/*                      F_TYPE scale  = phase timetag scale factor           */
/*                      F_TYPE *avg   = pointer to average tau of timetags   */
/*                      BOOL z        = flag to avoid zero freq              */
/*                                                                           */
/*      Return:         int           = # frequency data gaps,               */
/*                                      or -1 if error                       */
/*                                                                           */
/*      Notes:          Embedded zeros treated as gaps in phase data.        */
/*                      Frequency data in gaps interpolated from nearest     */
/*                      non-gap phase values.  Function processes n =        */
/*                      x[2] - x[1] +1 phase points and produces n-1         */
/*                      frequency points.  Timetag units are scaled to       */
/*                      seconds by multiplying them by fScale.               */
/*                                                                           */
/*      Revision record:                                                     */
/*          09/26/02    Created from NewConvPhaseToFreq()                    */
/*			03/16/03	Brought into FrequenC DLL V 2.0                      */
/*                                                                           */
/* (c)Copyright 1991-2003 Hamilton Technical Services All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int  ConvPhaseToFreqUsingTimetags(F_TYPE x[],
	F_TYPE y[], F_TYPE t[], F_TYPE scale, F_TYPE *avg, BOOL z)
{
    /* local variables */
    int g;                                                 /* # phase gaps */
    int i;                                                      /* x index */
    int j;                                                      /* y index */
    int num;                                             /* # phase points */
    int c;                                          /* count of avg points */
    F_TYPE dt;                                       /* timetag difference */

    /* start of code */
    num = (int)(x[2] - x[1] +1);               /* set num = # phase points */
    y[0] = num-1;                                         /* # freq points */
    y[1] = 1;                                       /* # of 1st freq point */
    y[2] = num-1;                                  /* # of last freq point */

    /* initialize average */
    *avg = 0.0;
    c=0;

    /* put phase data to be converted into y[] array */
    j=3;                                       /* initialize index for y[] */
    for(i=(int)(x[1]+2); i<=(int)(x[2]+2); i++)
    {
        y[j] = x[i];                                    /* copy phase data */
        j++;                                            /* increment index */
    }

    /* convert phase data to freq data as delta y[] / delta t[] */
    g=0;                                              /* initialize # gaps */
    for(i=3; i<num+2; i++)
    {
        if( (y[i] && y[i+1]) || (i==3 && y[i+1]) || (y[i] && i==num+2) )
       /* neither phase point a gap, or 1st point with non-zero 2nd point, */
                         /* or last point with non-zero next-to-last point */
        {
            /* Check for zero timetag spacing */
            if((dt=(t[i+1]-t[i]))==0.0)
            {
                // Error
                return -1;
            }

            y[i] = ( y[i+1] - y[i] ) / ( dt * scale );

            /* avoid zero freq = gap */
            if(!y[i] && z)
            {
                y[i]=1e-99;
            }

            /* accumulate average */
            *avg += dt;
            c++;
        }                                                        /* end if */
        else                                               /* imbedded gap */
        {
            y[i] = 0;
            g++;
        }                                                      /* end else */
    }                                                           /* end for */

    /* calc average */
    *avg *= scale;
    *avg /= c;
    
    return( g );
}                                    /* end ConvPhaseToFreqUsingTimetags() */

/***************************************************************************/

