/*****************************************************************************/
/*                                                                           */
/*					         CalcQuadraticDrift()                            */
/*                                                                           */
/*		Function to calculate of least-squares regression                    */
/*      of phase on a quadratic (and find frequency drift)                   */
/*                                                                           */
/* 		Reference:                                                           */
/* 		J.A. Barnes, "The Measurement of Linear Frequency Drift              */
/*		in Oscillators",                                                     */
/* 		Proc. 15th Annual PTTI Meeting, 1983, pp. 551-582                    */
/* 		x(t) = a + b*t + c*t^2                                               */
/* 		x(t) , t & a in sec, b in sec/sec, c in sec/sec^2                    */
/* 		frequency drift (slope) = 2*c; intercept = b                         */
/* 		t is time interval of equally-spaced time data                       */
/* 		t goes from 1 to n with no missing points                            */
/*                                                                           */
/*		Parameters: 	x[]		phase data array                             */
/*						t		time interval for phase data                 */
/*                      *p_a	pointer to phase intercept                   */
/*						*p_b	pointer to frequency intercept               */
/*						*p_c    pointer to frequency slope                   */
/*						*p_v	pointer to fit variance                      */
/*                                                                           */
/*		Return:	int	= # analysis points or -1 if error                       */
/*                                                                           */
/*		Note: Does handle gaps.                                              */
/*                                                                           */
/*		Revision record:                                                     */
/*          05/04/97	Adapted from STABLE_A.C                              */
/*			01/01/98	Changes for MS VC++ compatibility & warnings         */
/*			03/19/03	Added code to handle gaps                            */
/*                                                                           */
/* (c) 1996-2003 W. Riley Hamilton Technical Services All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int CalcQuadraticDrift(F_TYPE x[], F_TYPE t,
	F_TYPE *p_a, F_TYPE *p_b, F_TYPE *p_c, F_TYPE *p_v)
{
	// Local variables
	F_TYPE *z;			           /* Pointer to array to save original data */
	double aa,bb,cc,dd,ee,ff,gg;                             /* coefficients */
	double sx,sxx,snx,snnx;                                    /* summations */
	int size=(int)x[0];                                        /* array size */
	int num=(int)(x[2]-x[1]+1);                             /* # data points */
	float n=0;                                                   /* # points */
	int i;                                                     /* data index */
	int j=1;                                              /* summation index */

	/* check arguments */
	if(t<=0)                                        /* tau negative or zero? */
	{
		return(-1);                                                 /* error */
	}

	if(num<3)                   /* need at least 3 points to calc parameters */
	{
		return(-1);                                                 /* error */
	}

	// Copy original data (in case it has gaps that are filled )
	// Allocate working memory for complete phase data array
	if((z=(F_TYPE*)malloc((size+ARRAY_OFFSET)*sizeof(F_TYPE)))==NULL)
	{
		return -1;            	                                    /* error */
	}

	// Copy data
	for(i=0; i<size+ARRAY_OFFSET; i++)
	{
		z[i]=x[i];
	}

	// Fill gaps if necessary
	FillGaps(z, PHASE);

	/* initialize sums */
	sx = sxx = snx = snnx = 0;

	/* calculate sums for quadratic fit using gap-filled copied data */
	for(i=(int)z[1]; i<=(int)z[2]; i++)
	{
		sx  += z[i+2];
		sxx += SQR(z[i+2]);
		snx += z[i+2]*j;
		snnx+= z[i+2]*j*j;
		j++;
		n++;
	}                                                             /* end for */

	/* bail out if too few points */
	if(n<4)
	{
		*p_a=*p_b=*p_c=*p_v=0;                                  /* no result */
		return(-1);                               /* to avoid divide by zero */
	}

	aa = 3*((3*n*(n+1))+2);
	bb = -18*((2*n)+1);
	cc = 30;
	dd = (12*((2*n)+1)*((8*n)+11))/((n+1)*(n+2));
	ee = -180/(n+2);
	ff = 180/((n+1)*(n+2));
	gg = n*(n-1)*(n-2);

	*p_a = ((aa*sx)+(t*bb*snx)+(t*t*cc*snnx))/(gg    );
	*p_b = ((bb*sx)+(t*dd*snx)+(t*t*ee*snnx))/(gg*t  );
	*p_c = ((cc*sx)+(t*ee*snx)+(t*t*ff*snnx))/(gg*t*t);
	*p_v = (sxx-(*p_a*sx)-(t**p_b*snx)-(t*t**p_c*snnx))/(n-3);

	// Free working memory
	free(z);

	// Done
	return(int)n;
}

/*****************************************************************************/
