/***************************************************************************/
/*                                                                         */
/*                             CalcRatio()                                 */
/*                                                                         */
/*		Function to calculate the function R(n),                           */
/* 		the ratio of modified to normal Allan variance                     */
/*                                                                         */
/*		Reference:     NIST Technical Note 1337, Sec. A.6.                 */
/*                                                                         */
/*      Parameters:	   int alpha = power-law slope of Sy(f)                */
/*					   int num   = # phase samples averaged                */
/*					   float bw  = measurement system bandwidth factor     */
/*							     = 2fh0                                */
/*		Return:		   float     = R(n) = Mod y()/y(),               */
/*                                 or -1 if error                          */
/*                                                                         */
/*		Revision Record:                                                   */
/*			12/16/90   Created                                             */
/*			12/31/91   Renamed                                             */
/*			01/04/92   Added test for n=1                                  */
/*					   Changed error code to -1.0                          */
/*					   Added more error traps                              */
/*          01/18/92   Edited title; parameter b changed to bw             */
/*			02/03/96   Modified for use as Win 3.1 DLL                     */
/*			01/01/98   Changes for MS VC++ compatibility & warnings        */
/*                                                                         */
/* (c) Copyright 1990-8  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

float CalcRatio(int alpha, int num,
	float bw)
{
	float n, r, f1, f2, sum;
	int k;

	n=(float) abs(num);     /* make sure n is positive, then conv to float */

	if(n==0.0)                                  /* avoid math domain error */
	{
		return(-1.0);                                        /* error code */
	}

	if(n==1.0)                /* no need to calc & avoid math domain error */
	{
		return(1.0);
	}

	switch(alpha)
	{
		case -2:                                  /* random walk FM noise */
		{
			r=(float)((0.825)+((1.0)/(8*n*n))+((1.0)/(20*n*n*n*n)));
			break;
		}
		case -1:                                      /* flicker FM noise */
		{
			r=(float)((148+(99.9*pow(n,3.35))-99.9)/(148*pow(n,3.35)));
													  /* r=0.675 for n>14 */
			break;
		}
		case 0:                                         /* white FM noise */
		{
			r=(float)(((n*n)+1.0)/(2*n*n));
			break;
		}
		case 1:                                        /* flicker PM noise */
		{
			if(bw<=0)
			{
				return(-1.0);                                /* error code */
			}
			if(n>100)
			{
				r=(float)((3.37)/(1.04+(3*log(bw*n))));
				break;
			}
			else
			{
				sum=0;
				for(k=1; k<=(n-1); k++)
				{
					f1=((4*n*n)/(k*k))-1;
					f2=((n*n)/(k*k))-1;
					sum+=(float)((n-k)*((4*log(f2))-(log(f1))));
				}
				r=(float)((1/(n*n))*(n+((sum)/((3*log(n*bw))))));
				break;
			}
		}
		case 2:                                          /* white PM noise */
		{
			r=(float)(1.0/n);
			break;
		}
		default:                                        /* unallowed alpha */
		{
			r=-1.0;                                          /* error code */
			break;
		}
	}                                                        /* end switch */

	return(r);
}                                                       /* end CalcRatio() */

/***************************************************************************/
