/*****************************************************************************/
/*                                                                           */
/*                                CalcStarB1()                               */
/*                                                                           */
/*   Function to calculate the Barnes B1 ratio of the N-sample to 2-sample   */
/*   (standard to Allan) variance for frequency data treated as phase data   */
/*   which is used to resolve alpha=3 or -4 ambiguity for Hadamard sigmas.   */
/*   This is called *B1 to distinguish it from the normal B1 ratio.          */
/*                                                                           */
/*   Parameters:    F_TYPE  *fData          Data array                       */
/*                  BOOL    bDataType       Data type (0=Phase, 1=Freq)      */
/*                  int     fTau                                             */
/*                  int     nAF             Averaging factor                 */
/*                                                                           */
/*   Return:        float   fB             *B1 ratio                         */
/*                                          or -1.0 if error                 */
/*                                                                           */
/*   Notes:         What we are doing here is to use Chuck Greenhall's       */
/*                  idea to calculate the B1 ratio for the freq data         */
/*                  as phase data, and use the standard noise ID method      */
/*                  except for adding 2 to the resulting mu (subtracting     */
/*                  2 from alpha).                                           */
/*                                                                           */
/*   References:    1.  C.A. Greenhall, (private communication)              */
/*                  2.  D. Howe, et al PTTI 2000 paper                       */
/*                                                                           */
/*      Revision record:                                                     */
/*          11/14/00    Created                                              */
/*          11/15/00    Changed parameters                                   */
/*                                                                           */
/* (c) Copyright 2000   Hamilton Technical Services   All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/
 
#include "frequenc.h"

float CalcStarB1(F_TYPE fData[],
	BOOL bDataType,	F_TYPE fTau, int nAF)
{
    // Local variables
    F_TYPE *fWork;   // Pointer to working data array
    F_TYPE fSig;     // Allan deviation
    F_TYPE fStdDev;  // Standard deviation
    int nNum;        // # data points
    int i;           // Index
    
    // Get # data points in data array
    nNum=(int)fData[0];

    // Allocate working array of freq data
    // Data is subject to being averaged, so must use a working array
    if((fWork=(F_TYPE*)malloc((nNum+ARRAY_OFFSET)*sizeof(F_TYPE)))==NULL)
    {
        // Error
        return -1.0;
    }

    // For phase data, convert it to freq data in working array
    if(bDataType==PHASE_DATA)
    {
        // Conv phase data to freq data
        ConvPhaseToFreq(fData, fWork, fTau, TRUE);
    }
    else // Freq data - no conversion needed - copy it to working array
    {
        // Copy freq data to working array
        for(i=0; i<nNum+ARRAY_OFFSET; i++)
        {
            fWork[i]=fData[i];
        }
    }
                        
    // Average working freq data
    AverageFreqData(fWork, nAF);
   
    // Calc its sigma and std dev as if it were phase data
    CalcPhaseSigma(fWork, &fSig, fTau);
    CalcPhaseStdDev(fWork, &fStdDev, fTau);

    // Free the working memory
    free(fWork);

    // Calc & return *B1 = ratio of standard to Allan variance
    return (float)(SQR(fStdDev/fSig));
}

/*****************************************************************************/

