/***************************************************************************/
/*                                                                         */
/*					     CalcThreePointDrift()                             */
/*                                                                         */
/*		Function to calculate frequency drift using the first, middle      */
/*      and last phase data points.                                        */
/*                                                                         */
/*	                                                                       */
/*		Parameters: 	x[]		phase data array                           */
/*                                                                         */
/*		Return: 		float	frequency drift                            */
/*                                                                         */
/*		Notes:  This is the best estimator of drift for the combination    */
/*				of white and random walk FM noise (see reference).         */
/*				Tau=1 is assumed.   Drift is per tau interval.             */
/*				This function calls FrequenC functions CountGaps() and     */
/*				FillGaps().  To preserve gaps, save original phase data    */
/*			    before calling CalcThreePointDrift().                      */
/*                                                                         */
/* 		Reference:                                                         */
/* 		M. Weiss, D. Allan, and D. Howe, "Confidence on the Second Diff-   */
/*		erence Estimation of Frequency Drift", 1992 IEEE Frequency Control */
/*		Symposium, pp.300-305, June, 1992.                                 */
/*                                                                         */
/*		Revision record:                                                   */
/*			11/01/92	Created                                            */
/*			02/03/96	Removed unused variable: int i                     */
/*						Modified for use as Win 3.1 DLL                    */
/*			01/01/98	Changes for MS VC++ compatibility & warnings       */
/*                                                                         */
/*	(c) Copyright 1992-8  Hamilton Technical Services  All Rights Reserved */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"                              /* FrequenC header file */

float CalcThreePointDrift(F_TYPE x[])
{
	int start;                               /* first phase analysis point */
	int mid;                                /* middle phase analysis point */
	int end;                                  /* last phase analysis point */

	float n1, n2;              /* # analysis points in 1st and last halves */
	float n3;                        /* avg # analysis points in each half */
	float d;                           /* frequency drift per tau interval */

	/* fill gaps if necessary */
	if( CountGaps(x, PHASE) )                                 /* any gaps? */
	{
		FillGaps(x, PHASE);                     /* fill gaps in phase data */
	}                                                            /* end if */

	/* calculate drift */
	start = (int) x[1];                            /* first analysis point */
	mid = (int) ((x[2]-x[1])+2)/2;                /* middle analysis point */
	end = (int) x[2];                               /* last analysis point */
	n1 = (float) mid-start;                        /* # points in 1st half */
	n2 = (float) end-mid;                          /* # points in 2nd half */
	n3 = (n1+n2)/2;                                      /* drift interval */

	d = (float)((((x[end+2]-x[mid+2])/n2) - ((x[mid+2]-x[start+2])/n1))/n3);

	return(d);                                          /* frequency drift */
}                                             /* end CalcThreePointDrift() */

/***************************************************************************/
