/*****************************************************************************/
/*                                                                           */
/* 		                        EDF()                                        */
/*                                                                           */
/* 	Function to determine the estimated # of chi-squared degrees of freedom  */
/*  for the modified Allan variance (MVAR) of a power law noise process.     */
/*                                                                           */
/*  Based on edf() of STAB_F38.C from Stable/DOS                             */
/*                                                                           */
/*	Original:                                                                */
/*	Parameters:		int beta  = exponent of Sx(f) of power law noise         */
/*                              where: 0 = W                                */
/*                                    -1 = F                                */
/*                                    -2 = W  f                              */
/*                                    -3 = F  f	                             */
/*                                    -4 = RW f                              */
/*					int m     = averaging factor                             */
/*					float p   = M/m                                          */
/*							    where: M = # phase analysis points           */
/*										 = N-3m+1                            */
/*										   where: N = # phase data points    */
/*  New Params:		int nBeta = beta                                         */
/*                  int nAvgFactor = m                                       */
/*                  int nNum = M                                             */
/*                                                                           */
/*                  nNum and nAvgFactor are used to find p                   */
/*                                                                           */
/*	Return:         float edf = estimated degrees of freedom                 */
/*                                                                           */
/*	Note:			This function calculates the MVAR edf for the usual      */
/*					case of fully overlapping samples (m1=1 & r=m)           */
/*                                                                           */
/*	Reference:		C. H. Greenhall, "Estimating the Modified Allan          */
/*                  Variance", 1995 IEEE International Frequency Control     */
/*                  Symposium (to be published)                              */
/*				                                                             */
/*	Revision record:                                                         */
/*			01/01/98	Changes for MS VC++ compatibility & warnings         */
/*                                                                           */
/*	(c) Copyright 1996-8 Hamilton Technical Services    All Rights Reserved  */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float EDF(int nBeta, int nAvgFactor, int nNum)
{
	// Local variables
	// the a0 and a1 coefficients for estimating MVAR edf
	// are stored in the following static arrays
	//  varies from 0 to -4 ( - varies from 0 to 4)
	// m varies from 1 to >2 (m-1 varies from 0 to 2)

	// a0 term: a0[-][m-1]
	static float a0[5][3] =
	{
		{ 0.51429F, 0.93506F, 1.22450F },
		{ 0.57640F, 0.97339F, 1.00300F },
		{ 0.66667F, 1.01010F, 0.96774F },
		{ 0.81057F, 1.02660F, 0.94663F },
		{ 1.00000F, 0.86580F, 0.76791F }
	};

	// a1 term: a1[-][m-1]
	static float a1[5][3] =
	{
		{ 0.00000F, 0.00000F, 0.58929F },
		{ 0.00000F, 0.00000F, 0.60163F },
		{ 0.00000F, 0.00000F, 0.57124F },
		{ 0.00000F, 0.00000F, 0.41643F },
		{ 0.00000F, 0.00000F, 0.41115F }
	};

	// Convert args to those of original edf() function
	int beta=nBeta;
	int m=nAvgFactor;
	float p=(float)(nNum)/(float)nAvgFactor;

	// truncate m
	if(m>3)
	{
		m=3;
	}

	// calc and return edf
	return (a0[-beta][m-1]*p)/(1-(a1[-beta][m-1]/p));
}

/*****************************************************************************/
