/***************************************************************************/
/*                                                                         */
/*					    FindFreqOutliers()                                 */
/*                                                                         */
/*		Function to find and optionally remove outliers from freq data     */
/*                                                                         */
/*		Parameters:		F_TYPE	y[]			frequency data                 */
/*											y[0] = # data points           */
/*									 		y[1] = analysis start          */
/*									 		y[2] = analysis end            */
/*						float	sig_mult	sigma limit multiplier         */
/*						BOOL	remove_flag 0 = don't replace outliers     */
/*                                  		1 = replace outliers           */
/*                                                                         */
/*		Return:			int		# outliers                                 */
/*                                                                         */
/*      Notes:         	Outliers are those freq data points	that are more  */
/*						than  sig_mult	standard deviations from average.  */
/*						They are counted and optionally removed only       */
/*						between the analysis limits y[1] thru y[2].        */
/*						Replaced outliers become gaps = 0.                 */
/*						This function calls FrequenC functions CalcMean()  */
/*						and	CalcFreqStdDev().                              */
/*						A good default value for sig_mult is 5.0.          */
/*                                                                         */
/*		Revision record:                                                   */
/*			11/01/92	Created                                            */
/*			04/03/93	Added extern statements for called functions       */
/*			02/03/96	Modified for use as Win 3.1 DLL                    */
/*			01/01/98	Changes for MS VC++ compatibility & warnings       */
/*			                                                               */
/* (c) Copyright 1992-8  Hamilton Technical Services  All Rights Reserved  */
/*			                                                               */
/***************************************************************************/

#include "frequenc.h"                              /* FrequenC header file */

int FindFreqOutliers(F_TYPE y[],
	float sig_mult,	BOOL remove_flag)
{
	int i;                                                        /* index */
	int start, end;                         /* analysis start & end points */
	int count=0;                                             /* # outliers */
	F_TYPE avg;                                    /* average of freq data */
	F_TYPE dev;                                    /* std dev of freq data */
	F_TYPE high_lim, low_lim;                        /* limits for ok data */

	/* set analysis limits */
	start=(int)y[1];                               /* analysis start point */
	end=(int)y[2];                                   /* analysis end point */

	/* set up outlier limits */
	CalcMean(y, &avg, FREQ);                         /* find average value */
	CalcFreqStdDev(y, &dev);                    /* find standard deviation */

	high_lim=avg+(dev*sig_mult);                /* upper limit for ok data */
	low_lim =avg-(dev*sig_mult);                /* lower limit for ok data */

	/* search for outliers, counting and removing them as required */
	for(i=start+2; i<end+2; i++)          /* loop thru all analysis points */
	{
		if(y[i])                                              /* skip gaps */
		{
			if( (y[i]<low_lim) || (y[i]>high_lim) )            /* outlier? */
			{
				count++;                        /* increment outlier count */
				if(remove_flag)               /* replace outlier with gap? */
				{
					y[i]=0;                                       /* gap=0 */
				}                                                /* end if */
			}                                                    /* end if */
		}                                                        /* end if */
	}                                                           /* end for */

	return(count);                                           /* # outliers */

}                                                /* end FindFreqOutliers() */

/***************************************************************************/
