/***************************************************************************/
/*                                                                         */
/*                             FillGaps()                                  */
/*                                                                         */
/*      Function to fill gaps in data with interpolated values             */
/*      New version of FillGaps() from FrequenC library                    */
/*      Original version was very slow for long series of leading or       */
/*      trailing gaps, so created this new faster function as substitute   */
/*      This version also returns # leading (not total) gaps removed       */
/*                                                                         */
/*      Parameters:     F_TYPE z[]  = phase or frequency data              */
/*                                    z[0] = # data points                 */
/*                                    z[1] = analysis start                */
/*                                    z[2] = analysis end                  */
/*                      BOOL datype = data type: 0=phase, 1=frequency      */
/*                                                                         */
/*      Return:         int         = # leading gaps removed               */
/*                                                                         */
/*      Notes:          All zeros treated as gaps in frequency data.       */
/*                      Embedded zeros treated as gaps in phase data.      */
/*                      No adjustments made to timetags                    */
/*                      For freq data, all leading and trailing zeros      */
/*                      are removed.  For phase data, there is no way      */
/*                      to tell whether a leading or trailing zero is      */
/*                      a gap or a data point - we consider them to be     */
/*                      data points.  So, in removing a block of zeros     */
/*                      from the beginning or end of phase data, one       */
/*                      zero is kept as a data point.                      */
/*                                                                         */
/*      Revision record:                                                   */
/*          05/18/02    Created from FillGaps() of FrequenC library        */
/*                      Changed all FillGaps() calls to this function      */
/*                      which is much faster for removing leading and      */
/*                      trailing gap blocks.  The return value was changed */
/*                      to provide only the number of gaps removed from    */
/*                      the beginning of the file (those that affect       */
/*                      timetags.  Timetag adjustment was added to the     */
/*                      Fill function.                                     */
/*			03/15/03	Added to FrequenC DLL - Replaces FG.C (same func)  */
/*                                                                         */
/* (c) Copyright 1991-2003 Hamilton Technical Services All Rights Reserved */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

int FillGaps(F_TYPE z[], BOOL datype)
{
    // Local variables
    int g=0;                                            /* total gap count */
    int i;                                        /* general (outer) index */
    int i_start;               /* index of last data point at start of gap */
    int i_end;                   /* index of next data point at end of gap */
    int j;                                                  /* inner index */
    int k;                                              /* local gap count */
    int l=0;                                     /* # leading gaps removed */
    int n = (int)*z;                                      /* # data points */
    double increment;              /* freq increment for interpolated data */

    // Code to eliminate leading gaps
    // Set i according to data type to skip 1st point if phase data
    i=1-datype+3;

    // Is there a leading gap?
    if(!*(z+i))                                /* is 1st data point a gap? */
    {
        // Find out how many contiguous gaps there are
        k=0;
        while(!*(z+i+k))
        {
            k++;
        }                                                     /* end while */

        // Move up that many data points in one operation
        for(j=i; j<n+3-k; j++)        /* move all data up to remove gap(s) */
        {
            *(z+j) = *(z+j+k);
        }                                                       /* end for */
        g+=k;                             /* increment the total gap count */
        n-=k;                                 /* decrement the data counts */
        *z=n;                              /* adjust the # of array points */
        *(z+2)=*(z+2)-k;                /* adjust the analysis end point # */
            
        // Save # leading gaps removed
        l=k;
    }                                                            /* end if */

    // Code to elimninate trailing gaps
    // Set i to end of data according to data type
    i=(int)(*z)+1+datype;

    // Is there a trailing gap?
    if(!*(z+i))                               /* is last data point a gap? */
    {
        // Find out how many contiguous gaps there are
        k=0;
        while(!*(z+i-k))
        {
            k++;
        }                                                     /* end while */

        // No need to move data - just truncate analysis limits
        g+=k;                             /* increment the total gap count */
        n-=k;                                 /* decrement the data counts */
        *z=n;                              /* adjust the # of array points */
        *(z+2)=*(z+2)-k;                /* adjust the analysis end point # */
    }                                                            /* end if */

    // Loop to replace interior gaps with interpolated values
    k=0;                
    for(i=(int)(*(z+1)+3-datype); i<=(int)(*(z+2)+1+datype); i++)
    {
        if(!*(z+i))                                /* is data point a gap? */
        {
            k++;                          /* increment the local gap count */
        }                                                        /* end if */
        else
        {
            if(k)                                      /* are we at a gap? */
            {
                i_end = i;        /* save the index for the end of the gap */
                i_start = i-k-1;    /* save index for the start of the gap */
                increment = (*(z+i_end) - *(z+i_start)) / (k+1);
                for(j=1; j<=k; j++)
                {
                    *(z+i_start+j) = *(z+i_start) + increment*j;
                    g++;                        /* increment the gap count */
                }                                               /* end for */
            k=0;                        /* reinitialize count for next gap */
            }                                                    /* end if */
        }                                                      /* end else */
    }                                                           /* end for */

    // g=total # gaps removed - no longer the return value
    return(l);                            /* return # leading gaps removed */
}                                                     /* end NewFillGaps() */

/***************************************************************************/

