/****************************************************************************/
/*                                                                          */
/*                             Median Functions                             */
/*                                                                          */
/*      Functions to find median value in array.                            */
/*      FindMedian() calls FindMedian1() or FindMedian2()depending on       */
/*      array size.  FindMedian1() calls FindMed() for size <100.           */
/*      FindMedian2() calls nrselect() for size >=100.                      */
/*      Only top-level FindMedian() function is exported.                   */
/*                                                                          */
/*      Revision record:                                                    */
/*          03/18/03    File Created                                        */
/*                                                                          */
/* (c) Copyright 2003  Hamilton Technical Services  All Rights Reserved     */
/*                                                                          */
/****************************************************************************/

// Include FrequenC header file
#include "frequenc.h"

// Local wrapper function prototypes
int FindMedian1(F_TYPE fData[], BOOL bDataType, F_TYPE *pfMedian);
int FindMedian2(F_TYPE fData[], BOOL bDataType, F_TYPE *pfMedian);

/****************************************************************************/
/*                                                                          */
/*                               FindMedian()                               */
/*                                                                          */
/*      Function to call one of two FindMedian wrapper functions            */
/*                                                                          */
/*      Parameters:     F_TYPE fData     = data array                       */
/*                                         fData[0] = # data points         */
/*                                         fData[1] = analysis start        */
/*                                         fData[2] = analysis end          */
/*                                                                          */
/*                      BOOL bDataType   = data type flag: 0=phase, 1=freq  */
/*                                                                          */
/*                      F_TYPE *pfMedian = pointer to median                */
/*                                                                          */
/*      Returns:        int              = 0 if OK or -1 if error           */
/*                                                                          */
/*      Note:           One of two FindMedian wrapper functions is called   */
/*                      depending on size of data array                     */
/*                                                                          */
/*      Revision record:                                                    */
/*          05/07/02    Created                                             */
/*          03/18/03    Adapted for FrequenC.dll                            */
/*                                                                          */
/* (c) Copyright 2002-3  Hamilton Technical Services  All Rights Reserved   */
/*                                                                          */
/****************************************************************************/

int FindMedian(F_TYPE fData[],
    BOOL bDataType, F_TYPE *pfMedian)
{
    // Local variables
    int nSize;          // Size of data array
    int nCode;          // Error code

    // Find size of data array
    nSize=(int)(fData[2]-fData[1]+1);

    // Call one of two wrapper functions
    if(nSize<100)
    {
        nCode=FindMedian1(fData, bDataType, pfMedian);
    }
    else // Large array
    {
        nCode=FindMedian2(fData, bDataType, pfMedian);
    }

    // Done
    return nCode;
}

/****************************************************************************/
/*                                                                          */
/*                             FindMedian1()                                */
/*                                                                          */
/*      Wrapper function to safely call FindMed()for small array size       */
/*                                                                          */
/*      Parameters:     F_TYPE fData   = data array                         */
/*                                       fData[0] = # data points           */
/*                                       fData[1] = analysis start          */
/*                                       fData[2] = analysis end            */
/*                                                                          */
/*                      BOOL bDataType = data type flag: 0=phase, 1=freq    */
/*                                                                          */
/*                      F_TYPE *median = pointer to median                  */
/*                                                                          */
/*      Returns:        int            = 0 if OK or -1 if error             */
/*                                                                          */
/*      Note:           Embedded zeros treated as gaps in phase data.       */
/*                      All zeros gaps in frequency data.                   */
/*                      Must have at least 1 data point.                    */
/*                                                                          */
/*      Revision record:                                                    */
/*          09/14/98    Created                                             */
/*          03/18/03    Adapted for FrequenC.dll                            */
/*                                                                          */
/* (c) Copyright 1998-2003 Hamilton Technical Services All Rights Reserved  */
/*                                                                          */
/****************************************************************************/

// Prototype for local median function
int FindMed(F_TYPE z[], BOOL datype, F_TYPE *median);

int FindMedian1(F_TYPE fData[], BOOL bDataType, F_TYPE *pfMedian)
{
    // Local variables
    int i;              // General index
    int nNum;           // # data points in input array
    F_TYPE *fTemp;      // Pointer to temporary array to save original data

    // Save data in temporary array
    // Get array size
    nNum=(int)fData[0];

    // Allocate memory
    if((fTemp=(F_TYPE*)malloc((nNum+ARRAY_OFFSET)*sizeof(F_TYPE)))==NULL)
    {
        *pfMedian=0.0;
        return -1; // Error code
    }

    // Save data
    for(i=0; i<nNum+ARRAY_OFFSET; i++)
    {
        fTemp[i]=fData[i];
    }

    // Call FindMed()
    FindMed(fData, bDataType, pfMedian);

    // Restore data
    for(i=0; i<nNum+ARRAY_OFFSET; i++)
    {
        fData[i]=fTemp[i];
    }

    // Free temporary data array
    free(fTemp);

    // Done
    return 0; // OK
}

/****************************************************************************/
/*                                                                          */
/*                             FindMedian2()                                */
/*                                                                          */
/*      Wrapper function to safely call NR nrselect()                       */
/*      FindMed() gives strictly correct result for # odd or even           */
/*      Faster nrselect() used when # > 100                                 */
/*                                                                          */
/*      Parameters:     F_TYPE fData   = data array                         */
/*                                       fData[0] = # data points           */
/*                                       fData[1] = analysis start          */
/*                                       fData[2] = analysis end            */
/*                                                                          */
/*                      BOOL bDataType = data type flag: 0=phase, 1=freq    */
/*                                                                          */
/*                      F_TYPE *median = pointer to median                  */
/*                                                                          */
/*      Returns:        int            = 0 if OK or -1 if error             */
/*                                                                          */
/*      Note:           Embedded zeros treated as gaps in phase data.       */
/*                      All zeros gaps in frequency data.                   */
/*                      Must have at least 1 data point.                    */
/*                                                                          */
/*                      This function best used for large N, say N>1OO      */
/*                      It handles analysis limits & gaps.  It is X10       */
/*                      faster than full sort for large N.                  */
/*                                                                          */
/*      Revision record:                                                    */
/*          05/06/02    Created and working                                 */
/*          03/18/03    Adapted for FrequenC.dll                            */
/*                                                                          */
/* (c) Copyright 2002-3 Hamilton Technical Services All Rights Reserved     */
/*                                                                          */
/****************************************************************************/

// Prototype for NR select() function used in median calc
float nrselect(unsigned long k, unsigned long n, float arr[]);

int FindMedian2(F_TYPE fData[], BOOL bDataType, F_TYPE *pfMedian)
{
    // Local variables
    int i;              // General index
    int k;              // Secondary index
    int nStart;         // Analysis start point
    int nNum;           // # analysis points
    int nGaps=0;        // # gaps found
    float *fTemp;       // Pointer to temporary array for median calc

    // Copy analysis data into temporary working array using indices 1 to nNum
    // Get start # and # analysis points
    nStart=(int)fData[1];
    nNum=(int)(fData[2]-fData[1]+1);

    // Allocate memory for analysis data only - no header
    // The +1 allows the array index to start at 1 per NR style
    if((fTemp=(float*)malloc((nNum+1)*sizeof(float)))==NULL)
    {
        *pfMedian=0.0;
        return -1; // Error code
    }

    // Copy analysis data into temporary float array
    k=1;
    for(i=1; i<=nNum; i++)
    {
        if(bDataType==FREQ_DATA)
        {
            // All zeros are gaps to be omitted
            if((fTemp[k]=(float)fData[ARRAY_OFFSET+nStart+i-2])==0)
            {
                nGaps++;
            }
            else
            {
                k++;
            }
        }
        else // Phase data
        {
            // All except 1st & last zeros are gaps to be omitted
            if((fTemp[k]=(float)fData[ARRAY_OFFSET+nStart+i-2])==0)
            {
                if( (k=1) || (k=nNum) )
                {
                    nGaps++;
                }
                else
                {
                    k++;
                }
            }
            else
            {
                k++;
            }
        }
    }

    // Adjust # points
    nNum-=nGaps;
    
    // Call NR function to find median
    // 1st arg is kth smallest value to find = (N+1)/2
    // This is exactly correct median for N odd
    // For N even, strictly speaking, median S/B avg of N/2 and N/2 +1 points
    // We aren't going to worry about that, esp for large N
    *pfMedian=(F_TYPE)nrselect((unsigned long)((nNum+1)/2), (unsigned long)nNum,
        fTemp);

    // Free temporary data array
    free(fTemp);

    // Done
    return 0; // OK
}

/*****************************************************************************/
/*                                                                           */
/*                              FindMed()                                    */
/*                                                                           */
/*      Function to find median of array of phase or frequency data          */
/*                                                                           */
/*      Parameters:     F_TYPE z[]    = phase or frequency data              */
/*                                      z[0] = # data points                 */
/*                                      z[1] = analysis start                */
/*                                      z[2] = analysis end                  */
/*                      BOOL   datype = data type: 0=phase, 1=frequency      */
/*                      F_TYPE *median= pointer to median value of data      */
/*                                                                           */
/*      Notes:          All zeros treated as gaps in frequency data.         */
/*                      Embedded zeros treated as gaps in phase data.        */
/*                      This function first fills gaps in the data           */
/*                      with interpolated values by calling fill().          */
/*                      It then sorts the data into assending order,         */
/*                      using qsort(), to find the median value.             */
/*                      The median is the center value for an odd # of       */
/*                      data points, and the average of the two center       */
/*                      values for an even # of data points.                 */
/*                                                                           */
/*      Return:         int            = error flag: 0=OK, 1=NG              */
/*                                                                           */
/*      Revision record:                                                     */
/*          05/03/97    Adapted from FindMedian() of STABLE_A.C              */
/*          01/01/98    Changes for MS VC++ compatibility & warnings         */
/*          03/18/03    Adapted for FrequenC.dll                             */
/*                                                                           */
/* (c) Copyright 1992-2003 Hamilton Technical Services All Rights Reserved   */
/*                                                                           */
/*****************************************************************************/

// Declare comparison function - not exported */
static int median_compare(const void *a, const void *b);

int FindMed(F_TYPE z[], BOOL datype, F_TYPE *median)
{
    int i;                                                      /* 1st index */
    int j=0;                                                    /* 2nd index */
    int start=(int)z[1];                                   /* analysis start */
    int end=(int)z[2];                                       /* analysis end */
    int num=(int)z[0];                                      /* # data points */
    F_TYPE *temp;                 /* pointer to temporary data storage array */
    F_TYPE med;                                              /* median value */

    // save data
    if((temp=(F_TYPE *)malloc((num+3)*sizeof(F_TYPE)))==NULL)
                                    /* allocate temporary data storage space */
    {
        return(1);                                           /* error return */
    }                                                              /* end if */

    for(i=0; i<num+3; i++)
    {
        temp[i]=z[i];                                           /* save data */
    }                                                             /* end for */

    /* Fill any gaps in data */
    FillGaps(z, datype);

    /* eliminate non-analysis points */
    for(i=start+2; i<=end+2; i++)
    {
        z[j]=z[i];
        j++;
    }                                                             /* end for */
    num=end-start+1;

    /* sort data into numerical order */
    qsort((void *) z, (size_t) num, sizeof(F_TYPE), median_compare);

    /* find & return median */
    if(num%2)                                                 /* is num odd? */
    {
        med=z[num/2];                                        /* middle value */
    }                                                              /* end if */
    else
    {                                                           /* n is even */
        med=(z[(num/2)-1]+z[num/2])/2.0;       /* average of 2 middle values */
    }                                                            /* end else */

    // restore data
    for(i=0; i<num+3; i++)
    {
        z[i]=temp[i];                                        /* restore data */
    }                                                             /* end for */
    free(temp);                                         /* deallocate temp[] */

    *median=med;                       /* set median to med = median of data */
    return(0);                                              /* normal return */

}                                                        /* end FindMedian() */

/*****************************************************************************/
/*                                                                           */
/*                           median_compare()                                */
/*                                                                           */
/*      Qsort compare function for finding median of array                   */
/*                                                                           */
/*****************************************************************************/

static int median_compare(const void *a, const void *b)
{
    F_TYPE x =  (*(F_TYPE *) a) - (*(F_TYPE *) b) ;
    return (x>0) ? 1 : (x<0) ? -1 : 0;
}

/*****************************************************************************/
/*                                                                           */
/*                              nrselect()                                   */
/*                                                                           */
/*      Fast Numerical Recipes function to find median of array              */
/*                                                                           */
/*****************************************************************************/

// Copied from on-line NR, Sec. 8.5 (later version than I have in book)
// Note:  The NR working array arr[] indices go from 1 to n; arr[0] not used
// Note:  The nrselect() array is just a float, but this precision S/B OK 

// Local macro
#define SWAPP(a,b) temp=(a);(a)=(b);(b)=temp;

// Start of function - prototype is at top of this file - not in Stable_w.h
// This function returns the kth smallest value in the n-point array arr[]
// k is set to (n+1)/2 to find median
// About x10 faster than sorting - smaller elements moved in arbitrary order
float nrselect(unsigned long k, unsigned long n, float arr[])
{
    unsigned long i,ir,j,l,mid;
    float a,temp;
    
    l=1;
    ir=n;
    for(;;) {
        if (ir <= l+1) {
            if (ir == l+1 && arr[ir] < arr[l]) {
                SWAPP(arr[l],arr[ir])
            }
            return arr[k];
        } else {
            mid=(l+ir) >> 1;
            SWAPP(arr[mid],arr[l+1])
            if (arr[l] > arr[ir]) {
                SWAPP(arr[l],arr[ir])
            }
            if (arr[l+1] > arr[ir]) {
                SWAPP(arr[l+1],arr[ir])
            }
            if (arr[l] > arr[l+1]) {
                SWAPP(arr[l],arr[l+1])
            }
            i=l+1;
            j=ir;
            a=arr[l+1];
            for (;;) {
                do i++; while (arr[i] < a);
                do j--; while (arr[j] > a);
                if (j < i) break;
                SWAPP(arr[i],arr[j])
            }
            arr[l+1]=arr[j];
            arr[j]=a;
             if (j >=k) ir=j-1;
             if (j <=k) l=i;
        }
    }
}

/*****************************************************************************/

