/***************************************************************************/
/*                                                                         */
/*  							FindPlotScale()                            */
/*                                                                         */
/* 	    Function to find scale factor for plotting phase or freq data      */
/*                                                                         */
/*		Parameters:    F_TYPE min          = minimum freq value            */
/*      	           F_TYPE max          = maximum freq value            */
/*			           F_TYPE *p_scale_min = pointer to scale minimum      */
/*			           F_TYPE *p_scale_max = pointer to scale maximum      */
/*                                                                         */
/*      Return:        int = # tick intervals in scale,                    */
/*                           or -1 if error                                */
/*                                                                         */
/*		Revision record:                                                   */
/*			12/16/91   Created                                             */
/*			12/26/91   Changed to int returning # decades in scale         */
/*			12/31/91   Renamed                                             */
/*			01/04/91   Changed return value to # ticks                     */
/*					   Added 4 and 8 tick choices                          */
/*			01/05/92   Added range check & error code return if range NG   */
/*          01/15/92   Added 6 tick choice; changed to else if style       */
/*                     Changed to tighter range check logic at end         */
/*			01/16/92   (int) casts made floor() or ceil() to avoid O/F     */
/*					   Added e in fine tests; added 3 tick choice          */
/*                     Additional changes to range check/correction logic  */
/*			01/18/92   Edited title block                                  */
/*			01/22/92   Changed order of min and max parameter pairs        */
/*			02/17/92   Changed min & max params from pointers to values    */
/*			02/03/96   Modified for use as Win 3.1 DLL                     */
/*			01/01/98   Changes for MS VC++ compatibility & warnings        */
/*                                                                         */
/* (c) Copyright 1991-8  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

int FindPlotScale(F_TYPE min, F_TYPE max,
	F_TYPE *p_scale_min, F_TYPE *p_scale_max)
{
	double range, log_range, coarse, fine, ticks;
	double full_scale, digits, e;
	int k;                                                /* # ticks added */

	e=1e-6;                                             /* error tolerance */
	range = max - min;

	if(range<=0)                                     /* check for NG range */
	{
		*p_scale_min=0;
		*p_scale_max=0;
		return(-1);                                          /* error code */
	}

	log_range = log10(range) + 100;
	coarse = floor(log_range);
	fine = log_range - coarse;


	if(fine < log10(2)+e)
	{
		ticks = 2;
	}                                                            /* end if */
	else if(fine < log10(3)+e)
	{
    	ticks = 3;
	}                                                            /* end if */
	else if(fine < log10(4)+e)
	{
    	ticks = 4;
	}                                                            /* end if */
	else if(fine < log10(5)+e)
	{
		ticks = 5;
	}                                                            /* end if */
	else if(fine < log10(6)+e)
	{
		ticks = 6;
	}                                                            /* end if */
	else if(fine < log10(8)+e)
	{
		ticks = 8;
	}                                                            /* end if */
	else
	{
		ticks = 10;
	}                                                          /* end else */

	full_scale = pow(10, coarse - 100) * ticks;
	if(log10(full_scale) >= 0)
	{
		digits = floor(log10(full_scale));
	}                                                            /* end if */
	else
	{
		digits = ceil(log10(full_scale)) - 1;
	}                                                          /* end else */

	if((min / pow(10, digits)) >= 0)
	{
		*p_scale_min = floor(min / pow(10, digits)) * pow(10, digits);
	}                                                            /* end if */
	else
	{
		*p_scale_min = (floor(min / pow(10, digits))-1) * pow(10, digits);
	}                                                          /* end else */
	*p_scale_max = *p_scale_min + full_scale;

	k=0;

	if(*p_scale_max < max)                /* check for scale max too small */
	{
		while(*p_scale_max < max)                         /* move scale up */
		{
			*p_scale_max += full_scale/ticks;
			*p_scale_min += full_scale/ticks;
		}
		if(*p_scale_min > min)
		{
			while(*p_scale_min > min)           /* add tic interval to max */
			{
				*p_scale_min -= full_scale/ticks;
				k++;
			}
		}
	}

	else if(*p_scale_min > min)           /* check for scale min too large */
	{
		while(*p_scale_min > min)                       /* move scale down */
		{
			*p_scale_max -= full_scale/ticks;
			*p_scale_min -= full_scale/ticks;
		}
		if(*p_scale_max < max)                  /* add tic interval to min */
		{
			while( *p_scale_max < max)
			{
				*p_scale_max += full_scale/ticks;
				k++;
			}
		}
	}

	ticks+=k;                                    /* adjust # tic intervals */

	return(int)ticks;
}                                                   /* end FindPlotScale() */

/***************************************************************************/
