/*****************************************************************************/
/*                                                                           */
/*                             GreenhallDrift()                              */
/*                                                                           */
/*      Function to calculate the Greenhall frequency drift estimator        */
/*      for an array of phase data                                           */
/*                                                                           */
/*      Reference:                                                           */
/*      C.A. Greenhall,"A Frequency-Drift Estimator and Its Removal          */
/*      from Modified Allan Variance", Proc. 1997 IEEE Freq. Contrl.         */
/*      Symp., pp. 428-432, June 1997.                                       */
/*                                                                           */
/*      The idea is to integrate the phase data in the form of a             */
/*      cumulative sum to reduce the noise of the freq drift estimate        */
/*                                                                           */
/*      Parameters:     F_TYPE fData   = phase data array                    */
/*                                       fData[0] = # data points            */
/*                                       fData[1] = analysis start           */
/*                                       fData[2] = analysis end             */
/*                      F_TYPE fTau    = Sampling time of data (sec)         */
/*                      F_TYPE *fDrift = pointer to freq drift               */
/*                                                                           */
/*      Returns:        int            = 0 if OK or -1 if error              */
/*                                                                           */
/*      Notes:          Embedded zeros treated as gaps in phase data.        */
/*                      Must have at least 4 data points                     */
/*                      fDrift set to zero if error                          */
/*                                                                           */
/*      Revision record:                                                     */
/*          11/20/11    Created                                              */
/*          12/12/17	Edited for Linux                                     */
/*                                                                           */
/*                                                                           */
/* (c) Copyright 2011-2017 Hamilton Technical Services All Rights Reserved   */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int GreenhallDrift(F_TYPE fData[], F_TYPE fTau, F_TYPE *fDrift)
{
    // Local variables
    int i;              // General index
	int n;				// Estimator index
	double r;			// Estimator ratio
    int nNum;           // # analysis data points
	double fNum;		// nNum as double
	int nStart;			// # of first analysis point
	int iStart;			// Index of first analysis point
    F_TYPE *fTemp;      // Pointer to working cumulative sum array

	// Get # analysis points
    nNum=(int)fData[2]-(int)fData[1]+1;
	fNum=(double)nNum;

    // Allocate memory for temporary cumulative sum array
    if((fTemp=(F_TYPE*)malloc(nNum*sizeof(F_TYPE)))==NULL)
    {
        *fDrift=0.0;
        return -1; // Error code
    }

	// Set first value
	nStart=(int)fData[1];
	iStart=nStart+ARRAY_OFFSET-1;
	fTemp[0]=fData[iStart];

	// Form cumulative sum of data
	// Note:  Greenhall's w0 is zero
    for(i=1; i<nNum; i++)
    {
        fTemp[i]=fTemp[i-1] + fData[i+iStart];
    }

	// Choose n equal to nNum/10 (Greenhall's n1)
	n=nNum/10;

	// Find r equal to n/nNum (Greenhall's r1)
	r=(double)n/(double)nNum;

	// Find freq drift

	// Greenhall's 1st term
	*fDrift=6.0/(fNum*fNum*fNum*fTau*fTau*r*(1.0-r));

	// Multiplied by Greenhall's 2nd term
	*fDrift*=(fTemp[nNum-1]-((fTemp[nNum-n-1]-fTemp[n-1])/(1.0-2*r)));

    // Free temporary data array
    free(fTemp);

    // Done
    return 0; // OK
}

