/*****************************************************************************/
/*                                                                           */
/*                                   GN.C                                    */
/*                                                                           */
/* 		Noise Generation Functions                                          */
/*                                                                           */
/*      GenNoise() - was f_beta()                                                             */
/*      four1()                                                              */
/*      gasdev()                                                             */
/*      rand1()                                                              */
/*      realft()                                                             */
/*                                                                           */
/*      Notes: NR function prototypes local to this file - not exported      */
/*             GenerateNoise() top-level function still in Stable32          */
/*                                                                           */
/*		Revision record:                                                     */
/*			06/30/97	Replaced FAR PASCAL _export with WINAPI              */
/*			07/25/97	Change to HUGE usage for Win 32 compatibility        */
/*			12/24/97	Made changes for AOK BC 32 bit compliation           */
/*			01/16/98	Added ShortenExp() usage for MSC VC++                */
/*			06/16/02	Changed noise data generation floats to doubles      */
/*			03/15/03	Moved into FrequenC V 2.0 as GN.C                    */
/*						IT IS ESSENTIAL THAT THE COMPLIER BE SET TO DEFAULT  */
/*						NOT OPTIMIZED FOR SPEED FOR THESE FUNCTIONS OR THEY  */
/*						WON'T WORK FOR NOISE DATA SETS LARGER THAN 32        */ 
/*                                                                           */
/*  (c) 1996-2002 W. Riley Hamilton Technical Services  All Rights Reserved  */
/*                                                                           */
/*****************************************************************************/

// FrequenC header file
#include "frequenc.h"

// Local prototypes for Numerical Recipes functions
void four1(double *data, int nn, int isign);
float gasdev(int *idum);
float ran1(int *idum);
void realft(double *data, int n, int isign);

// Local defines
#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr
#define M1 259200L                                       /* made L 2/13/93 */
#define IA1 7141L                                                 /* ditto */
#define IC1 54773L                                                /* ditto */
#define RM1 (1.0/M1)
#define M2 134456L                                                /* ditto */
#define IA2 812                                                   /* ditto */
#define IC2 28411L                                                /* ditto */
#define RM2 (1.0/M2)
#define M3 243000L                                                /* ditto */
#define IA3 4561L                                                 /* ditto */
#define IC3 51349L                                                /* ditto */

/*****************************************************************************/
/*                                                                           */
/*                               GenNoise()                                  */
/*                                                                           */
/*	Function to generate simulated discrete power law noise Sy(f)=h()f^    */
/*                                                                           */
/*	Reference:  N.J. Kasdin and T. Walter, "Discrete Simulation of Power     */
/*  			Law Noise", 1992 IEEE Frequency Control Symposium.           */
/*                                                                           */
/*	Parameters:	int n_pts    = total # noise points to generate              */
/*				double x[]   = vector for generated noise                    */
/*				float Q_d    = variance of generated noise                   */
/*				float b      = noise slope,                                 */
/*				int *idum    = reinitialization flag for pseudo-random       */
/* 							   noise generator: idum<0 = reinitialize        */
/*				int add_flag = 0 to replace x data, 1 to add x data          */
/*                                                                           */
/*	Return:		int = # noise points generated, or -1 if general error       */
/*				or -2 if memory allocation error                             */
/*                                                                           */
/*	Notes:      (1) Uses discrete transfer function H(z)=(1-z^-1)^/2        */
/*					where Sx(f)=g()f^, g()=h()/2 and =-2             */
/*              (2) -40, and not necessarily an integer                   */
/*				(3) Q_d=g()/[2((2)^)(^(+1))]                         */
/*				(3) Uses gasdev() and realft() functions from                */
/*					Numerical Recipes in C.                                  */
/*				(4) NR arrays start at 1, but x[] starts at 0                */
/*				(5) n_pts must be a power of 2                               */
/*                                                                           */
/*	Revision record:                                                         */
/*				03/29/96  Copied from STAB_N02.C                             */
/*				09/22/96  Variable nn made long - fixes memory problem!      */
/*				02/07/98  HUGE keyword removed for this 32-bit version       */
/*				06/12/99  Added -2 error code for memory allocation error    */
/*				03/15/03  Adapted for FrequenC DLL was f_beta()              */
/*                                                                           */
/* 	W. Riley  (c) Hamilton Technical Services  All Rights Reserved           */
/*                                                                           */
/*****************************************************************************/

int GenNoise(int n_pts, double x[],
	float Q_d, float b,	int *idum, int add_flag)
{
	int i;
	long nn;
	double *hfb;
	double *wfb;
	double mhb, wr, wi;

											 	  /* check for NG parameters */
	if((Q_d<=0) || (n_pts<1) || (b<-4) || (b>0) )
	{
		return(-1);
	}


	if(n_pts%2)                          /* check that n_pts is a power of 2 */
	{
		return(-1);
	}

	nn=n_pts+n_pts;
	mhb=(double)(-b/2.0);
	Q_d=(float)sqrt(Q_d);                     /* find deviation of the noise */

	// Allocate memory for data arrays
	if((hfb=(double*)malloc((nn+1)*sizeof(double)))==NULL)
	{
		// Return error code
		return(-2);
	}

	if((wfb=(double*)malloc((nn+1)*sizeof(double)))==NULL)
	{
		// Free memory that was successfully allocated
		free(hfb);

		// Return error code
		return(-2);
	}


	hfb[1]=1.0;
	wfb[1]=Q_d*gasdev(idum);


	for(i=2; i<=n_pts; i++)
	{                                            /* generate hk coefficients */
		// Note: This is where prgm used to bomb for n_pts>4096
		// It is also as far as it gets 01/16/99 in release build with #>32
		// OK for debug build
		hfb[i]=hfb[i-1]*(mhb+(double)(i-2))/((double)(i-1));
		wfb[i]=Q_d*gasdev(idum);        /* fill wk sequence with white noise */
	}

	for(i=n_pts+1; i<=nn; i++)            /* pad ends of sequence with zeros */
	{
		hfb[i]=0.0;
		wfb[i]=0.0;
	}

	/* Perform the discrete Fourier transform */
	realft(hfb, n_pts, 1);
	realft(wfb, n_pts, 1);

	/* Multiply the two complex vectors */
	wfb[1]=wfb[1]*hfb[1];
	wfb[2]=wfb[2]*hfb[2];

	for(i=3; i<=nn; i+=2)
	{
		wr=wfb[i];
		wi=wfb[i+1];
		wfb[i]=wr*hfb[i]-wi*hfb[i+1];
		wfb[i+1]=wr*hfb[i+1]+wi*hfb[i];
	}

	realft(wfb, n_pts, -1);          /* inverse Fourier transform the result */

	if(!add_flag)
	{                                                /* clear x[] data array */
		for(i=0; i<n_pts; i++)
		{
			x[i]=0.0;
		}
	}

	for(i=1; i<=n_pts; i++)
	{                                         /* add the final result to x[] */
		x[i-1]+=wfb[i]/((float)n_pts);
	}

	/* clean up and return */
	free(hfb);
	free(wfb);

	return(n_pts);
}

/*****************************************************************************/
/*                                                                           */
/*                               four1()                                     */
/*                                                                           */
/*	Function to compute FFT                                                  */
/*                                                                           */
/*	Reference:  W.H. Press, et al, "Numerical Recipes in C"                  */
/*  			Cambridge University Press, 1988, ISBN 0-521-35465-X,        */
/*				Section 12.2.                                                */
/*                                                                           */
/*	Parameters:	double *data = complex data array                            */
/*				int nn           = array size                                */
/*				int isign        = control code                              */
/*                                                                           */
/*	Return:		None (void)                                                  */
/*                                                                           */
/*	Notes:      This function replaces data by its discrete Fourier          */
/*				transform if isign=1, or by nn times its inverse discrete    */
/*				Fourier transform if isign=-1.  data is a complex array      */
/*				of length nn, input as a real array data[1..2*nn].  nn       */
/*				must be an integer power of 2.                               */
/*                                                                           */
/*	Revision record:                                                         */
/*			03/29/96  Copied from STAB_N01.C                                 */
/*			02/07/09  HUGE keyword removed for this 32-bit version           */		
/*                                                                           */
/* 	W. Riley  (c) Hamilton Technical Services  All Rights Reserved           */
/*                                                                           */
/*****************************************************************************/

void four1(double *data, int nn, int isign)
{
	int n,mmax,m,j,istep,i;
	double wtemp,wr,wpr,wpi,wi,theta;
	double tempr,tempi;

	n=nn << 1;
	j=1;
	for (i=1;i<n;i+=2) {
		if (j > i) {
			SWAP(data[j],data[i]);
			SWAP(data[j+1],data[i+1]);
		}
		m=n >> 1;
		while (m >= 2 && j > m) {
			j -= m;
			m >>= 1;
		}
		j += m;
	}
	mmax=2;
	while (n > mmax) {
		istep=2*mmax;
		theta=6.28318530717959/(isign*mmax);
		wtemp=sin(0.5*theta);
		wpr = -2.0*wtemp*wtemp;
		wpi=sin(theta);
		wr=1.0;
		wi=0.0;
		for (m=1;m<mmax;m+=2) {
			for (i=m;i<=n;i+=istep) {
				j=i+mmax;
				tempr=(double)(wr*data[j]-wi*data[j+1]);
				tempi=(double)(wr*data[j+1]+wi*data[j]);
				data[j]=data[i]-tempr;
				data[j+1]=data[i+1]-tempi;
				data[i] += tempr;
				data[i+1] += tempi;
			}
			wr=(wtemp=wr)*wpr-wi*wpi+wr;
			wi=wi*wpr+wtemp*wpi+wi;
		}
		mmax=istep;
	}
}

#undef SWAP

/*****************************************************************************/
/*                                                                           */
/*                              gasdev()                                     */
/*                                                                           */
/*	Function to generate a random number (Gaussian deviate)                  */
/*                                                                           */
/*	Reference:  W.H. Press, et al, "Numerical Recipes in C"                  */
/*  			Cambridge University Press, 1988, ISBN 0-521-35465-X,        */
/*				Section 7.2.                                                 */
/*                                                                           */
/*	Parameters:	int *idum  = control integer                                 */
/*							 Set to negative value to intialize or           */
/*						     reinitilaize the sequence                       */
/*                                                                           */
/*	Return:		float Gaussian deviate                                       */
/*                                                                           */
/*	Notes:		Returns a normally distributed deviate with zero mean        */
/*				and unit variance.  Uses ran1() as source for uniform        */
/*				deviates.                                                    */
/*                                                                           */
/*	Revision record:                                                         */
/*				03/29/96  Copied from STAB_N03.C                             */
/*                                                                           */
/* 	W. Riley  (c) Hamilton Technical Services   All Rights Reserved          */
/*                                                                           */
/*****************************************************************************/

float gasdev(int *idum)
{
	static int iset=0;
	static float gset;
	float fac,r,v1,v2;

	if  (iset == 0) {
		do {
			v1=(float)(2.0*ran1(idum)-1.0);
			v2=(float)(2.0*ran1(idum)-1.0);
			r=v1*v1+v2*v2;
		} while (r >= 1.0 || r == 0.0);
		fac=(float)sqrt(-2.0*log(r)/r);
		gset=v1*fac;
		iset=1;
		return v2*fac;
	} else {
		iset=0;
		return gset;
	}
}

/*****************************************************************************/
/*                                                                           */
/*                                rand1()                                    */
/*                                                                           */
/*	Function to generate a random number between 0 and 1                     */
/*                                                                           */
/*	Reference:  W.H. Press, et al, "Numerical Recipes in C"                  */
/*  			Cambridge University Press, 1988, ISBN 0-521-35465-X,        */
/*				Section 7.1.                                                 */
/*                                                                           */
/*	Parameters:	int *idum = control integer                                  */
/*                                                                           */
/*	Return:		float random deviate		                                 */
/*                                                                           */
/*	Notes:		Returns a uniform random deviate between 0 and 1.            */
/*				Set idum to a negative value to initialize or                */
/*				reinitilaize the sequence.                                   */
/*                                                                           */
/*	Revision record:                                                         */
/*		03/29/96  Copied from STAB_N06.C                                     */
/*                                                                           */
/* 	W. Riley  (c) Hamilton Technical Services  ALL Rights Reserved           */
/*                                                                           */
/*****************************************************************************/

float ran1(int *idum)
{
	static long ix1,ix2,ix3;
	static float r[98];
	float temp;
	static int iff=0;
	int j;

	if (*idum < 0 || iff == 0) {
		iff=1;
		ix1=(IC1-(*idum)) % M1;
		ix1=(IA1*ix1+IC1) % M1;
		ix2=ix1 % M2;
		ix1=(IA1*ix1+IC1) % M1;
		ix3=ix1 % M3;
		for (j=1;j<=97;j++) {
			ix1=(IA1*ix1+IC1) % M1;
			ix2=(IA2*ix2+IC2) % M2;
			r[j]=(float)((ix1+ix2*RM2)*RM1);
		}
		*idum=1;
	}
	ix1=(IA1*ix1+IC1) % M1;
	ix2=(IA2*ix2+IC2) % M2;
	ix3=(IA3*ix3+IC3) % M3;
	j=1 + (int)((97*ix3)/M3);            /* cast to int added by WJR 2/13/93 */
	if (j > 97 || j < 1)
	{
		; // error
	}
	temp=r[j];
	r[j]=(float)((ix1+ix2*RM2)*RM1);
	return temp;
}

#undef M1
#undef IA1
#undef IC1
#undef RM1
#undef M2
#undef IA2
#undef IC2
#undef RM2
#undef M3
#undef IA3
#undef IC3

/*****************************************************************************/
/*                                                                           */
/*                              realft()                                     */
/*                                                                           */
/*	Function to calculate Fourier transform                                  */
/*                                                                           */
/*	Reference:  W.H. Press, et al, "Numerical Recipes in C"                  */
/*  			Cambridge University Press, 1988, ISBN 0-521-35465-X,        */
/*				Section 12.3.                                                */
/*                                                                           */
/*	Parameters:	double *data = real data array                               */
/*				int n 			 = half # data points                        */
/*				int isign        1=forward, -1=inverse transform             */
/*                                                                           */
/*	Return:		None (void)	                                                 */
/*                                                                           */
/*	Notes:		Calculates the Fourier trnsform of a set of 2n real data     */
/*				points.  Replaces data set with positive half of complex     */
/*				frequency components.  n must be a power of 2.  realft()     */
/*				calls four1().                                               */
/*                                                                           */
/*	Revision record:                                                         */
/*		03/29/96  Copied from STAB_N07.C                                     */
/*		02/07/98  HUGE keyword removed for this 32-bit version               */		
/*                                                                           */
/* 	W. Riley  (c) Hamilton Technical Services  ALL Rights Reserved           */
/*                                                                           */
/*****************************************************************************/

void realft(double *data, int n, int isign)
{
	int i,i1,i2,i3,i4,n2p3;
	double c1=0.5,c2,h1r,h1i,h2r,h2i;
	double wr,wi,wpr,wpi,wtemp,theta;

	theta=3.141592653589793/(double) n;
	if (isign == 1) {
		c2 = -0.5;
		four1(data,n,1);
	} else {
		c2=0.5;
		theta = -theta;
	}
	wtemp=sin(0.5*theta);
	wpr = -2.0*wtemp*wtemp;
	wpi=sin(theta);
	wr=1.0+wpr;
	wi=wpi;
	n2p3=2*n+3;
	for (i=2;i<=n/2;i++) {
		i4=1+(i3=n2p3-(i2=1+(i1=i+i-1)));
		h1r=c1*(data[i1]+data[i3]);
		h1i=c1*(data[i2]-data[i4]);
		h2r = -c2*(data[i2]+data[i4]);
		h2i=c2*(data[i1]-data[i3]);
		data[i1]=(double)(h1r+wr*h2r-wi*h2i);
		data[i2]=(double)(h1i+wr*h2i+wi*h2r);
		data[i3]=(double)(h1r-wr*h2r+wi*h2i);
		data[i4]=(double)(-h1i+wr*h2i+wi*h2r);
		wr=(wtemp=wr)*wpr-wi*wpi+wr;
		wi=wi*wpr+wtemp*wpi+wi;
	}
	if (isign == 1) {
		data[1] = (h1r=data[1])+data[2];
		data[2] = h1r-data[2];
	} else {
		data[1]=c1*((h1r=data[1])+data[2]);
		data[2]=c1*(h1r-data[2]);
		four1(data,n,-1);
	}
}

/*****************************************************************************/
