/***************************************************************************/
/*                                                                         */
/*                              HadamardEDF()                              */
/*                                                                         */
/*  Function to calc the estimated # of chi-squared degrees of freedom     */
/*  for the overlapping Hadamard variance of a power law noise process.    */
/*                                                                         */
/*      Parameters:    int N     = # phase data points                     */
/*                     int m     = averaging factor                        */
/*                     int b     = beta (-2 to -6)                         */
/*                                                                         */
/*      Return:        float     = Hadamard edf                            */
/*                                 or -1 if error                          */
/*                                                                         */
/*      Reference:     C. Greenhall, "Hadamard edf Algorithm", May 1999    */
/*                     (private communication via e-mail)                  */
/*                                                                         */
/*      Revision Record:                                                   */
/*          05/15/99   Created from hedf3.c per C. Greenhall suggestions   */
/*          12/07/17   Added MIN macro usage                               */
/*                                                                         */
/* (c) Copyright 1999 Hamilton Technical Services  All Rights Reserved     */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

// Local function prototypes - not exported
double Rx(double t, int b);
double Log(double x);

float HadamardEDF(int N, int m, int b)
{
    // Local macros
    #define JMAX 100                // Max # terms to calc as sum
    #define SUM 0                   // Calc using sum
    #define BOUND 1                 // Calc using upper bound
    #define SMALL 2                 // Calc using smaller sum

    // Local variables
    int M=N-3*m;                    // M=N-3m
    int j;                          // Index
    int jmax;                       // Max j for summation
    int m_prime;                    // Smaller sum m
    int c;                          // Calculation type (SUM, BOUND or SMALL)
    double a0;                      // edf upper bound 1st term
    double a1;                      // edf upper bound 2nd term
    double edfinv;                  // Inverse of Hadamard edf
    double r=0;                     // r(t)
    double r0;                      // r(0)
    double e;                       // edf summation term
    double t;                       // Argument of r(t)
    double p;                       // M/m

    // Check for allowed beta
    if(b<-6 || b>-2)
    {
        return -1;                  // Error code
    }

    // Check for enough points: N must be >3m
    if(M<=0)
    {
        return -1;                  // Error code
    }

    // Find jmax
    jmax=MIN(M, 3*m);

    switch(b)                       // Different calc for each noise type
    {
        case -2:                    // White FM
        {
            // Set r0
            r0=12;

            // Test jmax
            if(jmax<JMAX)           // Calc edf as sum
            {
                // Set calc type
                c=SUM;

                // Find sum
                for(j=1; j<=jmax; j++)
                {
                    t=(double)j/(double)m;
                    e=Rx(t, b);
                    r+=e*e*(1-((double)j/(double)M));
                }
            }
            else                    // Too many terms
            {
                // Test M
                if(M>=3*m)          // Calc edf as upper bound
                {
                    // Set calc type
                    c=BOUND;

                    // Set coefficients
                    a0=0.777778;  // 7/9
                    a1=0.5;
                }
                else                // Use smaller sum
                {
                    // Set calc type
                    c=SMALL;

                    // Find nearest integer to JMAX/p
                    p=(double)M/(double)m;
                    m_prime=(int)(((double)JMAX/p)+0.5);

                    // Calc sum using this m
                    jmax=MIN(JMAX, 3*m_prime);

                    // Find smaller sum
                    for(j=1; j<=jmax; j++)
                    {
                        t=(double)j/(double)m_prime;
                        e=Rx(t, b);
                        r+=e*e*(1-((double)j/(double)JMAX));
                    }
                }
            }
        }
        break;

        case -3:                    // Flicker FM
        {
            // Set r0
            r0=13.49604;            // 48*log(2)-18*log(3)

            // Test jmax
            if(jmax<JMAX)           // Calc edf as sum
            {
                // Set calc type
                c=SUM;

                // Find sum
                for(j=1; j<=jmax; j++)
                {
                    t=(double)j/(double)m;
                    e=Rx(t, b);
                    r+=e*e*(1-((double)j/(double)M));
                }
            }
            else                    // Too many terms
            {
                // Test M
                if(M>=3*m)          // Calc edf as upper bound
                {
                    // Set calc type
                    c=BOUND;
                    
                    // Set coefficients
                    a0=1.0;
                    a1=0.62;
                }
                else                // Use smaller sum
                {
                    c=SMALL;

                    // Find nearest integer to JMAX/p
                    p=(double)M/(double)m;
                    m_prime=(int)(((double)JMAX/p)+0.5);

                    // Calc sum using this m
                    jmax=MIN(JMAX, 3*m_prime);

                    // Find smaller sum
                    for(j=1; j<=jmax; j++)
                    {
                        t=(double)j/(double)m_prime;
                        e=Rx(t, b);
                        r+=e*e*(1-((double)j/(double)JMAX));
                    }
                }
            }
        }
        break;

        case -4:                    // Random Walk FM
        {
            // Set r0
            r0=12;

            // Test jmax
            if(jmax<JMAX)           // Calc edf as sum
            {
                // Set calc type
                c=SUM;

                // Find sum
                for(j=1; j<=jmax; j++)
                {
                    t=(double)j/(double)m;
                    e=Rx(t, b);
                    r+=e*e*(1-((double)j/(double)M));
                }
            }
            else                    // Too many terms
            {
                // Test M
                if(M>=3*m)          // Calc edf as upper bound
                {
                    // Set calc type
                    c=BOUND;

                    // Set coefficients
                    a0=1.033334;  // 31/30
                    a1=0.607143;  // 17/28
                }
                else                // Use smaller sum
                {
                    // Set calc type
                    c=SMALL;

                    // Find nearest integer to JMAX/p
                    p=(double)M/(double)m;
                    m_prime=(int)(((double)JMAX/p)+0.5);

                    // Calc sum using this m
                    jmax=MIN(JMAX, 3*m_prime);

                    // Find smaller sum
                    for(j=1; j<=jmax; j++)
                    {
                        t=(double)j/(double)m_prime;
                        e=Rx(t, b);
                        r+=e*e*(1-((double)j/(double)JMAX));
                    }
                }
            }
        }
        break;

        case -5:                    // Flicker Walk FM
        {
            // Set r0
            r0=44.89093;            // -192*log(2)+162*log(3)

            // Test jmax
            if(jmax<JMAX)           // Calc edf as sum
            {
                // Set calc type
                c=SUM;

                // Find sum
                for(j=1; j<=jmax; j++)
                {
                    t=(double)j/(double)m;
                    e=Rx(t, b);
                    r+=e*e*(1-((double)j/(double)M));
                }
            }
            else                    // Too many terms
            {
                // Test M
                if(M>=3*m)          // Calc edf as upper bound
                {
                    // Set calc type
                    c=BOUND;
                    
                    // Set coefficients
                    a0=1.06;
                    a1=0.53;
                }
                else                // Use smaller sum
                {
                    // Set calc type
                    c=SMALL;

                    // Find nearest integer to JMAX/p
                    p=(double)M/(double)m;
                    m_prime=(int)(((double)JMAX/p)+0.5);

                    // Calc sum using this m
                    jmax=MIN(JMAX, 3*m_prime);

                    // Find smaller sum
                    for(j=1; j<=jmax; j++)
                    {
                        t=(double)j/(double)m_prime;
                        e=Rx(t, b);
                        r+=e*e*(1-((double)j/(double)JMAX));
                    }
                }
            }
        }
        break;

        case -6:                    // Random Run FM
        {
            // Set r0
            r0=132;

            // Test jmax
            if(jmax<JMAX)           // Calc edf as sum
            {
                // Set calc type
                c=SUM;

                // Find sum
                for(j=1; j<=jmax; j++)
                {
                    t=(double)j/(double)m;
                    e=Rx(t, b);
                    r+=e*e*(1-((double)j/(double)M));
                }
            }
            else                    // Too many terms
            {
                // Test M
                if(M>=3*m)          // Calc edf as upper bound
                {
                    // Set calc type
                    c=BOUND;
                    
                    // Set coefficients
                    a0=1.30;
                    a1=0.54;
                }
                else                // Use smaller sum
                {
                    // Set calc type
                    c=SMALL;

                    // Find nearest integer to JMAX/p
                    p=(double)M/(double)m;
                    m_prime=(int)(((double)JMAX/p)+0.5);

                    // Calc sum using this m
                    jmax=MIN(JMAX, 3*m_prime);

                    // Find smaller sum
                    for(j=1; j<=jmax; j++)
                    {
                        t=(double)j/(double)m_prime;
                        e=Rx(t, b);
                        r+=e*e*(1-((double)j/(double)JMAX));
                    }
                }
            }
        }
        break;
    }

    switch(c)                       // Different way for each calc type
    {
        case SUM:                   // Calc using sum
        {
            r*=2.0;
            r+=r0*r0;
            edfinv=r/(M*r0*r0);
        }
        break;

        case BOUND:                 // Calc using upper bound
        {
            p=(double)M/(double)m;
            edfinv=(1/p)*(a0-(a1/p));
        }
        break;

        case SMALL:                 // Calc using smaller sum
        {
            r*=2.0;
            r+=r0*r0;
            edfinv=r/(JMAX*r0*r0);
        }
        break;
    }

    return (float)(1/edfinv);       // Return Hadamard edf
}

/***************************************************************************/
/*                                                                         */
/*                                 Rx()                                    */
/*                                                                         */
/*  Function to calculate Rx(t,b) for Hamamard edf                         */
/*                                                                         */
/*      Parameters:    double t  = time (>0)                               */
/*                     int b     = beta (-2 to -6)                         */
/*                                                                         */
/*      Return:        double    = Rx(t,b)                                 */
/*                                                                         */
/*      Revision Record:                                                   */
/*          05/15/99   Created                                             */
/*          12/07/17   Added default                                       */
/*                                                                         */
/* (c) Copyright 1999 Hamilton Technical Services  All Rights Reserved     */
/*                                                                         */
/***************************************************************************/

double Rx(double t, int b)
{
    switch(b)       // Different expression for each noise type
    {
        case -2:    // White FM: Rx(t)=-|t|
        {
            return -20*t+15*(t+1)+15*fabs(t-1)-6*(t+2)-6*fabs(t-2)+
                (t+3)+fabs(t-3);
        }

        case -3:    // Flicker FM: Rx(t)=t*t*ln(|t|)
        {
            return 20*t*t*log(t)-15*(t+1)*(t+1)*Log(t+1)-
                15*(t-1)*(t-1)*Log(fabs((t-1)))+
                6*(t+2)*(t+2)*Log((t+2))+
                6*(t-2)*(t-2)*Log(fabs((t-2)))-
                (t+3)*(t+3)*Log((t+3))-(t-3)*(t-3)*Log(fabs((t-3)));
        }

        case -4:    // Random Walk FM: Rx(t)=-|t|^3
        {
            return 20*t*t*t-
                15*(t+1)*(t+1)*(t+1)-
                15*fabs((t-1)*(t-1)*(t-1))+
                6*(t+2)*(t+2)*(t+2)+
                6*fabs((t-2)*(t-2)*(t-2))-
                (t+3)*(t+3)*(t+3)-
                fabs((t-3)*(t-3)*(t-3));
        }

        case -5:    // Flicker Walk FM: Rx(t)=-t*t*t*t*ln(|t|)
        {
            return -20*t*t*t*t*log((t))+
                15*(t+1)*(t+1)*(t+1)*(t+1)*Log((t+1))+
                15*(t-1)*(t-1)*(t-1)*(t-1)*Log(fabs((t-1)))-
                6*(t+2)*(t+2)*(t+2)*(t+2)*Log((t+2))-
                6*(t-2)*(t-2)*(t-2)*(t-2)*Log(fabs((t-2)))+
                (t+3)*(t+3)*(t+3)*(t+3)*Log((t+3))+
                (t-3)*(t-3)*(t-3)*(t-3)*Log(fabs((t-3)));
        }

        case -6:    // Random Run FM: Rx(t)=-|t|^5
        {
            return -20*t*t*t*t*t+
                15*(t+1)*(t+1)*(t+1)*(t+1)*(t+1)+
                15*fabs((t-1)*(t-1)*(t-1)*(t-1)*(t-1))-
                6*(t+2)*(t+2)*(t+2)*(t+2)*(t+2)-
                6*fabs((t-2)*(t-2)*(t-2)*(t-2)*(t-2))+
                (t+3)*(t+3)*(t+3)*(t+3)*(t+3)+
                fabs((t-3)*(t-3)*(t-3)*(t-3)*(t-3));
        }
        
        default:	// Error
        {
			return 0.0;
		}
    }
}

/***************************************************************************/
/*                                                                         */
/*                                Log()                                    */
/*                                                                         */
/*  Wrapper function for log(x) modified to return 0 for x=0               */
/*                                                                         */
/*      Parameters:    double x  = argument for log(x)                     */
/*                                                                         */
/*      Return:        double    = log(x)                                  */
/*                                 or 0 if x=0                             */
/*                                                                         */
/*      Note: x can be 0 but must not be negative                          */
/*                                                                         */
/*      Revision Record:                                                   */
/*          05/10/99   Created                                             */
/*                                                                         */
/* (c) Copyright 1999 Hamilton Technical Services  All Rights Reserved     */
/*                                                                         */
/***************************************************************************/

double Log(double x)
{
    if(x)
    {
        return log(x);
    }
    else
    {
        return 0;
    }
}

/*****************************************************************************/

