/*****************************************************************************/
/*                                                                           */
/*                             HadTotvarBias()                               */
/*                                                                           */
/*      Function to calculate the Hadamard Totvar bias function              */
/*                                                                           */
/*      Parameters:     int   nAlpha = noise type (-4 to 0)                  */
/*                                                                           */
/*                                                                           */
/*      Return:         float fBias  = Mod Totvar bias factor                */
/*                                     or -1 if error                        */
/*                                                                           */
/*      Note:  fBias is independent of T and tau, and depends only on the    */
/*             the noise type.  It is defined as a=[E{TotHvar}/E{Hvar}]-1,   */
/*             but is calculated and used here as simply the ratio.          */
/*             This fBias is exactly the same as that for MTOT, but a        */
/*             separate function is used just in case that should change,    */
/*             and because of the diffeent alphas that apply.                */
/*                                                                           */
/*      Alpha     Noise        a    HTOT/HVAR = 1+a                          */
/*        0        W FM     -0.005      0.995                                 */
/*       -1        F FM     -0.149      0.851                                 */
/*       -2       RW FM     -0.229      0.771                                 */
/*       -3       FW FM     -0.283      0.717                                 */
/*       -4       RR FM     -0.321      0.679                                 */
/*                                                                           */
/*      The Hadamard Totvar bias factor is the ratio of the expected value   */
/*      HTOT to HVAR.  To correct a calculated Hadamard Totdev value for a   */
/*      certain noise type, divide the uncorrected Mod Totdev value by the   */
/*      square root of the bias factor returned by this function.            */
/*                                                                           */
/*      Notes:          (1) This function applies to -4 >= alpha >= 0        */
/*                      (2) This function applies for all T/tau              */
/*                                                                           */
/*      References:     (1) E-mail from D.A. Howe/NIST,  10/27/00            */
/*                      (2) Final draft of PTTI 2000 HTOT paper:             */
/*                          Final3-PTTI-00.pdf rcv'd 12/24/00                */
/*                                                                           */
/*      Revision record:                                                     */
/*          10/28/00    Created                                              */
/*          12/24/00    Revise coefficients per Ref. 2                       */
/*                                                                           */
/* (c) Copyright 2000   Hamilton Technical Services   All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float HadTotvarBias(int nAlpha)
{
    // Local variable
    float fBias;

    // Verify parameter
    if(nAlpha<-4 || nAlpha>0)
    {
        return (float) -1.0; // Error
    }

    // Sort by noise type
    switch(nAlpha)
    {
        case  0: // W FM
        {
            fBias=(float) 0.995;
        }
        break;

        case -1: // F FM
        {
            fBias=(float) 0.851;
        }
        break;

        case -2: // RW FM
        {
            fBias=(float) 0.771;
        }
        break;

        case -3: // FW FM
        {
            fBias=(float) 0.717;
        }
        break;

        case -4: // RR FM
        {
            fBias=(float) 0.679;
        }
        break;
    }

    return (fBias);
}

/*****************************************************************************/

