/*****************************************************************************/
/*                                                                           */
/*                               HadTotvarCalc()                             */
/*                                                                           */
/*      Function to calculate the Hadamard Total Variance from freq data.    */
/*      Uses uninverted even reflection of offset-removed freq data.         */
/*      This function uses fast inner loop algorithm.                        */
/*                                                                           */
/*      Parameters:     F_TYPE y[]     = freq data                           */
/*                                        y[0] = # data points               */
/*                                        y[1] = analysis start              */
/*                                        y[2] = analysis end                */
/*                      F_TYPE *fHTDev = pointer to Hadamard Totdev          */
/*                      int nAF        = averaging factor                    */
/*                      BOOL *bAbort   = pointer to abort flag               */                                                                     
/*                      BOOL bProgress = progress indicator flag             */                                                                     
/*                      void *ProgressFunction()                             */
/*                                     = pointer to progress indicator       */
/*                                       function with args int nPercentDone */
/*                                       and char *szMessage                 */
/*                                                                           */
/*      Return:         int            = # Hadamard Totvar analysis points,  */
/*                                       or -1 if memory alloc error         */
/*                                       or -2 if no result error            */
/*                                                                           */
/*      Notes:          1. Calculation is done entirely with a new array     */
/*                         that is deleted after this function closes.       */
/*                      2. Progress indicator is optional.                   */
/*                      3. This is fast method, and it handles gaps.         */
/*                                                                           */
/*      Overview:       Similar to mod totvar calc except freq instead of    */
/*                      phase data.  Uses 3m-point subsequence from which    */
/*                      linear freq drift is removed before being doubly     */
/*                      extended by uninverted, even reflection at each end. */
/*                      No tau argument needed.  Different final scaling.    */
/*                                                                           */
/*      Description:    Given the freq data array y[], Had Totvar is         */
/*                      calculated using z[], which is computed from a       */
/*                      subsequence of y[] comprising 3m points, where m     */
/*                      the avg factor nAF.  This subsequence is formed      */
/*                      from y[i] where i=n thru n+3m-1, where n is an       */
/*                      overall index that goes from 1 to the # of freq      */
/*                      data points nNum.  The subsequence z[i] is found     */
/*                      from y[i] by removing the frequency drift by         */
/*                      subtracting the slope found by averaging the 1st     */
/*                      and last halves of the subsequence and dividing by   */
/*                      half the interval.                                   */
/*                                                                           */
/*                      This subsequence is then extended at both ends by    */
/*                      an uninverted even reflection according to z(n-k)=   */
/*                      z(n+k-1) and z(n+3m+k-1)=z(n+3m-k) for k=1 thru 3m,  */
/*                      thus creating a new subsequence having a tripled     */
/*                      range from i=n-3m to i=n+6m-1.                       */
/*                                                                           */
/*                      Had Totvar is then calculated as the average of its  */
/*                      nNum-3m Totvar sub-estimates.                        */
/*                                                                           */
/*      Reference:      D. Howe, et al, "A Total Estimator of the Hadamard   */
/*                      Function Used for GPS Operations", Proc. 32th PTTI   */
/*                      Meeting, November 2000 (to be published).            */
/*                                                                           */
/*      Revision record:                                                     */
/*          10/28/00    Cloned from NewModTotvarCalc()                       */
/*          10/29/00    Running                                              */
/*          03/09/03    Made progress function and abprt flag arguments      */
/*          03/14/03    Moved into FrequenC DLL                              */
/*						Replaced GlobalAllocPtr() with malloc(), etc.        */
/*          12/07/17	Note: Could remove progress function for Linux       */
/*                                                                           */
/* (c) Copyright 2000-2017 Hamilton Technical Services All Rights Reserved   */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int HadTotvarCalc(F_TYPE fY[], F_TYPE *fHTDev,
    int nAF, BOOL *bAbort, BOOL bProgress,
    void *ProgressFunction(int nPercentDone, char *szMessage))
{
    // Local variables
    int i;              // Main index
    int j;              // Auxilary index
    int k;              // Inner index
    int m;              // Alias for averaging factor nAF
    int nNum;           // # points
    int nStart;         // Analysis start
    int nCount=0;       // # analysis points
    int nTic;           // Increment for progress indicator
    int nLast_i=0;      // Previous value of index

    double *fZ;         // Subsequence array pointer
    double fC1;         // Subsequence 1st half average
    double fC2;         // Subsequence 2nd half average
    double fC;          // Freq slope (drift)
    double fSum=0;      // Overall Had Totvar summation
    double fSubsum=0;   // Subsequence Had Totvar summation
    double fTemp;       // Temporary data value

    double fZ1;         // 1st freq average
    double fZ2;         // 2nd freq average
    double fZ3;         // 3rd freq average
 
    // Initializations
    nStart=(int)fY[1];          // Analysis start point
    nNum=(int)(fY[2]-fY[1]+1);  // # analysis freq data points = N
    m=nAF;                      // Averaging factor

    nTic=(int)ceil(nNum/100);   // Increment for progress report

    // Set fTemp to mean value of overall freq data
    CalcMean(fY, &fTemp, FREQ_DATA);

    // Start
    if(bProgress)
    {
        // Label progress bar
        ProgressFunction(0, "Calculating Hadamard Totdev"); 
    }

    // Allocate 9m points of fZ[] working memory with index from 0 to 9m-1
    // w/o array header (#, Start, End)
    if((fZ=(double*)malloc((9*m)*sizeof(double)))==NULL)
    {
        return(-1);  // Error - memory reallocation failed
    }

    // Outer loop thru all nNum-3m+1 subsequences of 3m freq data points
    // A subsequence will be processed during each iteration
    // and the average used to find Had-Totvar
    // This allows a max m of nNum/3

    // To handle analysis limits, initialize index to nStart-1 not 0
    // and end it before nStart-1-3*m+1 instead of nNum-3*m+1
    for(i=nStart-1; i<nStart-1+nNum-3*m+1; i++)
    {
        // Copy data subsequence into working array starting at index 3m
        // Note:  The fY[] array needs offset for its header info
        for(j=0; j<3*m; j++)
        {
            // Test for gap (zero value)
            if(fY[i+j+ARRAY_OFFSET])
            {
                fZ[3*m+j]=fY[i+j+ARRAY_OFFSET];
                fTemp=fZ[3*m+j];
            }
            else 
            {
                // Replace gap with interpolated value
                // if adjacent points are themselves not gaps
                if( (fY[i+j+ARRAY_OFFSET-1]) && (fY[i+j+ARRAY_OFFSET+1]) )
                {
                    fZ[3*m+j]=
                        (fY[i+j+ARRAY_OFFSET-1]+fY[i+j+ARRAY_OFFSET+1])/2.0;
                }
                else
                {
                    // For lack of a better idea,
                    // replace gap with last OK value
                    // or, if haven't found one, the overall mean value
                    fZ[3*m+j]=fTemp;
                }
            }
        }
    
        // Do removal of frequency drift
        // Find slope fC by averaging the 1st and last halves of the subsequence
        // subtracting them, and dividing by half the interval
        fC1=0;
        fC2=0;
        for(j=0; j<3*m/2; j++)
        {
            fC1+=fZ[3*m+j];
            fC2+=fZ[3*m+(int)ceil(3.0*(double)m/2.0)+j];
        }
        fC1/=(double)(3*m/2);
        fC2/=(double)(3*m/2);
        fC=(fC2-fC1)/ceil(3.0*(double)m/2.0);

        // Then subtract slope from each point
        for(j=0; j<3*m; j++)
        {
            fZ[3*m+j]-=fC*(double)j;
        }
                
        // Do uninverted even reflection at bottom and top
        for(j=0; j<3*m; j++)
        {
            // Bottom reflection
            fZ[j]=fZ[6*m-1-j];
    
            // Top reflection
            fZ[6*m+j]=fZ[j];
        }

        // Calc Hvar for the fZ[] subsequence
        // Will need to add gap testing and analysis limits
        // This data array does not need an offset
        // This code implements the summation of Eq(4) of the reference

        // Initialize inner sum
        fSubsum=0;

        // We have a doubly-extended subsequence fZ[] of 9m points
        // and are going to form m-point averages
        // in 6m+1 groups of 3 (called fZ1, fZ2 and fZ3)
        // from which all possible fully overlapping 2nd differences
        // will be used to calculate Hadamard Totvar
        // Note: Only 6m of the 6m+1 possible 2nd differences are used
        // because the last one is a duplicate
        // This implements Eq. 1 of the reference
        for(j=0; j<6*m; j++)
        {
            if(j==0)
            {
                // Find the 3 freq averages for the first 2nd difference
                // Initialize freq averages
                fZ1=0.0;
                fZ2=0.0;
                fZ3=0.0;

                // Sum freq averages for 1st 3 sets of m points
                // using points 0 thru m-1, m thru 2m-1 and 2m thru 3m-1
                // Note: These freq sums are divided by m = # points of each
                // sum in the 2nd difference expression below
                for(k=0; k<m; k++)
                {
                    fZ1 += fZ[k];
                    fZ2 += fZ[k+m];
                    fZ3 += fZ[k+2*m];
                }
            }
            else
            {
                // Now do the rest of the points w/o summing all each time
                // Just subtract the old point and add the new one
                // Sum freq averages for 3 sets of m points
                // using points j thru j+m-1, j+m thru j+2m-1 and
                // j+2m thru j+3m-1
                // Note: These freq sums are divided by m = # points of
                // each sum in the 2nd difference expression below
                fZ1 = fZ1 -fZ[j-1]     +fZ[j+m-1];
                fZ2 = fZ2 -fZ[j+m-1]   +fZ[j+2*m-1];
                fZ3 = fZ3 -fZ[j+2*m-1] +fZ[j+3*m-1];
            }

            // Calc 2nd diff of one of the rest of the 6m sets of 3 freq avgs
            fSubsum += SQR((fZ1 - 2*fZ2 + fZ3)/m);
        }

        // Scale the subsum and add it to overall sum
        fSum+=fSubsum/(6*m);

        // Increment the sum count
        nCount++;

        // Display progress
        // Update when index has changed by at least tic amount
        if(((i-nLast_i)>=nTic) && (bProgress))
        {
            ProgressFunction((int)(0.5+100.0*i/(nNum-3*m)),
                "Calculating Hadamard Totdev");
            nLast_i=i;
        }

        // Check for abort
        if(*bAbort)
        {
            break;
        }
   }

    // Free the subsequence array
    free(fZ);

    // Scale result - See Eq (4) of Reference
    // Without gaps, nCount=nNum-3m
    // Note: Divisor S/B 6
    // But 3 gives result that agrees exactly with normal and overlapping
    // Hadamard deviation for AF=1 for FREQ.DAT test data
    // 6 OK for other AFs
    // Results agree exactly with T.Peppler/NIST with 6 for all AFs
    // So 6 is clearly right - low result for AF=1 is apparently a property of HTOT
    fSum/=(6*nCount);

    // Find Hadamard Totdev
    if(nCount && fSum>0)
    {
        *fHTDev=(F_TYPE)sqrt(fSum);
    }
    else // No result error
    {
        *fHTDev=0.0;
        nCount=-2;
    }

    // For testing
    // TRACE_INT(nAF);
    // TRACE_EXP(*fHTDev);
    
    // Done
    // Clear progress display
    ProgressFunction(0, "");

    return(nCount);  // Return # freq analysis points
}

/*****************************************************************************/

