/*****************************************************************************/
/*                                                                           */
/*                             HadTotvarEDF()                                */
/*                                                                           */
/*  Function to determine the estimated # of chi-squared degrees of freedom  */
/*  for the Hadamard total variance (HTOT) of a power law noise process.     */
/*                                                                           */
/*  Params:         int   nAlpha = alpha                                     */
/*                  float fRatio = T/tau = N/m                               */
/*                                 where: N = # phase data points            */
/*                                        m = avg factor = tau/tau0          */
/*                                                                           */
/*  Return:         float edf = estimated degrees of freedom                 */
/*                              or -1 if error                               */
/*                                                                           */
/*  The edf is modeled by the expression (T/tau)/[b0+b1(tau/T)], where:      */
/*                                                                           */
/*  Alpha    Noise     b0      b1       Values per D. Howe, et.al            */
/*   0        W FM    0.559   1.004     12/24/00 draft of PTTI'00 paper      */
/*  -1        F FM    0.868   1.140                                          */
/*  -2       RW FM    0.938   1.696                                          */
/*  -3       FW FM    0.974   2.554                                          */
/*  -4       RR FM    1.276   3.149                                          */
/*                                                                           */
/*  Notes:          (1) This function applies only to -4 >= alpha >= 0       */
/*                  (2) This function applies only for m > 16                */
/*                  (3) T is defined as tau0 * # phase data points           */
/*                                                                           */
/*  References:     (1) E-Mail from D.A. Howe/NIST, 10/27/00                 */
/*                  (2) Final draft of PTTI 2000 HTOT paper:                 */
/*                      Final3-PTTI-00.pdf rcv'd 12/24/00                    */
/*                                                                           */
/*  Revision record:                                                         */
/*      10/28/00    Created                                                  */
/*      12/24/00    Revised per Ref. 2                                       */
/*                                                                           */
/* (c) Copyright 2000  Hamilton Technical Services   All Rights Reserved     */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float HadTotvarEDF(int nAlpha, float fRatio)
{
    // Local variables
    float fEDF;
    double b0;
    double b1;

    // Verify parameters
    if(nAlpha<-4 || nAlpha>0)
    {
        return -1.0; // Error
    }

    switch(nAlpha)
    {
        case 0: // W FM noise
        {
            b0=0.559;
            b1=1.004;
        }
        break;

        case -1: // F FM noise
        {
            b0=0.868;
            b1=1.140;
        }
        break;

        case -2: // RW FM noise
        {
            b0=0.938;
            b1=1.696;
        }
        break;

        case -3: // FW FM noise
        {
            b0=0.974;
            b1=2.554;
        }
        break;

        case -4: // RR FM noise
        {
            b0=1.276;
            b1=3.149;
        }
        break;
    }

    fEDF=(float)((fRatio)/(b0+(b1/fRatio)));
    return fEDF;
}

/*****************************************************************************/

