/*****************************************************************************/
/*                                                                           */
/*  						   MedDev()                                      */
/*                                                                           */
/*		Function to calc median of absolute deviation from freqency data     */
/*		Based on Stable/DOS function F37 of STAB_F37.C                       */
/*                                                                           */
/*      meddev = med { | x - Median | / 0.6745 }                             */
/*                                                                           */
/*      Procedure for calculating meddev:                                    */
/*      	1. Sort data                                                     */
/*          2. Find Median of data                                           */
/*          3. Subtract Median from all data points                          */
/*          4. Calculate absolute value of these differences                 */
/*			5. Divide values by 0.6745 (This makes the expected value of     */
/*			   meddev = the std dev for normally distributed data.)          */
/*		    6. Resort data                                                   */
/*          7. Find median value = meddev                                    */
/*      These operations are performed in-place on a working data array t[]  */
/*                                                                           */
/*		Parameters:                                                          */
/* 			y[]   = frequency data; y[0] = # data points                     */
/*			y[1]  = analysis start; y[2] = analysis end                      */
/*          p_mad = pointer to median absolute deviation                     */
/*			zero freq value treated as gap in data	      				     */
/*		Returns num = # analysis points (including filled gaps)              */
/*                                                                           */
/*      References: (1) Gernot M.R. Winkler, "Introduction to Robust         */
/*                      Statistics and Data Filtering".                      */
/*                  (2) D.B. Percival, "Use of Robust Statistical            */
/*                      Techniques in Time Scale Formation".                 */
/*                                                                           */
/*		Revision record:                                                     */
/*			01/14/95	Created                                              */
/*			01/15/95    Added 0.6745 factor for unbiased estimate            */
/*			04/30/96    Modified as MedDev() for Stable/Win                  */
/*			01/01/98	Changes for MS VC++ compatibility & warnings         */
/*                                                                           */
/*	(c) Copyright 1995-8  Hamilton Technical Services  All Rights Reserved   */																		     
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

// Declare comparison function - not exported
static int MedDevCompare(const void *a, const void *b);


int MedDev(F_TYPE y[], float *p_mad)
{
	int i;                                                      /* 1st index */
	int j=0;                                                    /* 2nd index */
	int start=(int)y[1];                                   /* analysis start */
	int end=(int)y[2];                                       /* analysis end */
	int num=(int)y[0];                                      /* # data points */
	F_TYPE *t;                    /* pointer to temporary data storage array */
	F_TYPE med;                                              /* median value */

	// Allocate storage for temporary array
	if((t=(F_TYPE *)malloc((num+3)*sizeof(F_TYPE)))==NULL)
									/* allocate temporary data storage space */
	{
		return(1);                                           /* error return */
	}                                                              /* end if */

	// Copy data
	for(i=0; i<num+3; i++)
	{
		t[i]=y[i];                                              /* copy data */
	}                                                             /* end for */

	// Fill any gaps in data
	if(CountGaps(t, FREQ))
	{
		FillGaps(t, FREQ);                                      /* fill gaps */
	}                                                              /* end if */

	// Eliminate non-analysis points
	for(i=start+2; i<=end+2; i++)
	{
		t[j]=t[i];
		j++;
	}                                                             /* end for */
	num=end-start+1;

	// Sort data into numerical order
	qsort((void *)t, (size_t) num, sizeof(F_TYPE), MedDevCompare);

	// Find median
	if(num%2)                                                 /* is num odd? */
	{
		med=t[num/2];                                        /* middle value */
	}                                                              /* end if */
	else
	{                                                           /* n is even */
		med=(t[(num/2)-1]+t[num/2])/2.0;       /* average of 2 middle values */
	}                                                            /* end else */

	// Subtract median, get absolute value and scale data points
	for(i=0; i<num; i++)
	{
		t[i]-=med;
		t[i]=fabs(t[i])/0.6745;
	}                                                             /* end for */

	// Resort data into numerical order
	qsort((void *)t, (size_t) num, sizeof(F_TYPE), MedDevCompare);

	// Find meddev
	if(num%2)                                                 /* is num odd? */
	{
		*p_mad=(float)t[num/2];                              /* middle value */
	}                                                              /* end if */
	else
	{                                                           /* n is even */
		*p_mad=(float)((t[(num/2)-1]+t[num/2])/2.0);  /* avg of 2 mid values */
	}                                                            /* end else */

	free(t);                                   /* deallocate temporary array */

	// Done
	return(num);                                   /* # non-zero data points */
}                                                            /* end MedDev() */

/*****************************************************************************/
/*                                                                           */
/*		Comparison Function for qsort()                                      */
/*                                                                           */
/*****************************************************************************/

int MedDevCompare(const void *a, const void *b)
{
	F_TYPE x =  (*(F_TYPE *) a) - (*(F_TYPE *) b) ;
	return (x>0) ? 1 : (x<0) ? -1 : 0;
}

/*****************************************************************************/
