/*****************************************************************************/
/*                                                                           */
/*                             ModTotvarBias()                               */
/*                                                                           */
/*      Function to calculate the Mod Totvar bias function                   */
/*                                                                           */
/*      Parameters:     int   nAlpha = noise type (-2 to +2)                 */
/*                                                                           */
/*                                                                           */
/*      Return:         float fBias  = Mod Totvar bias factor                */
/*                                     or -1 if error                        */
/*                                                                           */
/*      Alpha     Noise     MTOT/MVAR                                        */
/*       +2        W PM        0.94                                          */
/*       +1        F PM        0.83                                          */
/*        0        W FM        0.73                                          */
/*       -1        F FM        0.70                                          */
/*       -2       RW FM        0.69                                          */
/*                                                                           */
/*      The Mod Totvar bias factor is the ratio of the expected value of     */
/*      MTOT to MVAR.  To correct a calculated Mod Totdev value for a        */
/*      certain noise type, divide the uncorrected Mod Totdev value by the   */
/*      square root of the bias factor returned by this function.            */
/*                                                                           */
/*      Notes:          (1) This function applies to -2 >= alpha >= 2        */
/*                          (W PM, F PM, W FM, F FM, RW FM noise)            */
/*                      (2) This function applies for all T/tau              */
/*                                                                           */
/*      Reference:      E-mail from D.A. Howe/NIST,  3/13/00                 */
/*                                                                           */
/*      Revision record:                                                     */
/*          03/25/00    Created                                              */
/*          05/05/00    Corrected values per D. Howe e-mail                  */
/*                                                                           */
/* (c) Copyright 2000   Hamilton Technical Services   All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float ModTotvarBias(int nAlpha)
{
    // Local variable
    float fBias;

    // Verify parameter
    if(nAlpha<-2 || nAlpha>2)
    {
        return (float) -1.0; // Error
    }

    // Sort by noise type
    switch(nAlpha)
    {
        case -2: // RW FM
        {
            fBias=(float) 0.69;
        }
        break;

        case -1: // F FM
        {
            fBias=(float) 0.70;
        }
        break;

        case -0: // W FM
        {
            fBias=(float) 0.73;
        }
        break;

        case 1: // F PM
        {
            fBias=(float) 0.83;
        }
        break;

        case 2: // W PM
        {
            fBias=(float) 0.94;
        }
        break;
    }

    return (fBias);
}

/*****************************************************************************/

