/*****************************************************************************/
/*                                                                           */
/*                               NewModTotvarCalc()                          */
/*                                                                           */
/*      Filename: mtc.c                                                      */
/*                                                                           */
/*      Function to calculate the Modified Total Variance from phase data    */
/*      Uses uninverted even reflection of offset-removed phase data         */
/*      This function uses faster inner loop algorithm                       */
/*                                                                           */
/*      Parameters:     F_TYPE x[]      = phase data                         */
/*                                        x[0] = # data points               */
/*                                        x[1] = analysis start              */
/*                                        x[2] = analysis end                */
/*                      F_TYPE *fMTDev  = pointer to Mod Totdev              */
/*                      F_TYPE fTau     = interval between phase data points */
/*                      int nAF         = averaging factor                   */
/*                      BOOL bProgress  = flag for progress display          */
/*                                                                           */
/*      Return:         int           = # Mod Totvar analysis points,        */
/*                                      or -1 if memory alloc error          */
/*                                      or -2 if no result error             */
/*                                                                           */
/*      Notes:          1. Calculation is done entirely with a new array.    */
/*                         that is deleted after this function closes.       */
/*                      2. Progress indicator and abort not necessary.       */
/*                      3. The phase data need not be endmatched before      */
/*                         calling this function.                            */
/*                      4. This faster method now handles gaps.              */
/*                                                                           */
/*      Description:    Given the phase data array x[], Mod Totvar is        */
/*                      calculated using z[], which is computed from a       */
/*                      subsequence of x[] consisting of 3m points, where    */
/*                      m is the averaging factor nAF.  This subsequence is  */
/*                      given by x[i] where i=n thru n+3m-1, where n is an   */
/*                      overall index that goes from 1 to the # of phase     */
/*                      data points nNum.  The subsequence z[i] is found     */
/*                      from x[i] by removing the frequency offset by        */
/*                      subtracting the slope found by averaging the 1st     */
/*                      and last halves of the subsequence and dividing by   */
/*                      half the interval.                                   */
/*                                                                           */
/*                      This subsequence is then extended at both ends by    */
/*                      an uninverted even reflection according to z(n-k)=   */
/*                      z(n+k-1) and z(n+3m+k-1)=z(n+3m-k) for k=1 thru 3m,  */
/*                      thus creating a new subsequence having a tripled     */
/*                      range from i=n-3m to i=n+6m-1.                       */
/*                                                                           */
/*                      Mod Totvar is then calculated as the average of its  */
/*                      nNum-3m Totvar sub-estimates.                        */
/*                                                                           */
/*      Reference:      D.A. Howe and F. Vernotte, "Generalization of the    */
/*                      Total Variance Approach to the Modified Allan        */
/*                      Variance", Proc. 31th PTTI Meeting, Dec. 9, 1999,    */
/*                      (to be published).                                   */
/*                                                                           */
/*						This faster version is based on pseudo-code by T.K.  */
/*						Peppler/NIST, private communication, 6/6/00.         */
/*                                                                           */
/*      Revision record:                                                     */
/*          06/12/00    Cloned from ModTotvarCalc()                          */
/*			06/13/00	Added gap filling                                    */
/*			12/02/00	Added to FrequenC32 project but Stable32 still uses  */
/*						this code from STABLE_R.C file - DLL NG as-is        */
/*						WriteProgressMessage and bAbort removed because they */
/*						are Stable32 globals - must be changed for DLL       */
/*          12/07/17	Revise for Linux                                     */
/*                      Replace GlobalAllocPtr() & GlobalAllocFree()         */
/*                      with malloc() and free()                             */
/*                                                                           */
/* (c) Copyright 2000-2017  Hamilton Technical Services  All Rights Reserved */
/*                                                                           */
/*****************************************************************************/

// Headers
#include "frequenc.h"
// #include <windowsx.h>   // For GlobalAllocPtr() & GlobalAllocFree()

// Defines
#define ARRAY_OFFSET 3  // Offset to 1st data point
#define PHASE_DATA 0	// bDataType macro

int NewModTotvarCalc(F_TYPE fX[], F_TYPE *fMTDev,
    F_TYPE fTau, int nAF, BOOL bProgress)
{
    // Local variables
    int i;              // Main index
    int j;              // Auxilary index
    int k;              // Inner index
    int m;              // Alias for averaging factor nAF
    int nNum;           // # points
    int nStart;         // Analysis start
    // int nDummy;      // Dummy variable to avoid complier warning
    int nCount=0;       // # analysis points
    int nTic;           // Increment for progress indicator
    int nLast_i=0;      // Previous value of index

    double *fZ;         // Subsequence array pointer
    double fC1;         // Subsequence 1st half average
    double fC2;         // Subsequence 2nd half average
    double fC;          // Phase slope=frequency offset
    double fSum=0;      // Overall Mod Totvar summation
    double fSubsum=0;   // Subsequence Mod Totvar summation
	double fTemp;		// Temporary data value

    double fZ1;         // 1st phase average
    double fZ2;         // 2nd phase average
    double fZ3;         // 3rd phase average
 
    // Initializations
    nStart=(int)fX[1];          // Analysis start point
    nNum=(int)(fX[2]-fX[1]+1);  // # analysis phase data points = N
    m=nAF;                      // Averaging factor

    nTic=(int)ceil(nNum/100);   // Increment for progress report

	// Set fTemp to mean value of overall phase data
	CalcMean(fX, &fTemp, PHASE_DATA);

    // Start
    if(bProgress)
    {
        // Label progress bar
        // WriteProgressMessage(0, "Calculating Mod Totdev"); 
    }

    #if(0) // Original FrequenC code
    // Allocate 9m points of fZ[] working memory with index from 0 to 9m-1
    // w/o array header (#, Start, End)
    if((fZ=(double*)GlobalAllocPtr(GHND, (9*m)*sizeof(double)))==NULL)
    {
        return(-1);  // Error - memory reallocation failed
    }
    #endif

    #if(1) // Linux generic replacement
    // Allocate 9m points of fZ[] working memory with index from 0 to 9m-1
    // w/o array header (#, Start, End)
    // Note: Don't need to zero memory
    if((fZ=(double*)malloc((9*m)*sizeof(double)))==NULL)
    {
        return(-1);  // Error - memory reallocation failed
    }
    #endif

    // Outer loop thru all nNum-3m+1 subsequences of 3m phase data points
    // A subsequence will be processed during each iteration
    // and the average used to find Mod-Totvar
    // This should allow a max m of nNum/3
    // Note: PTTI Paper uses only nNum-3m subsequences
    // but at my suggestion the "official" version has now been expanded
    // to use nNum-3m+1 as done here

    // To handle analysis limits, need to initialize index to nStart-1 not 0
    // and end it before nStart-1-3*m+1 instead of nNum-3*m+1
    for(i=nStart-1; i<nStart-1+nNum-3*m+1; i++)
    {
        // Copy data subsequence into working array starting at index 3m
        // Note:  The fX[] array needs offset for its header info
        for(j=0; j<3*m; j++)
        {
			// Test for gap
			// First overall data point is at i+j=0
			// Last overall point is at i+j=nNum-1
			// Those are allowed to have zero values for phase data
			// Embedded zeros are treated as gaps
			if( (fX[i+j+ARRAY_OFFSET]) || (i+j==0) || (i+j==nNum-1) )
			{
				fZ[3*m+j]=fX[i+j+ARRAY_OFFSET];
				fTemp=fZ[3*m+j];
			}
			else 
			{
				// Replace gap with interpolated value
				// if adjacent points are themselves not gaps
				if(	(fX[i+j+ARRAY_OFFSET-1]) && (fX[i+j+ARRAY_OFFSET+1]) )
				{
					fZ[3*m+j]=
						(fX[i+j+ARRAY_OFFSET-1]+fX[i+j+ARRAY_OFFSET+1])/2.0;
				}
				else
				{
					// For lack of a better idea,
					// replace gap with last OK value
					// or, if haven't found one, the overall mean value
					fZ[3*m+j]=fTemp;
				}
			}
        }
    
        // Do removal of frequency offset
        // Find slope fC by averaging the 1st and last halves of the subsequence
        // subtracting them, and dividing by half the interval
        fC1=0;
        fC2=0;
        for(j=0; j<3*m/2; j++)
        {
            fC1+=fZ[3*m+j];
            fC2+=fZ[3*m+(int)ceil(3.0*(double)m/2.0)+j];
        }
        fC1/=(double)(3*m/2);
        fC2/=(double)(3*m/2);
        fC=(fC2-fC1)/ceil(3.0*(double)m/2.0);

        // Then subtract slope from each point
        for(j=0; j<3*m; j++)
        {
            fZ[3*m+j]-=fC*(double)j;
        }
                
        // Do uninverted even reflection at bottom and top
        for(j=0; j<3*m; j++)
        {
            // Bottom reflection
            fZ[j]=fZ[6*m-1-j];
    
            // Top reflection
            fZ[6*m+j]=fZ[j];
        }

        // Calc Mvar for the fZ[] subsequence
        // Will need to add gap testing and analysis limits
        // This data array does not need an offset
        // This code implements the right summation of Eq(8) of the reference

        // Initialize inner sum
        fSubsum=0;

        // We have a doubly-extended subsequence fZ[] of 9m points
        // and are going to form m-point averages
        // in 6m+1 groups of 3 (called fZ1, fZ2 and fZ3)
        // from which all possible fully overlapping 2nd differences
        // will be used to calculate Mod Totvar
        // Note: Only 6m of the 6m+1 possible 2nd differences are used
        // because the last one is a duplicate
		for(j=0; j<6*m; j++)
		{
			if(j==0)
			{
				// Find the 3 phase averages for the first 2nd difference
				// Initialize phase averages
				fZ1=0.0;
				fZ2=0.0;
				fZ3=0.0;

				// Sum phase averages for 1st 3 sets of m points
				// using points 0 thru m-1, m thru 2m-1 and 2m thru 3m-1
				// Note: These phase sums are divided by m = # points of each
				// sum in the 2nd difference expression below
				for(k=0; k<m; k++)
				{
					fZ1 += fZ[k];
					fZ2 += fZ[k+m];
					fZ3 += fZ[k+2*m];
				}
			}
			else
			{
				// Now do the rest of the points w/o summing all each time
				// Just subtract the old point and add the new one
	            // Sum phase averages for 3 sets of m points
		        // using points j thru j+m-1, j+m thru j+2m-1 and
				// j+2m thru j+3m-1
			    // Note: These phase sums are divided by m = # points of
				// each sum in the 2nd difference expression below
				fZ1 = fZ1 -fZ[j-1]     +fZ[j+m-1];
				fZ2 = fZ2 -fZ[j+m-1]   +fZ[j+2*m-1];
				fZ3 = fZ3 -fZ[j+2*m-1] +fZ[j+3*m-1];
 			}

			// Calc 2nd diff of one of the rest of the 6m sets of 3 phase avgs
			fSubsum += SQR((fZ1 - 2*fZ2 + fZ3)/m);
		}

        // Scale the subsum and add it to overall sum
        fSum+=fSubsum/(6*m);

        // Increment the sum count
        nCount++;

        // Display progress
        // Update when index has changed by at least tic amount
        if(((i-nLast_i)>=nTic) && (bProgress))
        {
            // WriteProgressMessage((int)(0.5+100.0*i/(nNum-3*m)),
            //    "Calculating Mod Totdev");
            nLast_i=i;
        }

        // Check for abort
        // if(bAbort)
        // {
        //     break;
        // }
    }

    // Free the subsequence array
    // nDummy=GlobalFreePtr(fZ);
    // nDummy=nDummy;
    free(fZ);

    // Scale result - See Eq (8) of Reference
    // Without gaps, nCount=nNum-3m
    fSum/=(fTau*fTau*nAF*nAF*2*nCount);

    // Find Mod-Totdev
    if(nCount && fSum>0)
    {
        *fMTDev=(F_TYPE)sqrt(fSum);
    }
    else // No result error
    {
        *fMTDev=0.0;
        nCount=-2;
    }

    // Done
    // Clear progress display
    // WriteProgressMessage(0, "");

    return(nCount);  // Return # phase analysis points
}

/*****************************************************************************/
