/*****************************************************************************/
/*                                                                           */
/*                                ModTotvarCalc()                            */
/*                                                                           */
/*      Function to calculate the Modified Total Variance from phase data    */
/*      Uses uninverted even reflection of offset-removed phase data         */
/*                                                                           */
/*      Parameters:     F_TYPE x[]      = phase data                         */
/*                                        x[0] = # data points               */
/*                                        x[1] = analysis start              */
/*                                        x[2] = analysis end                */
/*                      F_TYPE *fMTDev  = pointer to Mod Totdev              */
/*                      F_TYPE fTau     = interval between phase data points */
/*                      int nAF         = averaging factor                   */
/*						BOOL *bAbort    = pointer to abort flag              */                                                                     
/*						BOOL bProgress  = progress indicator flag            */                                                                     
/*						void *ProgressFunction()                             */
/*									    = pointer to progress indicator      */
/*									 	  funct with args int nPercentDone   */
/*										  and char *szMessage                */
/*                                                                           */
/*      Return:         int           = # Mod Totvar analysis points,        */
/*                                      or -1 if memory alloc error          */
/*                                      or -2 if no result error             */
/*                                                                           */
/*      Notes:          1. Calculation is done entirely with a new array.    */
/*                         that is deleted after this function closes.       */
/*                      2. Progress indicator and abort not necessary.       */
/*                      3. The phase data need not be endmatched before      */
/*                         calling this function.                            */
/*                                                                           */
/*      Description:    Given the phase data array x[], Mod Totvar is        */
/*                      calculated using z[], which is computed from a       */
/*                      subsequence of x[] consisting of 3m points, where    */
/*                      m is the averaging factor nAF.  This subsequence is  */
/*                      given by x[i] where i=n thru n+3m-1, where n is an   */
/*                      overall index that goes from 1 to the # of phase     */
/*                      data points nNum.  The subsequence z[i] is found     */
/*                      from x[i] by removing the frequency offset by        */
/*                      subtracting the slope found by averaging the 1st     */
/*                      and last halves of the subsequence and dividing by   */
/*                      half the interval.                                   */
/*                                                                           */
/*                      This subsequence is then extended at both ends by    */
/*                      an uninverted even reflection according to z(n-k)=   */
/*                      z(n+k-1) and z(n+3m+k-1)=z(n+3m-k) for k=1 thru 3m,  */
/*                      thus creating a new subsequence having a tripled     */
/*                      range from i=n-3m to i=n+6m-1.                       */
/*                                                                           */
/*                      Mod Totvar is then calculated as the average of its  */
/*                      nNum-3m Totvar sub-estimates.                        */
/*                                                                           */
/*      Reference:      D.A. Howe and F. Vernotte, "Generalization of the    */
/*                      Total Variance Approach to the Modified Allan        */
/*                      Variance", Proc. 31th PTTI Meeting, Dec. 9, 1999,    */
/*                      (to be published).                                   */
/*                                                                           */
/*      Revision record:                                                     */
/*          12/12/99    Adapted from TotvarCalc() and CalcPhaseModSigma()    */
/*                      of FrequenC Library.                                 */
/*          12/16/99    Basic code drafted - no gaps or analysis limits      */
/*          12/26/99    Changed progress bar argument from lpfn to flag      */
/*          12/30/99    Fixed & cleaned up a few details                     */
/*          02/06/00    Changed to fully overlapping per D. Howe review      */
/*          02/08/00    Changed inner sum index limit from 6m+1 to 6m to     */
/*                      elim duplicate term, and change divisor to 6m        */
/*          02/12/00    Added trace code to aid debugging                    */
/*          02/23/00    Added alternative freq offset removal method         */
/*          05/06/00    Added code to handle analysis limits                 */
/*                      Not sure about gaps - no specific code for them      */
/*                      But seems like they S/B OK as-is                     */
/*			03/09/03	Added abort and progress args                        */
/*						Changed from GlobalAllocPtr() to malloc()            */
/*          09/04/08	Copied from MTC2.C of FrequenC Library               */
/*                      Modified code for faster more efficient algorithm    */
/*                      similar to CalcFastModSigma() that practically       */
/*                      eliminates 3rd inner loop                            */
/*                      Complete run now varies as N^2 rather than N^3       */
/*                                                                           */
/* (c) Copyright 1999-2008  Hamilton Technical Services  All Rights Reserved */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int ModTotvarCalc(F_TYPE fX[], F_TYPE *fMTDev, F_TYPE fTau, int nAF,
	BOOL *bAbort, BOOL bProgress, void *ProgressFunction(int nPercentDone,
	char *szMessage))
{
    // Local variables
    int i;              // Main index
    int j;              // Auxilary index
    int k;              // Inner index
    int m;              // Alias for averaging factor nAF
    int nNum;           // # points
    int nStart;         // Analysis start
    int nCount=0;       // # analysis points
    int nTic;           // Increment for progress indicator
    int nLast_i=0;      // Previous value of index

    double *fZ;         // Subsequence array pointer
    double fC1;         // Subsequence 1st half average
    double fC2;         // Subsequence 2nd half average
    double fC;          // Phase slope=frequency offset
    double fSum=0;      // Overall Mod Totvar summation
    double fSubsum=0;   // Subsequence Mod Totvar summation

    double fZ1;         // 1st phase average
    double fZ2;         // 2nd phase average
    double fZ3;         // 3rd phase average
 
    // Initializations
    nStart=(int)fX[1];          // Analysis start point
    nNum=(int)(fX[2]-fX[1]+1);  // # analysis phase data points = N
    m=nAF;                      // Averaging factor

    nTic=(int)ceil(nNum/100);   // Increment for progress report

    // Start
    if(bProgress)
    {
        // Label progress bar
        ProgressFunction(0, "Calculating Mod Totdev"); 
    }

    // Allocate 9m points of fZ[] working memory with index from 0 to 9m-1
    // w/o array header (#, Start, End)
    if((fZ=(double*)malloc((9*m)*sizeof(double)))==NULL)
    {
        return(-1);  // Error - memory reallocation failed
    }

    // Outer loop thru all nNum-3m+1 subsequences of 3m phase data points
    // A subsequence will be processed during each iteration
    // and the average used to find Mod-Totvar
    // This should allow a max m of nNum/3
    // Note: PTTI Paper uses only nNum-3m subsequences
    // but at my suggestion the "official" version has now been expanded
    // to use nNum-3m+1 as done here

    // To handle analysis limits, need to initialize index to nStart-1 not 0
    // and end it before nStart-1-3*m+1 instead of nNum-3*m+1
    for(i=nStart-1; i<nStart-1+nNum-3*m+1; i++)
    {
        // Copy data subsequence into working array starting at index 3m
        // Note:  The fX[] array needs offset for its header info
        for(j=0; j<3*m; j++)
        {
            fZ[3*m+j]=fX[i+j+ARRAY_OFFSET];
        }
    
        // Do removal of frequency offset
        // Find slope fC by averaging the 1st and last halves of the subsequence
        // subtracting them, and dividing by half the interval
        fC1=0;
        fC2=0;
        for(j=0; j<3*m/2; j++)
        {
            fC1+=fZ[3*m+j];
            fC2+=fZ[3*m+(int)ceil(3.0*(double)m/2.0)+j];
        }
        fC1/=(double)(3*m/2);
        fC2/=(double)(3*m/2);
        fC=(fC2-fC1)/ceil(3.0*(double)m/2.0);

        // Then subtract slope from each point
        for(j=0; j<3*m; j++)
        {
            fZ[3*m+j]-=fC*(double)j;
        }
                
        // Do uninverted even reflection at bottom and top
        for(j=0; j<3*m; j++)
        {
            // Bottom reflection
            fZ[j]=fZ[6*m-1-j];
    
            // Top reflection
            fZ[6*m+j]=fZ[j];
        }

        // Calc Mvar for the fZ[] subsequence
        // Will need to add gap testing and analysis limits
        // This data array does not need an offset
        // This code implements the right summation of Eq(8) of the reference

        // Initialize inner sum
        fSubsum=0;

        // We have a doubly-extended subsequence fZ[] of 9m points
        // and are going to form m-point averages
        // in 6m+1 groups of 3 (called fZ1, fZ2 and fZ3)
        // from which all possible fully overlapping 2nd differences
        // will be used to calculate Mod Totvar
        // Note: Only 6m of the 6m+1 possible 2nd differences are used
        // because the last one is a duplicate
        // NOTE: For 6m+1 version, use j<6*m+1 instead of j<6*m
        for(j=0; j<6*m; j++)
        {
			#if(0) // Original code
            // Find the 3 phase averages for this 2nd difference
            // Initialize phase averages
            fZ1=0.0;
            fZ2=0.0;
            fZ3=0.0;
			
			// Sum phase averages for 3 sets of m points
			// using points j thru j+m-1, j+m thru j+2m-1 and j+2m thru j+3m-1
			// Note: These phase sums are divided by m = # points of each sum
			// in the 2nd difference expression below
			for(k=j; k<j+m; k++)
			{
				fZ1 += fZ[k];
				fZ2 += fZ[k+m];
				fZ3 += fZ[k+2*m];
			}
			#endif
            
			#if(1) // New more efficient code 09/04/08 - yields exactly same results
			// Do full summation for 1st time only
			if(j==0)
			{
				// Initialize phase averages
				fZ1=0.0;
				fZ2=0.0;
				fZ3=0.0;
			
				// Sum phase averages for 3 sets of m points
				// using points j thru j+m-1, j+m thru j+2m-1 and j+2m thru j+3m-1
				// Note: These phase sums are divided by m = # points of each sum
				// in the 2nd difference expression below
				for(k=j; k<j+m; k++)
				{
					fZ1 += fZ[k];
					fZ2 += fZ[k+m];
					fZ3 += fZ[k+2*m];
				}
			}
			else // Rest of sums
			{
				// Just drop old 1st term and add new last one
				fZ1 -= fZ[j-1];
				fZ1 += fZ[j+m-1];
				fZ2 -= fZ[j+m-1];
				fZ2 += fZ[j+2*m-1];
				fZ3 -= fZ[j+2*m-1];
				fZ3 += fZ[j+3*m-1];
			}
			#endif

			// Calc 2nd difference of one of the 6m sets of 3 phase averages
            fSubsum += SQR((fZ1 - 2*fZ2 + fZ3)/m);
        }

        // Scale the subsum and add it to overall sum
        // NOTE: For 6m+1 version this expression uses (6*m+1) instead of (6*m)
        fSum+=fSubsum/(6*m);

        // Increment the sum count
        nCount++;

        // Display progress
        // Update when index has changed by at least tic amount
        if(((i-nLast_i)>=nTic) && (bProgress))
        {
            ProgressFunction((int)(0.5+100.0*i/(nNum-3*m)),
                "Calculating Mod Totdev");
            nLast_i=i;
        }

        // Check for abort
        if(*bAbort)
        {
            break;
        }
    }

    // Free the subsequence array
    free(fZ);

    // Scale result - See Eq (8) of Reference
    // Without gaps, nCount=nNum-3m
    fSum/=(fTau*fTau*nAF*nAF*2*nCount);

    // Find Mod-Totdev
    if(nCount && fSum>0)
    {
        *fMTDev=(F_TYPE)sqrt(fSum);
    }
    else // No result error
    {
        *fMTDev=0.0;
        nCount=-2;
    }

    // Done
    // Clear progress display
    ProgressFunction(0, "");

    return(nCount);  // Return # phase analysis points
}


