/*****************************************************************************/
/*                                                                           */
/*                             ModTotvarEDF()                                */
/*                                                                           */
/*  Function to determine the estimated # of chi-squared degrees of freedom  */
/*  for the modified total variance (MTOT) of a power law noise process.     */
/*                                                                           */
/*  Params:         int   nAlpha = alpha                                     */
/*                  float fRatio = T/tau = N/m                               */
/*                                                                           */
/*  Return:         float edf = estimated degrees of freedom                 */
/*                              or -1 if error                               */
/*                                                                           */
/*  The edf is modeled by the expression b(T/tau) - c, where:                */
/*                                                                           */
/*  Alpha    Noise     b      c        Original values                       */
/*   2        W PM    1.90   2.10                                            */
/*   1        F PM    1.20   1.40                                            */
/*   0        W FM    1.10   1.20                                            */
/*  -1        F FM    0.85   0.50                                            */
/*  -2       RW FM    0.75   0.31                                            */
/*                                                                           */
/*  Alpha    Noise     b      c        New values per D. Howe                */
/*   2        W PM    2.16   2.28      e-mail of 05/20/00                    */
/*   1        F PM    1.73   2.99                                            */
/*   0        W FM    1.33   1.89                                            */
/*  -1        F FM    0.92   0.76                                            */
/*  -2       RW FM    0.79   0.37                                            */
/*                                                                           */
/*  Notes:          (1) This function applies only to -2 >= alpha >= 2       */
/*                      (W PM, F PM, W FM, F FM and RW FM noise)             */
/*                  (2) This function applies only for m > 8.0               */
/*                  (3) T is defined as tau0 * # phase data points           */
/*                  (4) The t/tau ratio should never be < 3                  */
/*                                                                           */
/*  References:     E-Mails from D.A. Howe/NIST, 3/13/00 & 5/20/00           */
/*                                                                           */
/*  Revision record:                                                         */
/*      03/25/00    Created                                                  */
/*      03/30/00    Parameter verification revised                           */
/*      05/21/00    Parameters revised per D. Howe update                    */
/*                                                                           */
/* (c) Copyright 2000  Hamilton Technical Services   All Rights Reserved     */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float ModTotvarEDF(int nAlpha, float fRatio)
{
    // Local variables
    float fEDF;

    // Verify parameters
    if(fRatio<3.0)
    {
        return -1.0; // Error
    }

    if(nAlpha<-2 || nAlpha>2)
    {
        return -1.0; // Error
    }

    switch(nAlpha)
    {
        case 2: // W PM noise
        {
            // fEDF=(float)(1.90*fRatio - 2.10);
            fEDF=(float)(2.16*fRatio - 2.28);
        }
        break;

        case 1: // F PM noise
        {
            // fEDF=(float)(1.20*fRatio - 1.40);
            fEDF=(float)(1.73*fRatio - 2.99);
        }
        break;

        case 0: // W FM noise
        {
            // fEDF=(float)(1.10*fRatio - 1.20);
            fEDF=(float)(1.33*fRatio - 1.89);
        }
        break;

        case -1: // F FM noise
        {
            // fEDF=(float)(0.85*fRatio - 0.50);
            fEDF=(float)(0.92*fRatio - 0.76);
        }
        break;

        case -2: // RW FM noise
        {
            // fEDF=(float)(0.75*fRatio - 0.31);
            fEDF=(float)(0.79*fRatio - 0.37);
        }
        break;
    }

    return fEDF;

}

/*****************************************************************************/

