/*****************************************************************************/
/*                                                                           */
/*                              CalcMTIE()                                   */
/*                                                                           */
/*      Function to calc MTIE (maximum time interval error) for phase data   */
/*      Parameters:                                                          */
/*          F_TYPE fX[]    = data array;     F_TYPE fX[0] = # data points    */
/*          F_TYPE fX[1]   = analysis start; F_TYPE fX[2] = analysis end     */
/*          F_TYPE *fMTIE  = pointer to MTIE                                 */
/*          int nAF        = averaging factor                                */
/*			BOOL *bAbort   = pointer to abort flag                           */                                                                     
/*			BOOL bProgress = progress indicator flag                         */                                                                     
/*			void *ProgressFunction()                                         */
/*						   = pointer to progress indicator function with     */
/*							 args int nPercentDone and char *szMessage       */
/*                                                                           */
/*      Return:                                                              */
/*          num   = # analysis points (not very useful)                      */
/*                                                                           */
/*      Algorithm:                                                           */
/*          An n-point wide window is moved thru the phase data and the      */
/*          difference between the maximum and minimum phase (time) values   */
/*          is found at each window position.  MTIE is the overall maximum   */
/*          of this time interval error over the whole data set.             */
/*                                                                           */
/*      Notes:                                                               */
/*          (1) This function handles gaps because FindMinMax() does.        */
/*          (2) No test suite values to check results against.  Results are  */
/*              consistent with the p-p range of the test data.              */
/*          (3) MTIE has no tau or AF dependence for white pm noise.  It has */
/*              a tau^+1/2 log slope for W FM noise. It is very sensitive to */
/*              peak values (transients or outliers).                        */
/*          (4) # time intervals is N-1, where N = # phase data points, so   */
/*              AF can go from 1 to N-1.                                     */
/*                                                                           */
/*      Reference:      S. Bregni, "Clock Stability Characterization and     */
/*                      Measurement in Telecommunications", IEEE             */
/*                      Transactions on Instrumentation and Measurement,     */
/*                      Vol. 46, No. 6, Dec 1997, pp 1284-1294, Eq. 14.      */
/*                                                                           */
/*      Revision record:                                                     */
/*          12/07/96    Drafted - seems to run OK - patched into Sigma       */
/*                      function instead of TOTALVAR (for freq data!) since  */
/*                      that call has same signature.                        */
/*          06/03/98    Put into Stable32                                    */
/*          06/04/98    Refined progress display                             */
/*          01/27/01    Extended allowable index by 1                        */
/*			03/09/03	Added abort and progress args                        */
/*          12/07/17	MAX macro usage                                      */
/*                                                                           */
/* (c) Copyright 1996-2003  Hamilton Technical Services  All Rights Reserved */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int CalcMTIE(F_TYPE fX[], F_TYPE *fMTIE,
	int nAF, BOOL *bAbort, BOOL bProgress,
	void *ProgressFunction(int nPercentDone, char *szMessage))
{
    // Define local variables
    // Ints
    int i;                      // Index
    int last_i=0;               // Previous index
    int tic;                    // Progress interval
    int nStart;                 // Analysis start
    int nEnd;                   // Analysis end
    int nNum;                   // # analysis points
    // F_TYPES
    F_TYPE fMin;                // Min phase data value within window
    F_TYPE fMax;                // Max phase data value within window

    // Set analysis limits & progress interval
    nStart=(int)fX[1];
    nEnd=(int)fX[2];
    nNum=nEnd-nStart+1;
    tic=(int)ceil((nNum-nStart-nAF)/100);  /* 1% interval for progress report */

    // Initialize MTIE value
    *fMTIE=0.0;

    // Calc MTIE
    // The analysis window is nAF points wide.  It starts at the 1st data point
    // and moves thru all the data until the end of the window is at the end of
    // the analysis data.

    // Initialize progress display
    if(bProgress)
	{
		ProgressFunction(0, "Calculating MTIE");
	}

    // Was i<nNum-nAF until 01/27/00
    for(i=nStart; i<=nNum-nAF; i++)
    {
        // Find the min & max values within the data window
        fX[1]=i;
        fX[2]=i+nAF;
        FindMinMax(fX, &fMin, &fMax, PHASE_DATA);

        // MTIE is the overall maximum p-p excursion
        *fMTIE=MAX(*fMTIE, fMax-fMin);

        // Display progress
        // Note:  This code provides updates only if index has changed
        // by at least tic amount (1%)
        if( ((i-last_i)>=tic) && (bProgress) )
        {
            ProgressFunction((int)(0.5+100.0*i/(nNum-nStart-nAF)),
                "Calculating MTIE");
            last_i=i;
        }

        // Check for abort
        if(*bAbort)
        {
            // Restore analysis limits
            fX[1]=nStart;
            fX[2]=nEnd;

            // Bail out
            break;
        }
    }

    // Restore analysis limits
    fX[1]=nStart;
    fX[2]=nEnd;

    // Done
    // Clear progress display
    if(bProgress)
	{
	    ProgressFunction(0, "");
	}

    // Return # analysis points
	return nNum;
}

/***************************************************************************/

