/*****************************************************************************/
/*                                                                           */
/*                              NoiseID()                                    */
/*                                                                           */
/*  Function to determine power law noise type fron B1 or R(n) ratio, where: */
/*      B1 is ratio of normal variance to Allan variance                     */
/*      R(n) is ratio of modified to normal Allan variance                   */
/*                                                                           */
/*  Parameters:     int     nNum            # non-gap data points            */
/*                                          (# data points-# gaps)           */
/*                  int     nSigmaType      Sigma type                       */
/*                  int     nAvgFactor      Averaging factor                 */
/*                  float   fBW             Bandwidth factor                 */
/*                  float   fRatio          B1 or  R(n) ratio                */
/*                                                                           */
/*  Return          int     nAlpha          Alpha (noise type)               */
/*                                          -4 <= alpha <= 2                 */
/*                                          RR FM to W PM                    */
/*                                                                           */
/*  Note 1: Use CalcStarB1() before call to this function with NORMAL_SIGMA  */
/*          as alternative to using CalcHadamardB1() herein with Hadamard    */
/*          sigmas.                                                          */
/*                                                                           */
/*  Note 2: Sigma types are as follows:     NORMAL_SIGMA                     */
/*                                          OVERLAPPING_SIGMA                */
/*                                          MODIFIED_SIGMA                   */
/*                                          TIME_SIGMA                       */
/*                                          TOTAL_SIGMA                      */
/*                                          MODTOTAL_SIGMA                   */
/*                                          TIMETOTAL_SIGMA                  */
/*                                          HADAMARD_SIGMA                   */
/*                                          OVERLAPHAD_SIGMA                 */
/*                                          HADTOTAL_SIGMA                   */
/*                                                                           */
/*      Revision record:                                                     */
/*          03/14/99    Added OVERLAPHAD_SIGMA sigma type                    */
/*          04/24/99    Added usage of CalcHadamardB1()                      */
/*          12/26/99    Added MODTOTAL_SIGMA sigma type                      */
/*          05/06/00    Added TIMETOTAL_SIGMA sigma type                     */
/*          11/05/00    Added HADTOTAL_SIGMA sigma type                      */
/*          11/15/00    Moved TIME_SIGMA to with other mod sigma cases       */
/*          11/23/00    Changed Drift or FW FM case to alpha=-3              */
/*                                                                           */
/*      (c) 1996-9 W. Riley Hamilton Technical Services All Rights Reserved  */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int NoiseID(int nNum, int nSigmaType,
	int nAvgFactor, float fBW, float fRatio)
{
    // Local variables
    // Ints
    int nAlpha;                                              // Alpha exps
    // Floats
    float fB_rrfm, fB_fwfm, fB_rwfm, fB_ffm, fB_wfm, fB_pm;  // Bias functs
    float fR_rwfm, fR_ffm,  fR_wfm, fR_fpm, fR_wpm;          // Ratio functs

    #ifdef LOGGING
    // Chars
    char szBuffer[BUFFER_SIZE+1];   // Text buffer
    #endif


    switch(nSigmaType)
    {
        case NORMAL_SIGMA:
        case OVERLAPPING_SIGMA:
        case TOTAL_SIGMA:
        {
            // Note: 2nd arg in CalcBias() is mu
            fB_fwfm=CalcBias(nNum,  2);
            fB_rwfm=CalcBias(nNum,  1);
            fB_ffm =CalcBias(nNum,  0);
            fB_wfm =CalcBias(nNum, -1);
            fB_pm  =CalcBias(nNum, -2);

            #ifdef LOGGING
            sprintf(szBuffer, "NoiseID() using "
                "fB_fwfm=%f, fB_rwfm=%f, fB_ffm=%f, fB_wfm=%f, fB_pm=%f",
                fB_fwfm, fB_rwfm, fB_ffm, fB_wfm, fB_pm);
            WriteLog(60, szBuffer);
            #endif

            if(fRatio>((fB_fwfm+fB_rwfm)/2))
            {
                // Drift or FW FM
                // Note: Can make it -2 to avoid alpha<-2
                // which causes trouble in CI calcs
                // Or can let it be -3 to detect FW FM noise
                // and deal with potential CI calc problem separately
                // The latter approach is used
                nAlpha=-3;
            }
            else if(fRatio>sqrt(fB_rwfm*fB_ffm))
            {
                // Random Walk FM
                nAlpha=-2;
            }
            else if(fRatio>sqrt(fB_ffm*fB_wfm))
            {
                // Flicker FM
                nAlpha=-1;
            }
            else if(fRatio>sqrt(fB_wfm*fB_pm))
            {
                // White FM
                nAlpha=0;
            }
            else
            {
                // White or Flicker PM
                // Make it +1 to agree with what Run function does
                nAlpha=1;  // Could also be +2
            }
        }
        break;

        case MODIFIED_SIGMA:
        case TIME_SIGMA:
        case MODTOTAL_SIGMA:
        case TIMETOTAL_SIGMA:
        {
            // Note: 1st arg in CalcRatio() is alpha = nAvg Factor
            // Was wrong (nNum) until 02/09/02
            fR_rwfm=CalcRatio(-2, nAvgFactor, fBW);
            fR_ffm =CalcRatio(-1, nAvgFactor, fBW);
            fR_wfm =CalcRatio( 0, nAvgFactor, fBW);
            fR_fpm =CalcRatio( 1, nAvgFactor, fBW);
            fR_wpm =CalcRatio( 2, nAvgFactor, fBW);

            #ifdef LOGGING
            sprintf(szBuffer, "NoiseID() using "
                "fR_rwfm=%f, fR_ffm=%f, fR_wfm=%f, fR_fpm=%f, fR_wpm=%f",
                fR_rwfm, fR_ffm, fR_wfm, fR_fpm, fR_wpm);
            WriteLog(60, szBuffer);
            #endif

            if(nAvgFactor==1)
            {
                // Undefined
                nAlpha=0;
            }
            else if(fRatio>sqrt(fR_rwfm*fR_ffm))
            {
                // Random Walk FM
                nAlpha=-2;
            }
            else if(fRatio>sqrt(fR_ffm*fR_wfm))
            {
                // Flicker FM
                nAlpha=-1;
            }
            else if(fRatio>sqrt(fR_wfm*fR_fpm))
            {
                // White FM
                nAlpha=0;
            }
            else if(fRatio>sqrt(fR_fpm*fR_wpm))
            {
                // Flicker PM
                nAlpha=1;
            }
            else
            {
                // White PM
                nAlpha=2;
            }
        }
        break;

        case HADAMARD_SIGMA:
        case OVERLAPHAD_SIGMA:
        case HADTOTAL_SIGMA:
        {
            // Note: 2nd arg in CalcHadamardB1() is mu
            fB_rrfm=CalcHadamardB1(nNum,  3);
            fB_fwfm=CalcHadamardB1(nNum,  2);
            fB_rwfm=CalcHadamardB1(nNum,  1);
            fB_ffm =CalcHadamardB1(nNum,  0);
            fB_wfm =CalcHadamardB1(nNum, -1);
            // Use ordinary B1 ratio for mu=-2 (F or F PM)
            fB_pm  =CalcBias(nNum, -2);

            #ifdef LOGGING
            sprintf(szBuffer, "NoiseID() using "
                "fB_rrfm=%f, fB_fwfm=%f, fB_ffm=%f, fB_wfm=%f, fB_pm=%f",
                fB_rrfm, fB_fwfm, fB_ffm, fB_wfm, fB_pm);
            WriteLog(60, szBuffer);
            #endif

            if(fRatio>((fB_rrfm*fB_fwfm)))
            {
                // RR FM
                nAlpha=-4;
            }
            else if(fRatio>((fB_fwfm*fB_rwfm)))
            {
                // Drift or FW FM
                nAlpha=-3;
            }
            else if(fRatio>sqrt(fB_rwfm*fB_ffm))
            {
                // Random Walk FM
                nAlpha=-2;
            }
            else if(fRatio>sqrt(fB_ffm*fB_wfm))
            {
                // Flicker FM
                nAlpha=-1;
            }
            else if(fRatio>sqrt(fB_wfm*fB_pm))
            {
                // White FM
                nAlpha=0;
            }
            else
            {
                // White or Flicker PM

                // Make it +1 to agree with what Run function does
                nAlpha=1;  // Could also be +2
            }
        }
        break;
    }
    
    return nAlpha;
}

/****************************************************************************/

