/*****************************************************************************/
/*                                                                           */
/*                              Prewhiten()                                  */
/*                                                                           */
/*      Function to prewhiten freq data by removing autoregressive noise     */
/*                                                                           */
/*      Parameters:     F_TYPE y[]  = freq data                              */
/*                                    y[0] = # freq data points              */
/*                                    y[1] = analysis start                  */
/*                                    y[2] = analysis end                    */
/*                      F_TYPE rho  = lag 1 autocorrelation value            */
/*                                                                           */
/*      Return:         int         = # frequency data gaps,                 */
/*                                    or -1 if error                         */
/*                                                                           */
/*      Notes:          All zeros treated as gaps in freq data      .        */
/*                      Function processes n points and produces n-1 points  */
/*                      Rho can be either sign or zero but not > |1|         */
/*                      Operation done in-place so original data is lost     */
/*                      Rho=0 would leave data unchanged                     */
/*                      but 1 fewer point (first point removed)              */
/*                                                                           */
/*      Revision record:                                                     */
/*          02/12/08    Created from CPTF2.C of FrequenC library             */
/*                                                                           */
/* (c) Copyright 2008  Hamilton Technical Services  All Rights Reserved      */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int Prewhiten(F_TYPE y[], F_TYPE rho)
{
    int i;                                                        /* y index */
    int num;                                       /* original # freq points */
	int g;                                                         /* # gaps */

    // Check for valid rho
	if( (rho >1.0) || (rho<-1.0) )
    {
        return(-1);                                                 /* error */
    }

    num=(int)(y[2]-y[1]+1);              /* set num = original # freq points */

    // Prewhiten freq data by removing AR(1) noise
	// 1st data point is at array index i=3
	// Last data point is at array index i=num+2
	// Index i extends to 1 fewer than that because we access index i+1
    for(i=3; i<num+2; i++)
    {
        if((y[i] && y[i+1]))                     /* neither freq point a gap */
        {
            y[i]=y[i+1] - rho*y[i];                    /* remove AR(1) noise */
        }                                                          /* end if */
        else                                                     /* freq gap */
        {
            y[i]=0;                                    /* denote gap by zero */
            g++;                                      /* increment gap count */
        }                                                        /* end else */
    }                                                             /* end for */

	// Set new freq array numbers
    y[0]=num-1;                                             /* # freq points */
    y[1]=1;                                           /* # of 1st freq point */
    y[2]=num-1;                                      /* # of last freq point */

    return(g);
}                                                        /* end Prewhiten() */

