/*****************************************************************************/
/*                                                                           */
/*                                 Theo1()                                   */
/*                                                                           */
/*      Function to calculate Theoretical Variance #1 from phase data        */
/*                                                                           */
/*      Parameters:     F_TYPE x[]      = phase data                         */
/*                                        x[0] = # data phase points         */
/*                                        x[1] = analysis start              */
/*                                        x[2] = analysis end                */
/*                      F_TYPE *p_sig   = pointer to Theo1 deviation         */
/*                      F_TYPE tau      = data averaging time = tau_0        */
/*                      int m           = averaging factor = tau/tau_0       */
/*						BOOL *bAbort    = pointer to abort flag              */
/*						BOOL bProgress  = progress indicator flag            */
/*						void *ProgressFunction()                             */
/*									    = pointer to progress indicator      */
/*									 	  funct with args int nPercentDone   */
/*										  and char *szMessage                */
/*                                                                           */
/*      Return:         float           = # analysis points,                 */
/*                                        or -1 if error                     */
/*                                                                           */
/*      Notes:          Embedded zero phase values treated as gaps in data.  */
/*                      The analysis data consists of n phase samples at     */
/*                      interval tau_0.  Denominator term (n-m) must be >0.  */
/*                      This sets max m=n-1.  m and tau must be >0, and m    */
/*                      must be even.  Theo1 was originally intended for     */
/*						m>=10, but, has been extended down to m=2 by a later */
/*						improved bias factor formula.                        */
/*                                                                           */
/*      Ref:            D.A. Howe and T.K. Peppler, "Estimation of Very      */
/*                      Long Term Frequency Stability Using a Special        */
/*                      Purpose Statistic", IEEE Int'l FCS 2003.             */
/*                                                                           */
/*      Revision record:                                                     */
/*          02/06/03    Created                                              */
/*			02/08/03	Running                                              */
/*			02/09/03	Added progress indicator                             */
/*			02/20/03	Changed count from int to double                     */
/*						Changed return type to float                         */
/*			03/09/03	Made progress function and abort flag arguments      */
/*						Now can be put into DLL                              */
/*			03/11/03	Trapped NULL progress function                       */
/*			04/29/03	Revised to become D. Howe's final version            */
/*			11/12/06	Adapted for DLL from Stable_r.c						 */
/*                                                                           */
/* (c) Copyright 2003-6  Hamilton Technical Services  All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float Theo1(F_TYPE x[], F_TYPE *p_sig,
	F_TYPE tau, int m, BOOL *bAbort, BOOL bProgress,
	void (*ProgressFunction)(int nPercentDone, char *szMessage))
{
    double sum_i;                                  /* outer summing variable */
    double sum_j;                                  /* inner summing variable */
    double fm;                                        /* double version of m */
    double count;                          /* # points in outer summing loop */
    int i;                                   /* index for outer summing loop */
    int j;                             /* array index for inner summing loop */
    int d;                                      /* loop index for inner loop */
    int n;                               /* # phase data points for analysis */
	int m2;								                              /* m/2 */
	int g;							                       /* # gaps in data */
    int nTic;                            /* Increment for progress indicator */
    int nLast_i=0;                                /* Previous value of index */

	// Note that double is used for count because can get too large for an int
	
	// Set n to # analysis points
	n=(int)(x[2]-x[1]+1);
    
	// Check for NG arguments
	if( (m<1) || (n<m) || (tau<=0) || (m%2) )
    {
        return(-1);                                                 /* error */
    }

    // Initializations
    fm=(float)m;
    sum_i=0;
    count=0.0;
	m2=m/2;
    nTic=(int)ceil(n/100);   // Increment for progress report

	// Progress label
    if(bProgress && ProgressFunction)
    {
		// Label progress bar
		ProgressFunction(0, "Calculating Tho1"); 
    }

	// Check for gaps in data
    g=CountGaps(x, PHASE_DATA);

	// Code for data with gaps
	if(g)
	{
		// To simplify gap handling, if first or last point is zero
		// make it slightly larger
		// We assume that the original data is restored by the calling function
		if(x[3]==0)
		{
			x[3]=1e-99;
		}
		
		if(x[(int)x[0]+2]==0)
		{
			x[(int)x[0]+2]=1e-99;
		}

		// Outer Loop
		// i goes from 1 to n-m where n=# phase points & m=avg factor
		for(i=1; i<=n-m; i++)  
		{
			// Initialize inner sum
			sum_j=0;

			// Inner Loop
			// d goes from 0 to ((m/2)-1)
			// Index j includes offset for analysis start and array offset
			for(d=0; d<=m2-1; d++) 
			{
				// Set array index
				j=(int)(i+x[1]+1);

				// Check that all terms are non-zero (not gaps)
				if(x[j] && x[j-d+m2] && x[j+m] && x[j+d+m2])
				{
					// Inner summation
					sum_j+=(SQR(x[j]-x[j-d+m2]+x[j+m]-x[j+d+m2]))/(m2-d);

					// Increment count
					count++;
				}
			}

			// Outer summation
			sum_i+=sum_j;

			// Display progress
			// Update when index has changed by at least tic amount
			if(((i-nLast_i)>=nTic) && (bProgress))
			{
				if(ProgressFunction)
				{
					ProgressFunction((int)(0.5+100.0*i/(n-m)),
						"Calculating Tho1");
				}
				nLast_i=i;
			}

			// Check for abort
			if(*bAbort)
			{
				break;
			}
		}
	}
	else // No gaps
	{
		// Outer Loop
		// i goes from 1 to n-m where n=# phase points & m=avg factor
		for(i=1; i<=n-m; i++)  
		{
			// Initialize inner sum
			sum_j=0;

			// Inner Loop
			// d goes from 0 to ((m/2)-1)
			// Index j includes offset for analysis start and array offset
			for(d=0; d<=m2-1; d++) 
			{
				// Set array index
				j=(int)(i+x[1]+1);

				// Inner summation
				sum_j+=(SQR(x[j]-x[j-d+m2]+x[j+m]-x[j+d+m2]))/(m2-d);

				// Increment count
				count++;
			}

			// Outer summation
			sum_i+=sum_j;

			// Display progress
			// Update when index has changed by at least tic amount
			if(((i-nLast_i)>=nTic) && (bProgress))
			{
				if(ProgressFunction)
				{
					ProgressFunction((int)(0.5+100.0*i/(n-m)),
						"Calculating Theo1");
				}
				nLast_i=i;
			}

			// Check for abort
			if(*bAbort)
			{
				break;
			}
		}
    }

    if(bProgress && ProgressFunction)
    {
		// Clear progress display
		ProgressFunction(0, "");
	}

	// Final scaling
    if(sum_i>0)                                     /* make sure no div by 0 */
    {
        // Divide by (n-m) and (m*tau)^2 and 0.75 and take sqrt
		// Note:  tau is 0.75(m-1)tau0
        *p_sig=sqrt(sum_i/((n-m)*(tau*tau*fm*fm)*0.75));

 		return (float)count;                    /* # non-gap analysis points */
    }
    else
    {
        return(-1);                                                 /* error */
    }
}                                                             /* end Theo1() */
