/*****************************************************************************/
/*                                                                           */
/* 		                         Theo1Bias()                                 */
/*                                                                           */
/* 	Function to determine the bias correction factor for the Tho1 variance  */
/*                                                                           */
/*  Params:			int nAlpha  = Power law noise exponent                   */
/*					float fTau  = Effective normalized Tho1 averaging time  */
/*								= Tho1 tau for unity tau_0                  */
/*								= 0.75 * m, where m = averaging factor       */
/*								= 0.75 * tau/tau_0                           */
/*                                                                           */
/*	Return:         float fBias =  Tho1 bias factor                         */
/*								   or -1 if error                            */
/*                                                                           */
/* 	References:		(1) D.A. Howe e-mail of 05/01/03                         */
/*					(2) D.A. Howe e-mail of 05/16/03 with "Tho1 Summary"    */
/*					(3) D/A. Howe e-mail of 05/29/03 w/ W PM & F PM values   */
/*					(4) T.N. Tasset and D.A. Howe, "A Practical Tho1        */
/*						Algorithm".                                          */
/*                                                                           */
/*  Notes:			This bias factor is the ratio of AVAR to Tho1           */
/*					To correct Tho1 deviation, multiply it by the sqrt of   */
/*					this bias factor.  The fTau parameter is really m.       */
/*                                                                           */
/*  Revision record:                                                         */
/*      05/11/03    Created                                                  */
/*		05/16/03	Updated per latest D. Howe inputs                        */
/*		05/29/03	Ditto                                                    */
/*		12/09/03	Revised from Tho1Bias() per Eq (1) of Reference (4)     */
/*		11/12/06	Adapted for DLL from NewTheo1Bias() of Stable_r.c		 */
/*					Renamed Theo1Bias()                                      */
/*		12/31/06	NewTheo1Bias() of Stable_r.c renamed Theo1Bias()         */
/*					for compatibility with this DLL function                 */
/*                                                                           */
/* (c) Copyright 2003-6	 Hamilton Technical Services   All Rights Reserved   */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float Theo1Bias(int nAlpha, float fTau)
{
	// Local variables
	double fA, fB, fC;	// Coefficients of bias function
	float fBias;		// Tho1 bias factor

	// Calc Tho1 bias factor
	switch(nAlpha)
	{
		case W_PM:
		{
			// Assign coefficient values
			fA=0.09;
			fB=0.74;
			fC=0.40;
		}
		break;

		case F_PM:
		{
			// Assign coefficient values
			fA=0.14;
			fB=0.82;
			fC=0.30;
		}
		break;

		case W_FM:
		{
			// Assign coefficient values
			fA=1.00;
			fB=0.00;
			fC=0.00;
		}
		break;

		case F_FM:
		{
			// Assign coefficient values
			fA= 1.87;
			fB=-1.05;
			fC= 0.79;
		}
		break;

		case RW_FM:
		{
			// Assign coefficient values
			fA= 2.70;
			fB=-1.53;
			fC= 0.85;
		}
		break;

		default:
		{
			// Return -1 (error)
			return -1.0;
		}
	}

	// Calculate bias factor
	fBias=(float)(fA + (fB/pow(fTau, fC)));

	
	// Return Tho1 bias factor
	return fBias;
}
