/*****************************************************************************/
/*                                                                           */
/* 		                     Theo1BiasToAlpha()                              */
/*                                                                           */
/* 	Function to determine alpha (noise type) from the Tho1 bias factor      */
/*                                                                           */
/*  Params:			float fBias    = Tho1 variance bias factor              */
/*					int nAvgFactor = Averaging factor = m = tau/tau_0        */
/*                                                                           */
/*	Return:         int nAlpha     = Power law noise exponent                */
/*                                                                           */
/*	Ref:			T.N. Tasset & D.A. Howe, "A Practical Tho1 Algorithm"   */
/*                                                                           */
/*  Method:			Calc bias factor for each noise type for given avg       */
/*					factor, use those values to set decision thresholds,     */
/*					and sort bias factor accordingly to determine alpha.     */
/*                                                                           */
/*  Note:			The 2nd arg is the actual avg factor - a correction is   */
/*                  applied in the body of the function to convert it to     */
/*                  the effective normalized Tho1 tau for NewTheo1Bias()    */
/*                                                                           */
/*  Revision record:                                                         */
/*      12/09/03    Created from Theo1BiasToAlpha()                          */
/*		11/12/06	Adapted from NewTheo1BiasToAlpha() of Stable_r.c         */
/*					Renamed Theo1BiasToAlpha()                               */
/*					(Do not confuse with Theo1BiasToAlpha() of Stable_r.c    */
/*					which is obsolete)                                       */
/*					Local calls to NewTheo1Bias() changed to Theo1Bias()     */
/*                                                                           */
/* (c) Copyright 2003-6	 Hamilton Technical Services   All Rights Reserved   */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int Theo1BiasToAlpha(float fBias,
	int nAvgFactor)
{
	// Local variables
	int nAlpha;								// Alpha
	float fWPM, fFPM, fWFM, fFFM, fRWFM;	// Nominal bias factors

	// Calculate nominal bias factor for each noise type for given avg factor
	fWPM=Theo1Bias(2, (float)(0.75*nAvgFactor));
	fFPM=Theo1Bias(1, (float)(0.75*nAvgFactor));
	fWFM=Theo1Bias(0, (float)(0.75*nAvgFactor));
	fFFM=Theo1Bias(-1, (float)(0.75*nAvgFactor));
	fRWFM=Theo1Bias(-2, (float)(0.75*nAvgFactor));

	// Use Tho1 bias factor to determine noise type
	// Thresholds for noise sorting are geometric means
	// of nominal Tho1 bias values

	// Find noise type
	if(fBias<sqrt(fWPM*fFPM))
	{
		nAlpha=2;
	}
	else if(fBias<sqrt(fFPM*fWFM))
	{
		nAlpha=1;
	}
	else if(fBias<sqrt(fWFM*fFFM))
	{
		nAlpha=0;
	}
	else if(fBias<sqrt(fFFM*fRWFM))
	{
		nAlpha=-1;
	}
	else
	{
		nAlpha=-2;
	}

	return nAlpha;
}
