/*****************************************************************************/
/*                                                                           */
/* 		                        Theo1EDF()                                   */
/*                                                                           */
/* 	Function to determine the estimated # of chi-squared degrees of freedom  */
/*  for the Tho1 variance of a power law noise process.                     */
/*                                                                           */
/*  Params:			int nAlpha   = Power law noise exponent                  */
/*					int nNum     = # phase analysis points                   */
/*                  float fRatio = Tau/Tau0 ratio = m = averaging factor     */
/*                                                                           */
/*	Return:         float EDF    = estimated degrees of freedom              */
/*								   or -1 if error                            */
/*                                                                           */
/*	Notes:			(1) This function applies to -2 >= alpha >= 2            */
/*						(W PM, F PM, W FM, F FM and RW FM noise)             */
/*					(2) fRatio is entered as AF=m, and is scaled to          */
/*						fR=0.75m in this function for use in the             */
/*						Tho1 edf formuale                                   */
/*                                                                           */
/* 	References:		(1) D.A. Howe e-mail of 05/01/03                         */
/*					(2) D.A. Howe e-mail of 05/16/03 with "Tho1 Summary"    */
/*					(3) D.A. Howe e-mail of 05/18/03 clarifications          */
/*					(4) D.A. Howe e-mail of 05/29/03 with F PM edf formula   */
/*						and fR=0.75m instead of previous fR=0.75(m-1)        */
/*                                                                           */
/*  Revision record:                                                         */
/*      05/01/03    Created                                                  */
/*		05/16/03	Updated per latest D. Howe inputs                        */
/*		05/18/03	Ditto                                                    */
/*		05/29/03	Ditto again                                              */
/*		01/05/04	Fixed int O/F error for F FM case                        */
/*		11/12/06	Adapted for DLL from Stable_r.c						     */
/*                                                                           */
/* (c) Copyright 2003-6	 Hamilton Technical Services   All Rights Reserved   */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float Theo1EDF(int nAlpha, int nNum,
	float fRatio)
{
	// Local variables
	float fEDF;
	float fR;

	// Verify parameters
	if(nAlpha<-2 || nAlpha>2)
	{
		return -1.0; // Error
	}

	// Calc Tho1 ratio 0.75m
	fR=(float)(0.75*fRatio);

	switch(nAlpha)
	{
		case 2:		// W PM noise
		{
			fEDF=(float)(((0.86*(nNum+1)*(nNum-((4.0/3.0)*fR)))/
				(nNum-fR))*(fR/(fR+1.14)));
		}
		break;

		case 1:		// F PM noise
		{
			fEDF=(float)((((4.798*nNum*nNum)-(6.374*nNum*fR)+(12.387*fR))/
				(sqrt(fR+36.6)*(nNum-fR)))*(fR/(fR+0.3)));
		}
		break;

		case 0:		// W FM noise
		{
			fEDF=(float)((((4.1*nNum+0.8)/fR)-((3.1*nNum+6.5)/nNum))*
				(pow(fR,1.5)/(pow(fR,1.5)+5.2)));
		}
		break;

		case -1:	// F FM noise
		{
			fEDF=(float)(((2.0*nNum*nNum-1.3*nNum*fR-3.5*fR)/
				(nNum*fR))*((fR*fR*fR)/
				(fR*fR*fR+2.3)));
		}
		break;

		case -2:	// RW FM noise
		{
			fEDF=(float)(((4.4*nNum-2)/(2.9*fR))*(((4.4*nNum-1)*
				(4.4*nNum-1)-8.6*fR*(4.4*nNum-1)+
				11.4*fR*fR)/((4.4*nNum-3)*(4.4*nNum-3))));
		}
		break;
	}

	return fEDF;
}
