/*****************************************************************************/
/*                                                                           */
/*                             TotvarBias()                                  */
/*                                                                           */
/*      Function to calculate the TOTVAR bias function                       */
/*                                                                           */
/*      Parameters:     int   nAlpha = noise type (-2 to +2)                 */
/*                      float fRatio = T/                                   */
/*																			 */
/*                                                                           */
/*      Return:         float fBias  = TOTVAR bias factor                    */
/*                                     or -1 if error                        */
/*                                                                           */
/*      The TOTVAR bias factor is the ratio of the expected value of         */
/*		TOTVAR to AVAR.  To correct a calculated TOTDEV value for a          */
/*		certain noise type, record length & analysis tau, calc the T/       */
/*      ratio, call this function, and divide the uncorrected TOTDEV value   */
/*		by the square root of the bias factor returned by this function.     */
/*                                                                           */
/*		Notes:			(1) This function applies to -2 >= alpha >= 2        */
/*		 				(W PM, F PM, W FM, F FM, RW FM noise)                */
/*						(2) This function applies only for T/  >= 2.0       */
/*						(3) T is defined as 0 * # phase data points         */
/*						(4)	No bias correction is necessary for W PM,        */
/*                          F PM & W FM noise (alpha=2, 1 or 0)              */
/*                                                                           */
/* 		Reference:		D.A. Howe and C.A. Greenhall, "Total Variance: A     */
/* 						Progress Report on a New Frequency Stability         */
/* 						Characterization", Proc. 29th PTTI Meeting,          */
/* 						Dec. 2, 1997, (to be published).                     */
/*																			 */
/*                      TOTVAR Bias Factor =  1 - a * (tau / T)              */
/*                      where   a = 0 for W FM noise (no bias) (alpha=0)     */
/*								  = 1/(3 ln 2) for F FM noise  (alpha=-1)    */
/*                                = 3/4        for RW FM noise (alpha=-2)    */
/*                                                                           */
/*						Note that the fRatio entered is T/tau not tau/T!     */
/*                                                                           */
/*      Revision record:                                                     */
/*          12/06/97    Created & debugged                                   */
/*			12/09/97	Added to FrequenC Library                            */
/*			12/21/97	Fixed error reported by D. Howe/NIST:                */
/*						F FM and RW FM constants were reversed               */
/*			01/01/98	Changes for MS VC++ compatibility & warnings         */
/*                                                                           */
/* (c) Copyright 1997   Hamilton Technical Services   All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float TotvarBias(int nAlpha, float fRatio)
{
	// Local variables
	float fBias;

	// Verify parameters
	if(fRatio<2.0)
	{
		return -1.0; // Error
	}

	if(nAlpha<-2 || nAlpha>2)
	{
		return -1.0; // Error
	}

	// Sort by noise type
	switch(nAlpha)
	{
		case -2: // RW FM
		{
			fBias=(float)(1.0 - 0.7500/fRatio);
		}
		break;

		case -1: // F FM
		{
			fBias=(float)(1.0 - 0.4809/fRatio);
		}
		break;

		case -0: // W FM
		{
			fBias=1.0;
		}
		break;

		case 1: // F PM
		{
			fBias=1.0;
		}
		break;

		case 2: // W PM
		{
			fBias=1.0;
		}
		break;
	}

	return (fBias);
}

/*****************************************************************************/
