/*****************************************************************************/
/*                                                                           */
/*                                 TotvarCalc()                              */
/*                                                                           */
/*      Function to calculate TOTVAR using doubly reflected phase data       */
/*                                                                           */
/*      Parameters:     F_TYPE x[]    	=   phase data                       */
/*                                      	x[0] = # data points             */
/*                                      	x[1] = analysis start            */
/*                                      	x[2] = analysis end              */
/*						F_TYPE *TotDev= pointer to TOTDEV                    */
/*						F_TYPE fTau 	= interval between phase data points */
/*						int nAF			= averaging factor                   */
/*						void *lpfnProgress = ptr to progress display funct   */
/*                                                                           */
/*      Return:         int           = # TOTALVAR analysis points,          */
/*                                      or -1 if memory alloc error          */
/*										or -2 if no result error             */
/*                                                                           */
/*      Notes:          1. Calculation is done entirely with a new array.    */
/*						   that is deleted after this function closes.       */
/*						2. Progress indicator and abort not necessary.       */
/*						3. The phase data need NOT be endmatched before      */
/*						   calling this function.                            */
/*                                                                           */
/*		Reference:		D.A. Howe and C.A. Greenhall, "Total Variance: A     */
/*						Progress Report on a New Frequency Stability         */
/*						Characterization", Proc. 29th PTTI Meeting,          */
/*						Dec. 2, 1997, (to be published).                     */
/*                                                                           */
/*      Revision record:                                                     */
/*          12/06/97    Adapted from ReflectedTotalvarCalc()                 */
/*			12/09/97	Added to FrequenC Library                            */
/*			01/01/98	Changes for MS VC++ compatibility & warnings         */
/*          12/07/17	Revise for Linux                                     */
/*                      Replace GlobalAllocPtr() & GlobalAllocFree()         */
/*                      with malloc() and free()                             */
/*                                                                           */
/* (c) Copyright 1997-2017 Hamilton Technical Services  All Rights Reserved  */
/*                                                                           */
/*****************************************************************************/

// Headers
#include "frequenc.h"
// #include <windowsx.h>   // For GlobalAllocPtr() & GlobalAllocFree()

// Defines
#define ARRAY_OFFSET 3	// Offset to 1st data point

int TotvarCalc(F_TYPE fX[], F_TYPE *fTotDev,
	F_TYPE fTau, int nAF, void *lpfnProgress)
{
	// Local variables
	int i;              // Index
	int j;				// Auxilary index
	int nNum;           // # points
	int nStart;			// Analysis start
	// int nDummy;		// Dummy variable to avoid complier warning
	int nCount=0;		// # analysis points

	#if(0) // Not needed if no progress indicator
	int nTic;			// Increment for progress indicator
	int nLast_i=0;      // Previous value of index
	#endif

	double *fP;   	// TOTVAR phase array pointer
	F_TYPE fSum=0;		// TOTVAR summation
	F_TYPE fEdge;		// Phase data value at edge of reflection

	// Avoid compiler warning if no progress indicator
	lpfnProgress=lpfnProgress;

	// Initializations
	nStart=(int)fX[1];          // Analysis start point
	nNum=(int)(fX[2]-fX[1]+1);	// # analysis phase data points = N

	#if(0) // Not needed if no progress indicator
	nTic=(int)ceil(nNum/100);	// Increment for progress report

	if(lpfnProgress!=NULL)
	{
		lpfnProgress(0, "Calculating Total Sigma"); // Label progress bar
	}
	#endif

	#if(0) // For testing - check input params
	TRACE_EXP(fTau);
	TRACE_INT(nStart);
	TRACE_INT(nNum);
	#endif

	// Allocate a new "virtual" phase data array to size 3N-4, where N is
	// the # of phase data points to be analyzed.  This virtual array is the
	// result of extension by reflection about both endpoints.  N-2 reflected
	// points are added at both ends of the original phase data to be analyzed.
	// Do NOT need the array header (#, Start, End)
	// Note that fP is cast to a huge double ptr and that array size is long
    #if(0) // Original FrequenC code
	if((fP=(double HUGE *)GlobalAllocPtr(GHND,
		(long)(3*nNum-4)*sizeof(double)))==NULL)
	{
		#if(0) // Eliminated for FrequenC DLL usage
		AllocErrorMessage();
		#endif
		return(-1);  // Error - memory reallocation failed
	}
	#endif

    #if(1) // Linux generic replacement
    // Note: Don't need to zero memory
	if((fP=(double*)malloc((long)(3*nNum-4)*sizeof(double)))==NULL)
	{
		return(-1);  // Error - memory reallocation failed
	}
	#endif
	// Note that in the referenced paper the index of the virtual phase data
	// array goes from 3-N (a negative number) to 2N-2 (a positive number),
	// with the original data having indices from 1 to N.  Our indices start at
	// 0 and go to 3N-5.  The lower reflected data has indices from 0 to N-3.
	// The original data in the middle of the virtual array has indices from
	// N-2 to 2N-3. The upper reflected data has indices from 2N-2 to 3N-5.

	// Copy original phase data array fX[] to (headerless) working array fP[]
	// Analysis limits are handled by copying only the points between the
	// start and end.  i is the index into the virtual array fP[].  j is the
	// variable part of the index into the original phase data array fX[].
	j=0;
	for(i=nNum-2; i<2*nNum-2; i++)
	{
		fP[i]=fX[j+nStart+ARRAY_OFFSET-1];
		j++;
	}

	// Fill the lower reflected phase data
	// These values are twice the first phase data point minus the particular
	// data point value to be reflected
	j=0;
	fEdge=2*fP[nNum-2];

	for(i=0; i<nNum-2; i++)
	{
		fP[i]=fEdge-fP[2*nNum-4-j];
		j++;
	}

	// Fill the upper reflected phase data
	// These values are twice the last phase data point minus the particular
	// data point value to be reflected
	j=0;
	fEdge=2*fP[2*nNum-3];

	for(i=2*nNum-2; i<3*nNum-4; i++)
	{
		fP[i]=fEdge-fP[2*nNum-4-j];
		j++;
	}

	#if(0) // For testing - examine virtual array
	for(i=0; i<3*nNum-4; i++)
	{
		TRACE_EXP(fP[i]);
	}
	#endif

	// Calc TOTVAR - See Eq (3) of Reference
	for(i=nNum-1; i<2*nNum-3; i++)
	{
		#if(1) // To eliminate gap checking
		// Check for gap
		if( (fP[i-nAF] && fP[i] && fP[i+nAF]) ||
			(i==nNum+nAF-2                  ) ||
			(i==2*nNum-nAF-3                ) )
		#endif
		{
			// Sum 2nd differences
			fSum+=SQR(fP[i-nAF]-2*fP[i]+fP[i+nAF]);
			nCount++;
		}
		#if(0) // For testing - display gap index
		else
		{
			TRACE_INT(i);
		}
		#endif


		#if(0) // Not needed if no progress indicator or abort
		// Display progress
		// Note:  This code provides updates only if index has changed
		// by at least tic amount
		// Uses pointer to progress display function PFVi argument
		// Pass NULL for no progress display
		if(((i-nLast_i)>=nTic) && (lpfnProgress!=NULL))
		{
			lpfnProgress(0.5+100.0*i/nNum, "Calculating Total Sigma");
			nLast_i=i;
		}

		// Check for abort
		if(bAbort)
		{
			nCount=0;                                         /* force error */
			break;
		}
		#endif
	}


	#if(0) // For testing - display # points analyzed
	TRACE_INT(nCount);
	#endif

	// Scale result - See Eq (3) of Reference
	fSum/=(fTau*fTau*nAF*nAF*2*nCount);

	// Find TOTDEV
	if(nCount && fSum>0)
	{
		*fTotDev=sqrt(fSum);
	}
	else // No result error
	{
		*fTotDev=0.0;
		nCount=-2;
	}

	// Free memory
	// nDummy=GlobalFreePtr(fP);
	// nDummy=nDummy;
	free(fP);

	return(nCount);  // Return # phase analysis points
}

/*****************************************************************************/
