/*****************************************************************************/
/*                                                                           */
/* 		                       TotvarEDF()                                   */
/*                                                                           */
/* 	Function to determine the estimated # of chi-squared degrees of freedom  */
/*  for the total variance (TOTVAR) of a power law noise process.            */
/*                                                                           */
/*  Params:			int   nAlpha = alpha                                     */
/*                  float fRatio = T/                                       */
/*                                                                           */
/*	Return:         float edf = estimated degrees of freedom                 */
/*								or -1 if error                               */
/*                                                                           */
/*	Notes:			(1) This function applies only to -2 >= alpha >= 0       */
/*						(W FM, F FM and RW FM noise)                         */
/*					(2) This function applies only for T/  >= 2.0           */
/*					(3) T is defined as 0 * # phase data points             */
/*                                                                           */
/* 	Reference:		D.A. Howe and C.A. Greenhall, "Total Variance: A         */
/* 					Progress Report on a New Frequency Stability             */
/* 					Characterization", Proc. 29th PTTI Meeting,              */
/* 					Dec. 2, 1997, (to be published).                         */
/*                                                                           */
/*  Revision record:                                                         */
/*      12/06/97    Created                                                  */
/*		12/09/97	Added to FrequenC Library                                */
/*		01/01/98	Changes for MS VC++ compatibility & warnings             */
/*                                                                           */
/* (c) Copyright 1997-8	 Hamilton Technical Services   All Rights Reserved   */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float TotvarEDF(int nAlpha, float fRatio)
{
	// Local variables
	float fEDF;

	#if(0) // For testing
	TRACE_INT(nAlpha);
	TRACE_EXP(fRatio);
	#endif

	// Verify parameters
	if(fRatio<2.0)
	{
		return -1.0; // Error
	}

	if(nAlpha<-2 || nAlpha>0)
	{
		return -1.0; // Error
	}

	switch(nAlpha)
	{
		case 0:		// W FM noise
		{
			fEDF=(float)(1.500*fRatio);
		}
		break;

		case -1:	// F FM noise
		{
			fEDF=(float)(1.16832*fRatio - 0.222);
		}
		break;

		case -2:	// RW FM noise
		{
			fEDF=(float)(0.92715*fRatio -0.358);
		}
		break;
	}

	#if(0) // For testing
	TRACE_EXP(fEDF);
	#endif

	return fEDF;

}

/*****************************************************************************/
