/****************************************************************************/
/*                                                                          */
/*                                 TIErms()                                 */
/*                                                                          */
/*      Function to calc rms time interval error from phase data            */
/*                                                                          */
/*      Parameters:     F_TYPE fX[]   = phase data array                    */
/*                                      fX[0] = # data points               */
/*                                      fX[1] = analysis start              */
/*                                      fX[2] = analysis end                */
/*                      F_TYPE *fTIE  = pointer to TIE rms                  */
/*                      int nAF       = averaging factor                    */
/*                                                                          */
/*      Returns:        int           = # analysis points,                  */
/*                                      or -1 if error                      */
/*                                                                          */
/*      Note:           Embedded zero value treated as gap in phase data.   */
/*                      Must have at least 1 analysis point; this           */
/*                      requires at least 2 adjacent non-gap phase          */
/*                      data points.                                        */
/*                                                                          */
/*      Reference:      S. Bregni, "Clock Stability Characterization and    */
/*                      Measurement in Telecommunications", IEEE            */
/*                      Transactions on Instrumentation and Measurement,    */
/*                      Vol. 46, No. 6, Dec 1997, pp 1284-1294, Eq. 13.     */
/*                                                                          */
/*      Revision record:                                                    */
/*          06/03/98    Created                                             */
/*                                                                          */
/* (c) Copyright 1998  Hamilton Technical Services  All Rights Reserved     */
/*                                                                          */
/****************************************************************************/

#include "frequenc.h"

int TIErms(F_TYPE fX[], F_TYPE *fTIE,
	int nAF)
{
    F_TYPE fSum=0;                          /* summing variable to calc TIE */
    int i;                                             /* index for summing */
    int nNum=0;                                        /* # analysis points */

    for( i = (int)(fX[1]+2); i <= (int)(fX[2]+2-nAF); i++)
               /* For full analysis limits, index goes from 3 to nNum+2-nAF */
                   /* In summation, smallest index is 1st data point=3, and */
                                                 /* largest index is nNum+2 */
    {
        if(    (i==fX[1]+2 && fX[i+nAF])
            || (fX[i] && fX[i+nAF]) 
            || (i==fX[2]+2-nAF && fX[i]))
                           /* skip if any embedded phase data value is zero */
        {
            fSum += SQR(fX[i+nAF] - fX[i]);
            nNum++;
        }                                                        
    }
                                                               
    if(nNum>0)
    {
        *fTIE=sqrt(fSum/nNum);

        return(nNum);                                  /* # analysis points */
    }
    else
    {
        *fTIE=0;
        return(-1);                                                /* error */
    }
}                                                          

/*****************************************************************************/

